/*
 * $Id: switch.h,v 1.12 Broadcom SDK $
 * $Copyright: (c) 2018 Broadcom.
 * Broadcom Proprietary and Confidential. All rights reserved.$
 * 
 * Common internal definitions for BCM switch module
 */

#ifndef _BCM_INT_SWITCH_H_
#define _BCM_INT_SWITCH_H_

#ifdef CRASH_RECOVERY_SUPPORT
#include <soc/dcmn/dcmn_crash_recovery.h>
#include <soc/dcmn/dcmn_crash_recovery_test.h>
#include <soc/dcmn/dcmn_crash_recovery_utils.h>
#else
#if defined(BCM_DNX_SUPPORT)
#include <soc/dnx/dnx_err_recovery_manager.h>
#endif
#if defined(BCM_DNX_SUPPORT) || defined(BCM_DNXF_SUPPORT)
#include <soc/dnxc/dnxc_wb_test_utils.h>
#endif
#endif

#ifdef BCM_WARM_BOOT_SUPPORT
#include <bcm_int/control.h>


#include <soc/defs.h>
extern char warmboot_api_function_name[SOC_MAX_NUM_DEVICES][100];
#if defined(BCM_WARM_BOOT_API_TEST) && (defined(BCM_DNX_SUPPORT) || defined(BCM_DNXF_SUPPORT))
extern char dnxc_warmboot_api_function_name[SOC_MAX_NUM_DEVICES][DNXC_WARMBOOT_API_FUNCTION_NAME_MAX_LEN];
#endif

extern int _bcm_switch_state_sync(int unit, bcm_dtype_t dtype);

#if (defined(BCM_PETRA_SUPPORT) || defined(BCM_DFE_SUPPORT)) && defined(BCM_WARM_BOOT_API_TEST)

#include <soc/dcmn/dcmn_wb.h>
#endif /* BCM_PETRA_SUPPORT && BCM_WARM_BOOT_API_TEST */

#define BCM_STATE_SYNC(_u) \
    /* next line actually call sync only in autosync mode */ \
    _bcm_switch_state_sync(_u, dtype);

#else /* !BCM_WARM_BOOT_SUPPORT */
#define BCM_STATE_SYNC(_u)
#endif /* BCM_WARM_BOOT_SUPPORT */


/************** WARM BOOT API TEST ********************/

#if defined(BCM_WARM_BOOT_API_TEST)
#define DNXC_WARM_BOOT_API_TEST_RESET(unit) \
    do { \
        dnxc_warm_boot_api_test_reset(unit); \
    } while(0)

#define DNXC_WARM_BOOT_API_TEST_FUNCTION_NAME_UPDATE(unit) \
    sal_strncpy(dnxc_warmboot_api_function_name[unit], __FUNCTION__, DNXC_WARMBOOT_API_FUNCTION_NAME_MAX_LEN - 1)

#define DNXC_WB_API_TEST_COUNTER_INCREASE(unit) \
    do { \
        dnxc_wb_api_test_counter_increase(unit); \
    } while(0)
#define DNXC_WB_API_TEST_COUNTER_DECREASE(unit) \
    do { \
        dnxc_wb_api_test_counter_decrease(unit); \
    } while(0)
#else
#define DNXC_WARM_BOOT_API_TEST_RESET(unit)
#define DNXC_WARM_BOOT_API_TEST_FUNCTION_NAME_UPDATE(unit)
#define DNXC_WB_API_TEST_COUNTER_INCREASE(unit)
#define DNXC_WB_API_TEST_COUNTER_DECREASE(unit)
#endif

/******************************************************/



/****************** CRASH RECOVERY **************************/

#ifdef CRASH_RECOVERY_SUPPORT
#define BCM_CR_TRANSACTION_START(unit)\
    SOC_CR_DOCUMENT_API_NAME(unit);\
    BCM_CR_TEST_TRANSACTION_START(unit);\
    SOC_CR_DISP_ERR_CHECK(soc_dcmn_cr_dispatcher_transaction_start(unit))
#define BCM_CR_TRANSACTION_END(unit)\
    SOC_CR_DOCUMENT_API_NAME(unit);\
    SOC_CR_DISP_ERR_CHECK(soc_dcmn_cr_dispatcher_transaction_end(unit));\
    BCM_CR_TEST_TRANSACTION_END(unit)
#elif !defined(BCM_DNX_SUPPORT) && !defined(BCM_DNXF_SUPPORT)
#define BCM_CR_TRANSACTION_START(unit)
#define BCM_CR_TRANSACTION_END(unit)
#endif

/*************************************************************/



/******************* ERROR RECOVERY ***************************/

#if defined(BCM_DNX_SUPPORT)
#define DNXC_ERR_RECOVERY_TRANSACTION_START(unit) DNX_ERR_RECOVERY_TRANSACTION_START(unit)
#define DNXC_ERR_RECOVERY_TRANSACTION_END(unit, r_rv) DNX_ERR_RECOVERY_TRANSACTION_END(unit, r_rv)
#else
#define DNXC_ERR_RECOVERY_TRANSACTION_START(unit)
#define DNXC_ERR_RECOVERY_TRANSACTION_END(unit, r_rv)
#endif

/**************************************************************/


/* API TRANSACTION START and API TRANSACTION END */

#if (defined(BCM_DNX_SUPPORT) || defined(BCM_DNXF_SUPPORT)) && ! defined (CRASH_RECOVERY_SUPPORT)
#define BCM_CR_TRANSACTION_START(unit) \
    do { \
        DNXC_WARM_BOOT_API_TEST_FUNCTION_NAME_UPDATE(unit); \
        DNXC_WB_API_TEST_COUNTER_INCREASE(unit); \
        DNXC_ERR_RECOVERY_TRANSACTION_START(unit); \
    } while(0)

#define BCM_CR_TRANSACTION_END(unit) \
    do { \
        DNXC_ERR_RECOVERY_TRANSACTION_END(unit, r_rv); \
        DNXC_WARM_BOOT_API_TEST_RESET(unit); \
        DNXC_WB_API_TEST_COUNTER_DECREASE(unit); \
    } while(0)
#endif

#endif /* _BCM_INT_SWITCH_H_ */
