/*
 * $Id: $
 * 
 * $Copyright: (c) 2018 Broadcom.
 * Broadcom Proprietary and Confidential. All rights reserved.$
 *
 * THIS SOFTWARE MAY ONLY BE USED SUBJECT TO AN EXECUTED SOFTWARE LICENSE
 * AGREEMENT  BETWEEN THE USER AND BROADCOM.  YOU HAVE NO RIGHT TO USE OR
 * EXPLOIT THIS MATERIAL EXCEPT SUBJECT TO THE TERMS OF SUCH AN AGREEMENT.
 *
 * $brcm_Workfile: $
 * $brcm_Revision: $
 * $brcm_Date: $
 *
 * Module Description:
 *                     DO NOT EDIT THIS FILE DIRECTLY
 *
 *
 * Revision History:
 *
 * $brcm_Log: $
 *
 ***************************************************************************/

#ifndef DDR34_H__
#define DDR34_H__

/**
 * m = memory, c = core, r = register, f = field, d = data.
 */
#if !defined(GET_FIELD) && !defined(SET_FIELD)
#define BRCM_ALIGN(c,r,f)   DDR34_PHY_FIELDS_TABLE(c##_##r##r_##f, 1)
#define BRCM_BITS(c,r,f)    DDR34_PHY_FIELDS_TABLE(c##_##r##r_##f, 2)
#define BRCM_MASK(c,r,f)    DDR34_PHY_FIELDS_TABLE(c##_##r##r_##f, 0)
#define BRCM_SHIFT(c,r,f)   DDR34_PHY_FIELDS_TABLE(c##_##r##r_##f, 3)

#define GET_FIELD(m,c,r,f) \
	((((m) & BRCM_MASK(c,r,f)) >> BRCM_SHIFT(c,r,f)) << BRCM_ALIGN(c,r,f))

#define SET_FIELD(m,c,r,f,d) \
	((m) = (((m) & ~BRCM_MASK(c,r,f)) | ((((d) >> BRCM_ALIGN(c,r,f)) << \
	 BRCM_SHIFT(c,r,f)) & BRCM_MASK(c,r,f))) \
	)
#endif /* GET & SET */


#define DDR34_PHY_REGS_TABLE(_regs) _regs_tbl[_regs]
#define DDR34_PHY_FIELDS_TABLE(_fields, _t) _fields_tbl[(_fields << 2) + _t]

#include "phy_and28_e0.h"
#include "phy_and28_f0.h"

/*******************************************************************************
 *
 ******************************************************************************/
typedef enum ddr34_phy_regs_e {
    DDR34_PHY_CONTROL_REGS_REVISIONr,
    DDR34_PHY_CONTROL_REGS_PLL_STATUSr,
    DDR34_PHY_CONTROL_REGS_PLL_CONFIGr,
    DDR34_PHY_CONTROL_REGS_PLL_CONTROL1r,
    DDR34_PHY_CONTROL_REGS_PLL_CONTROL2r,
    DDR34_PHY_CONTROL_REGS_PLL_CONTROL3r,
    DDR34_PHY_CONTROL_REGS_PLL_DIVIDERSr,
    DDR34_PHY_CONTROL_REGS_PLL_FRAC_DIVIDERr,
    DDR34_PHY_CONTROL_REGS_PLL_SS_CONTROLr,
    DDR34_PHY_CONTROL_REGS_PLL_SS_LIMITr,
    DDR34_PHY_CONTROL_REGS_AUX_CONTROLr,
    DDR34_PHY_CONTROL_REGS_IDLE_PAD_CONTROLr,
    DDR34_PHY_CONTROL_REGS_IDLE_PAD_ENABLE0r,
    DDR34_PHY_CONTROL_REGS_IDLE_PAD_ENABLE1r,
    DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTLr,
    DDR34_PHY_CONTROL_REGS_STATIC_PAD_CTLr,
    DDR34_PHY_CONTROL_REGS_DRAM_CONFIGr,
    DDR34_PHY_CONTROL_REGS_DRAM_TIMING1r,
    DDR34_PHY_CONTROL_REGS_DRAM_TIMING2r,
    DDR34_PHY_CONTROL_REGS_DRAM_TIMING3r,
    DDR34_PHY_CONTROL_REGS_DRAM_TIMING4r,
    DDR34_PHY_CONTROL_REGS_VDL_CALIBRATEr,
    DDR34_PHY_CONTROL_REGS_VDL_CALIB_STATUS1r,
    DDR34_PHY_CONTROL_REGS_VDL_CALIB_STATUS2r,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_CONTROLr,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_REFr,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_STATUSr,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OVERRIDEr,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_CONTROLr,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUSr,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUS_CLEARr,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD00r,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD01r,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD02r,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD03r,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD04r,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD05r,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD06r,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD07r,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD08r,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD09r,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD10r,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD11r,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD12r,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD13r,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD14r,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD15r,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA0r,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA1r,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA2r,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX0r,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX1r,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX2r,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CS0r,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CS1r,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_PARr,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_RAS_Nr,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CAS_Nr,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CKEr,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_RST_Nr,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_ODTr,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_WE_Nr,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_Pr,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_Nr,
    DDR34_PHY_CONTROL_REGS_VDL_CLK_CONTROLr,
    DDR34_PHY_CONTROL_REGS_VDL_LDE_CONTROLr,
    DDR34_PHY_CONTROL_REGS_REFRESHr,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDLr,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP1r,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP2r,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG1r,
    DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG1r,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG2r,
    DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG2r,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG3r,
    DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG3r,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG4r,
    DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG4r,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG_TIMERr,
    DDR34_PHY_CONTROL_REGS_MODE_REG0r,
    DDR34_PHY_CONTROL_REGS_MODE_REG1r,
    DDR34_PHY_CONTROL_REGS_MODE_REG2r,
    DDR34_PHY_CONTROL_REGS_MODE_REG3r,
    DDR34_PHY_CONTROL_REGS_MODE_REG4r,
    DDR34_PHY_CONTROL_REGS_MODE_REG5r,
    DDR34_PHY_CONTROL_REGS_MODE_REG6r,
    DDR34_PHY_CONTROL_REGS_MODE_REG7r,
    DDR34_PHY_CONTROL_REGS_MODE_REG8r,
    DDR34_PHY_CONTROL_REGS_MODE_REG15r,
    DDR34_PHY_CONTROL_REGS_MODE_REG63r,
    DDR34_PHY_CONTROL_REGS_ALERT_CLEARr,
    DDR34_PHY_CONTROL_REGS_ALERT_STATUSr,
    DDR34_PHY_CONTROL_REGS_CA_PARITYr,
    DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_CONTROLr,
    DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_STATUS0r,
    DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_STATUS1r,
    DDR34_PHY_CONTROL_REGS_WRITE_LEVELING_CONTROLr,
    DDR34_PHY_CONTROL_REGS_WRITE_LEVELING_STATUSr,
    DDR34_PHY_CONTROL_REGS_READ_ENABLE_CONTROLr,
    DDR34_PHY_CONTROL_REGS_READ_ENABLE_STATUSr,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_LFSR_SEEDr,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ADDRESS1r,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ADDRESS2r,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROLr,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_DATA_CONTROLr,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_DQ_MASKr,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ECC_DQ_MASKr,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_STATUSr,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_DQ_STATUSr,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ECC_STATUSr,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ERR_CNT_CONTROLr,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ERR_CNT_STATUSr,
    DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROLr,
    DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_STATUSr,
    DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_CONNECTIONSr,
    DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_OVERRIDEr,
    DDR34_PHY_CONTROL_REGS_VREF_DAC_CONTROLr,
    DDR34_PHY_CONTROL_REGS_PHYBIST_CNTRLr,
    DDR34_PHY_CONTROL_REGS_PHYBIST_SEEDr,
    DDR34_PHY_CONTROL_REGS_PHYBIST_STATUSr,
    DDR34_PHY_CONTROL_REGS_PHYBIST_CTL_STATUSr,
    DDR34_PHY_CONTROL_REGS_PHYBIST_BL0_STATUSr,
    DDR34_PHY_CONTROL_REGS_PHYBIST_BL1_STATUSr,
    DDR34_PHY_CONTROL_REGS_PHYBIST_BL2_STATUSr,
    DDR34_PHY_CONTROL_REGS_PHYBIST_BL3_STATUSr,
    DDR34_PHY_CONTROL_REGS_PHYBIST_BL4_STATUSr,
    DDR34_PHY_CONTROL_REGS_STANDBY_CONTROLr,
    DDR34_PHY_CONTROL_REGS_DEBUG_FREEZE_ENABLEr,
    DDR34_PHY_CONTROL_REGS_DEBUG_MUX_CONTROLr,
    DDR34_PHY_CONTROL_REGS_DFI_CNTRLr,
    DDR34_PHY_CONTROL_REGS_WRITE_ODT_CNTRLr,
    DDR34_PHY_CONTROL_REGS_ABI_PAR_CNTRLr,
    DDR34_PHY_CONTROL_REGS_ZQ_CALr,
    DDR34_PHY_CONTROL_REGS_AC_SPARE_REGr,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_Pr,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_Nr,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ0r,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ1r,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ2r,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ3r,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ4r,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ5r,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ6r,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ7r,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DMr,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_EDCr,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSPr,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSNr,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0Pr,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0Nr,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1Pr,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1Nr,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2Pr,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2Nr,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3Pr,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3Nr,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4Pr,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4Nr,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5Pr,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5Nr,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6Pr,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6Nr,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7Pr,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7Nr,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMPr,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMNr,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCPr,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCNr,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS0r,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS1r,
    DDR34_PHY_BYTE_LANE_0_VDL_CLK_CONTROLr,
    DDR34_PHY_BYTE_LANE_0_VDL_LDE_CONTROLr,
    DDR34_PHY_BYTE_LANE_0_RD_EN_DLY_CYCr,
    DDR34_PHY_BYTE_LANE_0_WR_CHAN_DLY_CYCr,
    DDR34_PHY_BYTE_LANE_0_READ_CONTROLr,
    DDR34_PHY_BYTE_LANE_0_READ_FIFO_ADDRr,
    DDR34_PHY_BYTE_LANE_0_READ_FIFO_DATAr,
    DDR34_PHY_BYTE_LANE_0_READ_FIFO_DM_DBIr,
    DDR34_PHY_BYTE_LANE_0_READ_FIFO_STATUSr,
    DDR34_PHY_BYTE_LANE_0_READ_FIFO_CLEARr,
    DDR34_PHY_BYTE_LANE_0_IDLE_PAD_CONTROLr,
    DDR34_PHY_BYTE_LANE_0_DRIVE_PAD_CTLr,
    DDR34_PHY_BYTE_LANE_0_RD_EN_DRIVE_PAD_CTLr,
    DDR34_PHY_BYTE_LANE_0_STATIC_PAD_CTLr,
    DDR34_PHY_BYTE_LANE_0_WR_PREAMBLE_MODEr,
    DDR34_PHY_BYTE_LANE_0_ODT_CONTROLr,
    DDR34_PHY_BYTE_LANE_0_EDC_DPD_CONTROLr,
    DDR34_PHY_BYTE_LANE_0_EDC_DPD_STATUSr,
    DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_CONTROLr,
    DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_STATUSr,
    DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_STATUS_CLEARr,
    DDR34_PHY_BYTE_LANE_0_EDC_CRC_CONTROLr,
    DDR34_PHY_BYTE_LANE_0_EDC_CRC_STATUSr,
    DDR34_PHY_BYTE_LANE_0_EDC_CRC_COUNTr,
    DDR34_PHY_BYTE_LANE_0_EDC_CRC_STATUS_CLEARr,
    DDR34_PHY_BYTE_LANE_0_BL_SPARE_REGr,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_Pr,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_Nr,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ0r,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ1r,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ2r,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ3r,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ4r,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ5r,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ6r,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ7r,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DMr,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_EDCr,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSPr,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSNr,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0Pr,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0Nr,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1Pr,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1Nr,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2Pr,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2Nr,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3Pr,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3Nr,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4Pr,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4Nr,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5Pr,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5Nr,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6Pr,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6Nr,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7Pr,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7Nr,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMPr,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMNr,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCPr,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCNr,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS0r,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS1r,
    DDR34_PHY_BYTE_LANE_1_VDL_CLK_CONTROLr,
    DDR34_PHY_BYTE_LANE_1_VDL_LDE_CONTROLr,
    DDR34_PHY_BYTE_LANE_1_RD_EN_DLY_CYCr,
    DDR34_PHY_BYTE_LANE_1_WR_CHAN_DLY_CYCr,
    DDR34_PHY_BYTE_LANE_1_READ_CONTROLr,
    DDR34_PHY_BYTE_LANE_1_READ_FIFO_ADDRr,
    DDR34_PHY_BYTE_LANE_1_READ_FIFO_DATAr,
    DDR34_PHY_BYTE_LANE_1_READ_FIFO_DM_DBIr,
    DDR34_PHY_BYTE_LANE_1_READ_FIFO_STATUSr,
    DDR34_PHY_BYTE_LANE_1_READ_FIFO_CLEARr,
    DDR34_PHY_BYTE_LANE_1_IDLE_PAD_CONTROLr,
    DDR34_PHY_BYTE_LANE_1_DRIVE_PAD_CTLr,
    DDR34_PHY_BYTE_LANE_1_RD_EN_DRIVE_PAD_CTLr,
    DDR34_PHY_BYTE_LANE_1_STATIC_PAD_CTLr,
    DDR34_PHY_BYTE_LANE_1_WR_PREAMBLE_MODEr,
    DDR34_PHY_BYTE_LANE_1_ODT_CONTROLr,
    DDR34_PHY_BYTE_LANE_1_EDC_DPD_CONTROLr,
    DDR34_PHY_BYTE_LANE_1_EDC_DPD_STATUSr,
    DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_CONTROLr,
    DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_STATUSr,
    DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_STATUS_CLEARr,
    DDR34_PHY_BYTE_LANE_1_EDC_CRC_CONTROLr,
    DDR34_PHY_BYTE_LANE_1_EDC_CRC_STATUSr,
    DDR34_PHY_BYTE_LANE_1_EDC_CRC_COUNTr,
    DDR34_PHY_BYTE_LANE_1_EDC_CRC_STATUS_CLEARr,
    DDR34_PHY_BYTE_LANE_1_BL_SPARE_REGr,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_Pr,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_Nr,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ0r,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ1r,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ2r,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ3r,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ4r,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ5r,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ6r,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ7r,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DMr,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_EDCr,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSPr,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSNr,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0Pr,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0Nr,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1Pr,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1Nr,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2Pr,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2Nr,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3Pr,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3Nr,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4Pr,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4Nr,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5Pr,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5Nr,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6Pr,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6Nr,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7Pr,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7Nr,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMPr,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMNr,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCPr,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCNr,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS0r,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS1r,
    DDR34_PHY_BYTE_LANE_2_VDL_CLK_CONTROLr,
    DDR34_PHY_BYTE_LANE_2_VDL_LDE_CONTROLr,
    DDR34_PHY_BYTE_LANE_2_RD_EN_DLY_CYCr,
    DDR34_PHY_BYTE_LANE_2_WR_CHAN_DLY_CYCr,
    DDR34_PHY_BYTE_LANE_2_READ_CONTROLr,
    DDR34_PHY_BYTE_LANE_2_READ_FIFO_ADDRr,
    DDR34_PHY_BYTE_LANE_2_READ_FIFO_DATAr,
    DDR34_PHY_BYTE_LANE_2_READ_FIFO_DM_DBIr,
    DDR34_PHY_BYTE_LANE_2_READ_FIFO_STATUSr,
    DDR34_PHY_BYTE_LANE_2_READ_FIFO_CLEARr,
    DDR34_PHY_BYTE_LANE_2_IDLE_PAD_CONTROLr,
    DDR34_PHY_BYTE_LANE_2_DRIVE_PAD_CTLr,
    DDR34_PHY_BYTE_LANE_2_RD_EN_DRIVE_PAD_CTLr,
    DDR34_PHY_BYTE_LANE_2_STATIC_PAD_CTLr,
    DDR34_PHY_BYTE_LANE_2_WR_PREAMBLE_MODEr,
    DDR34_PHY_BYTE_LANE_2_ODT_CONTROLr,
    DDR34_PHY_BYTE_LANE_2_EDC_DPD_CONTROLr,
    DDR34_PHY_BYTE_LANE_2_EDC_DPD_STATUSr,
    DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_CONTROLr,
    DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_STATUSr,
    DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_STATUS_CLEARr,
    DDR34_PHY_BYTE_LANE_2_EDC_CRC_CONTROLr,
    DDR34_PHY_BYTE_LANE_2_EDC_CRC_STATUSr,
    DDR34_PHY_BYTE_LANE_2_EDC_CRC_COUNTr,
    DDR34_PHY_BYTE_LANE_2_EDC_CRC_STATUS_CLEARr,
    DDR34_PHY_BYTE_LANE_2_BL_SPARE_REGr,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_Pr,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_Nr,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ0r,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ1r,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ2r,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ3r,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ4r,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ5r,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ6r,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ7r,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DMr,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_EDCr,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSPr,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSNr,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0Pr,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0Nr,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1Pr,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1Nr,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2Pr,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2Nr,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3Pr,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3Nr,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4Pr,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4Nr,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5Pr,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5Nr,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6Pr,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6Nr,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7Pr,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7Nr,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMPr,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMNr,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCPr,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCNr,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS0r,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS1r,
    DDR34_PHY_BYTE_LANE_3_VDL_CLK_CONTROLr,
    DDR34_PHY_BYTE_LANE_3_VDL_LDE_CONTROLr,
    DDR34_PHY_BYTE_LANE_3_RD_EN_DLY_CYCr,
    DDR34_PHY_BYTE_LANE_3_WR_CHAN_DLY_CYCr,
    DDR34_PHY_BYTE_LANE_3_READ_CONTROLr,
    DDR34_PHY_BYTE_LANE_3_READ_FIFO_ADDRr,
    DDR34_PHY_BYTE_LANE_3_READ_FIFO_DATAr,
    DDR34_PHY_BYTE_LANE_3_READ_FIFO_DM_DBIr,
    DDR34_PHY_BYTE_LANE_3_READ_FIFO_STATUSr,
    DDR34_PHY_BYTE_LANE_3_READ_FIFO_CLEARr,
    DDR34_PHY_BYTE_LANE_3_IDLE_PAD_CONTROLr,
    DDR34_PHY_BYTE_LANE_3_DRIVE_PAD_CTLr,
    DDR34_PHY_BYTE_LANE_3_RD_EN_DRIVE_PAD_CTLr,
    DDR34_PHY_BYTE_LANE_3_STATIC_PAD_CTLr,
    DDR34_PHY_BYTE_LANE_3_WR_PREAMBLE_MODEr,
    DDR34_PHY_BYTE_LANE_3_ODT_CONTROLr,
    DDR34_PHY_BYTE_LANE_3_EDC_DPD_CONTROLr,
    DDR34_PHY_BYTE_LANE_3_EDC_DPD_STATUSr,
    DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_CONTROLr,
    DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_STATUSr,
    DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_STATUS_CLEARr,
    DDR34_PHY_BYTE_LANE_3_EDC_CRC_CONTROLr,
    DDR34_PHY_BYTE_LANE_3_EDC_CRC_STATUSr,
    DDR34_PHY_BYTE_LANE_3_EDC_CRC_COUNTr,
    DDR34_PHY_BYTE_LANE_3_EDC_CRC_STATUS_CLEARr,
    DDR34_PHY_BYTE_LANE_3_BL_SPARE_REGr,
    DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTL_2Tr,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CKr,
    DDR34_PHY_CONTROL_REGS_CMD_CYC_DLYr,
    DDR34_PHY_CONTROL_REGS_ALERT_DFIr,
    DDR34_PHY_CONTROL_REGS_PHYBIST_CA_MASKr,
    DDR34_PHY_CONTROL_REGS_PHYBIST_DQ_MASK1r,
    DDR34_PHY_CONTROL_REGS_PHYBIST_DQ_MASK2r,
    DDR34_PHY_CONTROL_REGS_ZQ_CAL2r,
    DDR34_PHY_CONTROL_REGS_RO_PROC_MON_CTLr,
    DDR34_PHY_CONTROL_REGS_RO_PROC_MON_STATUSr,
    DDR34_PHY_CONTROL_REGS_CK_DRIVE_PAD_CTLr,
    DDR34_PHY_CONTROL_REGS_BYTE_LANE_ALIGN_MASKSr,
    DDR34_PHY_CONTROL_REGS_CLOCK_IDLEr,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQSr,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_ALERTr,
    DDR34_PHY_BYTE_LANE_0_DQSP_DRIVE_PAD_CTLr,
    DDR34_PHY_BYTE_LANE_0_DQSN_DRIVE_PAD_CTLr,
    DDR34_PHY_BYTE_LANE_0_ALERT_DRIVE_PAD_CTLr,
    DDR34_PHY_BYTE_LANE_0_CLOCK_ENABLEr,
    DDR34_PHY_BYTE_LANE_0_CLOCK_IDLEr,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQSr,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_ALERTr,
    DDR34_PHY_BYTE_LANE_1_DQSP_DRIVE_PAD_CTLr,
    DDR34_PHY_BYTE_LANE_1_DQSN_DRIVE_PAD_CTLr,
    DDR34_PHY_BYTE_LANE_1_ALERT_DRIVE_PAD_CTLr,
    DDR34_PHY_BYTE_LANE_1_CLOCK_ENABLEr,
    DDR34_PHY_BYTE_LANE_1_CLOCK_IDLEr,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQSr,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_ALERTr,
    DDR34_PHY_BYTE_LANE_2_DQSP_DRIVE_PAD_CTLr,
    DDR34_PHY_BYTE_LANE_2_DQSN_DRIVE_PAD_CTLr,
    DDR34_PHY_BYTE_LANE_2_ALERT_DRIVE_PAD_CTLr,
    DDR34_PHY_BYTE_LANE_2_CLOCK_ENABLEr,
    DDR34_PHY_BYTE_LANE_2_CLOCK_IDLEr,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQSr,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_ALERTr,
    DDR34_PHY_BYTE_LANE_3_DQSP_DRIVE_PAD_CTLr,
    DDR34_PHY_BYTE_LANE_3_DQSN_DRIVE_PAD_CTLr,
    DDR34_PHY_BYTE_LANE_3_ALERT_DRIVE_PAD_CTLr,
    DDR34_PHY_BYTE_LANE_3_CLOCK_ENABLEr,
    DDR34_PHY_BYTE_LANE_3_CLOCK_IDLEr,
    DDR34_PHY_MAX_REG_INDEX
} ddr34_phy_regs_t;

/*******************************************************************************
 *
 ******************************************************************************/
typedef enum ddr34_phy_reg_fields_e {
    DDR34_PHY_CONTROL_REGS_REVISIONr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_REVISIONr_PERFORMANCE,
    DDR34_PHY_CONTROL_REGS_REVISIONr_TECHNOLOGY,
    DDR34_PHY_CONTROL_REGS_REVISIONr_WB,
    DDR34_PHY_CONTROL_REGS_REVISIONr_BITS,
    DDR34_PHY_CONTROL_REGS_REVISIONr_MAJOR,
    DDR34_PHY_CONTROL_REGS_REVISIONr_MINOR,
    DDR34_PHY_CONTROL_REGS_PLL_STATUSr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_PLL_STATUSr_LOCK_LOST,
    DDR34_PHY_CONTROL_REGS_PLL_STATUSr_CLOCKING_8X,
    DDR34_PHY_CONTROL_REGS_PLL_STATUSr_CLOCKING_4X,
    DDR34_PHY_CONTROL_REGS_PLL_STATUSr_CLOCKING_2X,
    DDR34_PHY_CONTROL_REGS_PLL_STATUSr_STATUS,
    DDR34_PHY_CONTROL_REGS_PLL_STATUSr_LOCK,
    DDR34_PHY_CONTROL_REGS_PLL_CONFIGr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_PLL_CONFIGr_RESERVED_for_eco1,
    DDR34_PHY_CONTROL_REGS_PLL_CONFIGr_CK_LDO_REF_CTRL,
    DDR34_PHY_CONTROL_REGS_PLL_CONFIGr_CK_LDO_BIAS,
    DDR34_PHY_CONTROL_REGS_PLL_CONFIGr_PLL_LDO_REF_SEL,
    DDR34_PHY_CONTROL_REGS_PLL_CONFIGr_PLL_LDO_REF_CTRL,
    DDR34_PHY_CONTROL_REGS_PLL_CONFIGr_PLL_LDO_BIAS,
    DDR34_PHY_CONTROL_REGS_PLL_CONFIGr_HOLD,
    DDR34_PHY_CONTROL_REGS_PLL_CONFIGr_ENABLE,
    DDR34_PHY_CONTROL_REGS_PLL_CONFIGr_RESERVED2,
    DDR34_PHY_CONTROL_REGS_PLL_CONFIGr_FB_OFFSET,
    DDR34_PHY_CONTROL_REGS_PLL_CONFIGr_RESERVED3,
    DDR34_PHY_CONTROL_REGS_PLL_CONFIGr_RESET_POST_DIV,
    DDR34_PHY_CONTROL_REGS_PLL_CONFIGr_RESERVED4,
    DDR34_PHY_CONTROL_REGS_PLL_CONFIGr_RESET,
    DDR34_PHY_CONTROL_REGS_PLL_CONFIGr_PWRDN,
    DDR34_PHY_CONTROL_REGS_PLL_CONTROL1r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_PLL_CONTROL1r_I_KP,
    DDR34_PHY_CONTROL_REGS_PLL_CONTROL1r_I_KI,
    DDR34_PHY_CONTROL_REGS_PLL_CONTROL1r_I_KA,
    DDR34_PHY_CONTROL_REGS_PLL_CONTROL2r_VCO_RANGE,
    DDR34_PHY_CONTROL_REGS_PLL_CONTROL2r_UNUSED2,
    DDR34_PHY_CONTROL_REGS_PLL_CONTROL2r_NDIV_RELOCK,
    DDR34_PHY_CONTROL_REGS_PLL_CONTROL2r_FAST_LOCK,
    DDR34_PHY_CONTROL_REGS_PLL_CONTROL2r_VCO_FB_DIV2,
    DDR34_PHY_CONTROL_REGS_PLL_CONTROL2r_POST_CTRL_RESETB,
    DDR34_PHY_CONTROL_REGS_PLL_CONTROL2r_PWM_RATE,
    DDR34_PHY_CONTROL_REGS_PLL_CONTROL2r_STAT_MODE,
    DDR34_PHY_CONTROL_REGS_PLL_CONTROL2r_UNUSED1,
    DDR34_PHY_CONTROL_REGS_PLL_CONTROL2r_STAT_UPDATE,
    DDR34_PHY_CONTROL_REGS_PLL_CONTROL2r_STAT_SELECT,
    DDR34_PHY_CONTROL_REGS_PLL_CONTROL2r_STAT_RESET,
    DDR34_PHY_CONTROL_REGS_PLL_CONTROL2r_DCO_CTRL_BYPASS_ENABLE,
    DDR34_PHY_CONTROL_REGS_PLL_CONTROL2r_DCO_CTRL_BYPASS,
    DDR34_PHY_CONTROL_REGS_PLL_CONTROL3r_PLL_CONTROL,
    DDR34_PHY_CONTROL_REGS_PLL_DIVIDERSr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_PLL_DIVIDERSr_MDIV,
    DDR34_PHY_CONTROL_REGS_PLL_DIVIDERSr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_PLL_DIVIDERSr_PDIV,
    DDR34_PHY_CONTROL_REGS_PLL_DIVIDERSr_RESERVED2,
    DDR34_PHY_CONTROL_REGS_PLL_DIVIDERSr_NDIV_INT,
    DDR34_PHY_CONTROL_REGS_PLL_FRAC_DIVIDERr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_PLL_FRAC_DIVIDERr_NDIV_FRAC,
    DDR34_PHY_CONTROL_REGS_PLL_SS_CONTROLr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_PLL_SS_CONTROLr_SSC_STEP,
    DDR34_PHY_CONTROL_REGS_PLL_SS_CONTROLr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_PLL_SS_CONTROLr_SSC_MODE,
    DDR34_PHY_CONTROL_REGS_PLL_SS_LIMITr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_PLL_SS_LIMITr_SSC_LIMIT,
    DDR34_PHY_CONTROL_REGS_PLL_SS_LIMITr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_AUX_CONTROLr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_AUX_CONTROLr_IS_ODT,
    DDR34_PHY_CONTROL_REGS_AUX_CONTROLr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_AUX_CONTROLr_IS_CS,
    DDR34_PHY_CONTROL_REGS_AUX_CONTROLr_RESERVED2,
    DDR34_PHY_CONTROL_REGS_AUX_CONTROLr_IS_AD,
    DDR34_PHY_CONTROL_REGS_IDLE_PAD_CONTROLr_IDLE,
    DDR34_PHY_CONTROL_REGS_IDLE_PAD_CONTROLr_DIB_MODE,
    DDR34_PHY_CONTROL_REGS_IDLE_PAD_CONTROLr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_IDLE_PAD_CONTROLr_RXENB,
    DDR34_PHY_CONTROL_REGS_IDLE_PAD_CONTROLr_IDDQ,
    DDR34_PHY_CONTROL_REGS_IDLE_PAD_CONTROLr_DOUT_N,
    DDR34_PHY_CONTROL_REGS_IDLE_PAD_CONTROLr_DOUT_P,
    DDR34_PHY_CONTROL_REGS_IDLE_PAD_ENABLE0r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_IDLE_PAD_ENABLE0r_IO_IDLE_ENABLE,
    DDR34_PHY_CONTROL_REGS_IDLE_PAD_ENABLE1r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_IDLE_PAD_ENABLE1r_IO_IDLE_ENABLE,
    DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTLr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTLr_ADDR_CTL_PD_IDLE_STRENGTH,
    DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTLr_ADDR_CTL_ND_IDLE_STRENGTH,
    DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTLr_ADDR_CTL_PD_TERM_STRENGTH,
    DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTLr_ADDR_CTL_ND_TERM_STRENGTH,
    DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTLr_ADDR_CTL_PD_STRENGTH,
    DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTLr_ADDR_CTL_ND_STRENGTH,
    DDR34_PHY_CONTROL_REGS_STATIC_PAD_CTLr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_STATIC_PAD_CTLr_AUTO_OEB,
    DDR34_PHY_CONTROL_REGS_STATIC_PAD_CTLr_IDDQ_GDDR5,
    DDR34_PHY_CONTROL_REGS_STATIC_PAD_CTLr_IDDQ_LPDDR,
    DDR34_PHY_CONTROL_REGS_STATIC_PAD_CTLr_IDDQ_CLK1,
    DDR34_PHY_CONTROL_REGS_STATIC_PAD_CTLr_IDDQ_CLK0,
    DDR34_PHY_CONTROL_REGS_STATIC_PAD_CTLr_IDDQ_ODT,
    DDR34_PHY_CONTROL_REGS_STATIC_PAD_CTLr_IDDQ_PAR,
    DDR34_PHY_CONTROL_REGS_STATIC_PAD_CTLr_IDDQ_BA,
    DDR34_PHY_CONTROL_REGS_STATIC_PAD_CTLr_IDDQ_AUX2,
    DDR34_PHY_CONTROL_REGS_STATIC_PAD_CTLr_IDDQ_AUX1,
    DDR34_PHY_CONTROL_REGS_STATIC_PAD_CTLr_IDDQ_AUX0,
    DDR34_PHY_CONTROL_REGS_STATIC_PAD_CTLr_IDDQ_CS1,
    DDR34_PHY_CONTROL_REGS_STATIC_PAD_CTLr_IDDQ_A15,
    DDR34_PHY_CONTROL_REGS_STATIC_PAD_CTLr_IDDQ_A14,
    DDR34_PHY_CONTROL_REGS_STATIC_PAD_CTLr_IDDQ_A13,
    DDR34_PHY_CONTROL_REGS_STATIC_PAD_CTLr_IDDQ_A12,
    DDR34_PHY_CONTROL_REGS_STATIC_PAD_CTLr_IDDQ_A11,
    DDR34_PHY_CONTROL_REGS_STATIC_PAD_CTLr_IDDQ_A10,
    DDR34_PHY_CONTROL_REGS_STATIC_PAD_CTLr_IDDQ_A09,
    DDR34_PHY_CONTROL_REGS_STATIC_PAD_CTLr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_STATIC_PAD_CTLr_RX_MODE,
    DDR34_PHY_CONTROL_REGS_DRAM_CONFIGr_INIT_MODE,
    DDR34_PHY_CONTROL_REGS_DRAM_CONFIGr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_DRAM_CONFIGr_ECC_ENABLED,
    DDR34_PHY_CONTROL_REGS_DRAM_CONFIGr_SPLIT_DQ_BUS,
    DDR34_PHY_CONTROL_REGS_DRAM_CONFIGr_BUS16,
    DDR34_PHY_CONTROL_REGS_DRAM_CONFIGr_BUS8,
    DDR34_PHY_CONTROL_REGS_DRAM_CONFIGr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_DRAM_CONFIGr_EDC_MODE,
    DDR34_PHY_CONTROL_REGS_DRAM_CONFIGr_RDQS_MODE,
    DDR34_PHY_CONTROL_REGS_DRAM_CONFIGr_GROUP_BITS,
    DDR34_PHY_CONTROL_REGS_DRAM_CONFIGr_BANK_BITS,
    DDR34_PHY_CONTROL_REGS_DRAM_CONFIGr_COL_BITS,
    DDR34_PHY_CONTROL_REGS_DRAM_CONFIGr_ROW_BITS,
    DDR34_PHY_CONTROL_REGS_DRAM_CONFIGr_DRAM_TYPE,
    DDR34_PHY_CONTROL_REGS_DRAM_TIMING1r_TRAS,
    DDR34_PHY_CONTROL_REGS_DRAM_TIMING1r_TRRD,
    DDR34_PHY_CONTROL_REGS_DRAM_TIMING1r_TRP,
    DDR34_PHY_CONTROL_REGS_DRAM_TIMING1r_TRCD,
    DDR34_PHY_CONTROL_REGS_DRAM_TIMING2r_TRTP,
    DDR34_PHY_CONTROL_REGS_DRAM_TIMING2r_TWR,
    DDR34_PHY_CONTROL_REGS_DRAM_TIMING2r_TCWL,
    DDR34_PHY_CONTROL_REGS_DRAM_TIMING2r_TCAS,
    DDR34_PHY_CONTROL_REGS_DRAM_TIMING3r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_DRAM_TIMING3r_TCAL,
    DDR34_PHY_CONTROL_REGS_DRAM_TIMING3r_TRTW,
    DDR34_PHY_CONTROL_REGS_DRAM_TIMING3r_TWTR,
    DDR34_PHY_CONTROL_REGS_DRAM_TIMING3r_TRFC,
    DDR34_PHY_CONTROL_REGS_DRAM_TIMING4r_temp,
    DDR34_PHY_CONTROL_REGS_VDL_CALIBRATEr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CALIBRATEr_RESERVED_for_padding1,
    DDR34_PHY_CONTROL_REGS_VDL_CALIBRATEr_HALF_STEPS,
    DDR34_PHY_CONTROL_REGS_VDL_CALIBRATEr_UPDATE_FAST,
    DDR34_PHY_CONTROL_REGS_VDL_CALIBRATEr_UPDATE_REGS,
    DDR34_PHY_CONTROL_REGS_VDL_CALIBRATEr_CALIB_FTM2,
    DDR34_PHY_CONTROL_REGS_VDL_CALIBRATEr_CALIB_PHYBIST,
    DDR34_PHY_CONTROL_REGS_VDL_CALIBRATEr_CALIB_ONCE,
    DDR34_PHY_CONTROL_REGS_VDL_CALIB_STATUS1r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CALIB_STATUS1r_CALIB_TOTAL_STEPS,
    DDR34_PHY_CONTROL_REGS_VDL_CALIB_STATUS1r_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CALIB_STATUS1r_CALIB_BUS_ERROR,
    DDR34_PHY_CONTROL_REGS_VDL_CALIB_STATUS1r_CALIB_REGS_DONE,
    DDR34_PHY_CONTROL_REGS_VDL_CALIB_STATUS1r_CALIB_LOCK_6B,
    DDR34_PHY_CONTROL_REGS_VDL_CALIB_STATUS1r_CALIB_LOCK_4B,
    DDR34_PHY_CONTROL_REGS_VDL_CALIB_STATUS1r_CALIB_LOCK_2B,
    DDR34_PHY_CONTROL_REGS_VDL_CALIB_STATUS1r_CALIB_IDLE,
    DDR34_PHY_CONTROL_REGS_VDL_CALIB_STATUS2r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CALIB_STATUS2r_CALIB_4B_STEPS,
    DDR34_PHY_CONTROL_REGS_VDL_CALIB_STATUS2r_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CALIB_STATUS2r_CALIB_2B_STEPS,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_CONTROLr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_CONTROLr_INTERVAL,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_CONTROLr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_CONTROLr_UPDATE,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_CONTROLr_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_CONTROLr_DATA_RATE,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_CONTROLr_ENABLE,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_REFr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_REFr_MONITOR_4B_STEPS,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_REFr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_REFr_MONITOR_2B_STEPS,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_STATUSr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_STATUSr_MONITOR_BUS_ERROR,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_STATUSr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_STATUSr_MONITOR_ADJ,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_STATUSr_MONITOR_CHANGE,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_STATUSr_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_STATUSr_MONITOR_TOTAL,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OVERRIDEr_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OVERRIDEr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OVERRIDEr_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OVERRIDEr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OVERRIDEr_ADJ,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OVERRIDEr_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OVERRIDEr_ENABLE,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_CONTROLr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_CONTROLr_LOWER_LIMIT,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_CONTROLr_UPPER_LIMIT,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_CONTROLr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_CONTROLr_ENABLE,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUSr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUSr_MONITOR_CHANGE,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUSr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUSr_MONITOR_TOTAL,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUSr_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUSr_VALID,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUSr_CLEAR_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUSr_CLEAR_CLEAR,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUS_CLEARr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUS_CLEARr_CLEAR,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD00r_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD00r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD00r_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD00r_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD00r_ADJ_EN,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD00r_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD00r_VDL_STEP,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD01r_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD01r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD01r_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD01r_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD01r_ADJ_EN,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD01r_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD01r_VDL_STEP,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD02r_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD02r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD02r_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD02r_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD02r_ADJ_EN,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD02r_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD02r_VDL_STEP,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD03r_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD03r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD03r_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD03r_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD03r_ADJ_EN,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD03r_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD03r_VDL_STEP,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD04r_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD04r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD04r_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD04r_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD04r_ADJ_EN,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD04r_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD04r_VDL_STEP,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD05r_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD05r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD05r_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD05r_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD05r_ADJ_EN,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD05r_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD05r_VDL_STEP,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD06r_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD06r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD06r_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD06r_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD06r_ADJ_EN,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD06r_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD06r_VDL_STEP,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD07r_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD07r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD07r_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD07r_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD07r_ADJ_EN,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD07r_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD07r_VDL_STEP,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD08r_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD08r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD08r_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD08r_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD08r_ADJ_EN,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD08r_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD08r_VDL_STEP,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD09r_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD09r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD09r_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD09r_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD09r_ADJ_EN,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD09r_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD09r_VDL_STEP,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD10r_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD10r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD10r_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD10r_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD10r_ADJ_EN,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD10r_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD10r_VDL_STEP,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD11r_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD11r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD11r_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD11r_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD11r_ADJ_EN,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD11r_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD11r_VDL_STEP,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD12r_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD12r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD12r_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD12r_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD12r_ADJ_EN,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD12r_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD12r_VDL_STEP,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD13r_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD13r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD13r_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD13r_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD13r_ADJ_EN,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD13r_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD13r_VDL_STEP,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD14r_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD14r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD14r_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD14r_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD14r_ADJ_EN,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD14r_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD14r_VDL_STEP,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD15r_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD15r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD15r_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD15r_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD15r_ADJ_EN,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD15r_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD15r_VDL_STEP,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA0r_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA0r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA0r_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA0r_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA0r_ADJ_EN,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA0r_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA0r_VDL_STEP,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA1r_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA1r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA1r_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA1r_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA1r_ADJ_EN,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA1r_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA1r_VDL_STEP,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA2r_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA2r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA2r_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA2r_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA2r_ADJ_EN,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA2r_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA2r_VDL_STEP,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX0r_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX0r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX0r_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX0r_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX0r_ADJ_EN,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX0r_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX0r_VDL_STEP,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX1r_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX1r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX1r_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX1r_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX1r_ADJ_EN,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX1r_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX1r_VDL_STEP,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX2r_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX2r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX2r_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX2r_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX2r_ADJ_EN,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX2r_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX2r_VDL_STEP,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CS0r_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CS0r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CS0r_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CS0r_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CS0r_ADJ_EN,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CS0r_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CS0r_VDL_STEP,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CS1r_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CS1r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CS1r_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CS1r_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CS1r_ADJ_EN,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CS1r_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CS1r_VDL_STEP,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_PARr_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_PARr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_PARr_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_PARr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_PARr_ADJ_EN,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_PARr_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_PARr_VDL_STEP,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_RAS_Nr_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_RAS_Nr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_RAS_Nr_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_RAS_Nr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_RAS_Nr_ADJ_EN,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_RAS_Nr_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_RAS_Nr_VDL_STEP,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CAS_Nr_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CAS_Nr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CAS_Nr_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CAS_Nr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CAS_Nr_ADJ_EN,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CAS_Nr_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CAS_Nr_VDL_STEP,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CKEr_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CKEr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CKEr_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CKEr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CKEr_ADJ_EN,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CKEr_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CKEr_VDL_STEP,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_RST_Nr_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_RST_Nr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_RST_Nr_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_RST_Nr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_RST_Nr_ADJ_EN,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_RST_Nr_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_RST_Nr_VDL_STEP,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_ODTr_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_ODTr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_ODTr_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_ODTr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_ODTr_ADJ_EN,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_ODTr_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_ODTr_VDL_STEP,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_WE_Nr_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_WE_Nr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_WE_Nr_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_WE_Nr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_WE_Nr_ADJ_EN,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_WE_Nr_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_WE_Nr_VDL_STEP,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_Pr_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_Pr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_Pr_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_Pr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_Pr_ADJ_EN,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_Pr_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_Pr_VDL_STEP,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_Nr_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_Nr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_Nr_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_Nr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_Nr_ADJ_EN,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_Nr_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_Nr_VDL_STEP,
    DDR34_PHY_CONTROL_REGS_VDL_CLK_CONTROLr_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_CLK_CONTROLr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CLK_CONTROLr_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_CLK_CONTROLr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CLK_CONTROLr_ADJ_EN,
    DDR34_PHY_CONTROL_REGS_VDL_CLK_CONTROLr_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_CLK_CONTROLr_VDL_STEP,
    DDR34_PHY_CONTROL_REGS_VDL_LDE_CONTROLr_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_LDE_CONTROLr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_LDE_CONTROLr_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_LDE_CONTROLr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_LDE_CONTROLr_ADJ_EN,
    DDR34_PHY_CONTROL_REGS_VDL_LDE_CONTROLr_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_LDE_CONTROLr_VDL_STEP,
    DDR34_PHY_CONTROL_REGS_REFRESHr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_REFRESHr_ENABLE,
    DDR34_PHY_CONTROL_REGS_REFRESHr_PERIOD,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDLr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDLr_MODE,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDLr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDLr_DISABLE_INPUT,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDLr_ENABLE,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDLr_SNOOP1_RESERVED0,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDLr_SNOOP1_MODE,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDLr_SNOOP1_RESERVED1,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDLr_SNOOP1_MASK,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDLr_SNOOP1_RESERVED2,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDLr_SNOOP1_CMD,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDLr_SNOOP1_RESERVED3,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDLr_SNOOP1_ENABLE,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDLr_SNOOP2_RESERVED0,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDLr_SNOOP2_MODE,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDLr_SNOOP2_RESERVED1,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDLr_SNOOP2_MASK,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDLr_SNOOP2_RESERVED2,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDLr_SNOOP2_CMD,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDLr_SNOOP2_RESERVED3,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDLr_SNOOP2_ENABLE,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP1r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP1r_MODE,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP1r_RESERVED1,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP1r_MASK,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP1r_RESERVED2,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP1r_CMD,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP1r_RESERVED3,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP1r_ENABLE,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP2r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP2r_MODE,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP2r_RESERVED1,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP2r_MASK,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP2r_RESERVED2,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP2r_CMD,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP2r_RESERVED3,
    DDR34_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP2r_ENABLE,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG1r_MCP,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG1r_CS,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG1r_AUX,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG1r_ACT,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG1r_WE,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG1r_CAS,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG1r_RAS,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG1r_BA,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG1r_AD,
    DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG1r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG1r_AUX,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG2r_MCP,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG2r_CS,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG2r_AUX,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG2r_ACT,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG2r_WE,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG2r_CAS,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG2r_RAS,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG2r_BA,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG2r_AD,
    DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG2r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG2r_AUX,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG3r_MCP,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG3r_CS,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG3r_AUX,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG3r_ACT,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG3r_WE,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG3r_CAS,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG3r_RAS,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG3r_BA,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG3r_AD,
    DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG3r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG3r_AUX,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG4r_MCP,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG4r_CS,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG4r_AUX,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG4r_ACT,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG4r_WE,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG4r_CAS,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG4r_RAS,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG4r_BA,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG4r_AD,
    DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG4r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG4r_AUX,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG_TIMERr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG_TIMERr_INIT_VAL,
    DDR34_PHY_CONTROL_REGS_COMMAND_REG_TIMERr_COUNT,
    DDR34_PHY_CONTROL_REGS_MODE_REG0r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_MODE_REG0r_VALID,
    DDR34_PHY_CONTROL_REGS_MODE_REG0r_AD,
    DDR34_PHY_CONTROL_REGS_MODE_REG1r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_MODE_REG1r_VALID,
    DDR34_PHY_CONTROL_REGS_MODE_REG1r_AD,
    DDR34_PHY_CONTROL_REGS_MODE_REG2r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_MODE_REG2r_VALID,
    DDR34_PHY_CONTROL_REGS_MODE_REG2r_AD,
    DDR34_PHY_CONTROL_REGS_MODE_REG3r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_MODE_REG3r_VALID,
    DDR34_PHY_CONTROL_REGS_MODE_REG3r_AD,
    DDR34_PHY_CONTROL_REGS_MODE_REG4r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_MODE_REG4r_VALID,
    DDR34_PHY_CONTROL_REGS_MODE_REG4r_AD,
    DDR34_PHY_CONTROL_REGS_MODE_REG5r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_MODE_REG5r_VALID,
    DDR34_PHY_CONTROL_REGS_MODE_REG5r_AD,
    DDR34_PHY_CONTROL_REGS_MODE_REG6r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_MODE_REG6r_VALID,
    DDR34_PHY_CONTROL_REGS_MODE_REG6r_AD,
    DDR34_PHY_CONTROL_REGS_MODE_REG7r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_MODE_REG7r_VALID,
    DDR34_PHY_CONTROL_REGS_MODE_REG7r_AD,
    DDR34_PHY_CONTROL_REGS_MODE_REG8r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_MODE_REG8r_VALID,
    DDR34_PHY_CONTROL_REGS_MODE_REG8r_AD,
    DDR34_PHY_CONTROL_REGS_MODE_REG15r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_MODE_REG15r_VALID,
    DDR34_PHY_CONTROL_REGS_MODE_REG15r_AD,
    DDR34_PHY_CONTROL_REGS_MODE_REG63r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_MODE_REG63r_VALID,
    DDR34_PHY_CONTROL_REGS_MODE_REG63r_AD,
    DDR34_PHY_CONTROL_REGS_ALERT_CLEARr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_ALERT_CLEARr_CLEAR,
    DDR34_PHY_CONTROL_REGS_ALERT_STATUSr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_ALERT_STATUSr_STATUS,
    DDR34_PHY_CONTROL_REGS_CA_PARITYr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_CA_PARITYr_ERROR,
    DDR34_PHY_CONTROL_REGS_CA_PARITYr_ENABLE,
    DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_CONTROLr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_CONTROLr_COUNT,
    DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_CONTROLr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_CONTROLr_SAMPLE,
    DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_STATUS0r_VALID,
    DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_STATUS0r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_STATUS0r_DATA1,
    DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_STATUS0r_RESERVED1,
    DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_STATUS0r_DATA0,
    DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_STATUS1r_VALID,
    DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_STATUS1r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_STATUS1r_DATA1,
    DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_STATUS1r_RESERVED1,
    DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_STATUS1r_DATA0,
    DDR34_PHY_CONTROL_REGS_WRITE_LEVELING_CONTROLr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_WRITE_LEVELING_CONTROLr_COUNT,
    DDR34_PHY_CONTROL_REGS_WRITE_LEVELING_CONTROLr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_WRITE_LEVELING_CONTROLr_SAMPLE,
    DDR34_PHY_CONTROL_REGS_WRITE_LEVELING_CONTROLr_CONTINUOUS,
    DDR34_PHY_CONTROL_REGS_WRITE_LEVELING_CONTROLr_ENABLE,
    DDR34_PHY_CONTROL_REGS_WRITE_LEVELING_STATUSr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_WRITE_LEVELING_STATUSr_EDC,
    DDR34_PHY_CONTROL_REGS_WRITE_LEVELING_STATUSr_STATUS,
    DDR34_PHY_CONTROL_REGS_WRITE_LEVELING_STATUSr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_WRITE_LEVELING_STATUSr_VALID,
    DDR34_PHY_CONTROL_REGS_READ_ENABLE_CONTROLr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_READ_ENABLE_CONTROLr_TEST_CYCLE,
    DDR34_PHY_CONTROL_REGS_READ_ENABLE_CONTROLr_SELECT,
    DDR34_PHY_CONTROL_REGS_READ_ENABLE_CONTROLr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_READ_ENABLE_CONTROLr_CS_N,
    DDR34_PHY_CONTROL_REGS_READ_ENABLE_CONTROLr_EDC_DATA,
    DDR34_PHY_CONTROL_REGS_READ_ENABLE_CONTROLr_EDC_PHASE,
    DDR34_PHY_CONTROL_REGS_READ_ENABLE_CONTROLr_DQS,
    DDR34_PHY_CONTROL_REGS_READ_ENABLE_CONTROLr_ENABLE,
    DDR34_PHY_CONTROL_REGS_READ_ENABLE_STATUSr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_READ_ENABLE_STATUSr_DATA,
    DDR34_PHY_CONTROL_REGS_READ_ENABLE_STATUSr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_READ_ENABLE_STATUSr_BL4_STATUS,
    DDR34_PHY_CONTROL_REGS_READ_ENABLE_STATUSr_BL3_STATUS,
    DDR34_PHY_CONTROL_REGS_READ_ENABLE_STATUSr_BL2_STATUS,
    DDR34_PHY_CONTROL_REGS_READ_ENABLE_STATUSr_BL1_STATUS,
    DDR34_PHY_CONTROL_REGS_READ_ENABLE_STATUSr_BL0_STATUS,
    DDR34_PHY_CONTROL_REGS_READ_ENABLE_STATUSr_RESERVED2,
    DDR34_PHY_CONTROL_REGS_READ_ENABLE_STATUSr_VALID,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_LFSR_SEEDr_SEED,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ADDRESS1r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ADDRESS1r_BANK,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ADDRESS1r_ROW,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ADDRESS2r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ADDRESS2r_BANK,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ADDRESS2r_ROW,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROLr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROLr_DIAG_WRO,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROLr_DIAG_WRO_RD,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROLr_DIAG_WR_RD,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROLr_WR_NOISE,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROLr_RD_NOISE,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROLr_CLEAR_DRAM,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROLr_MASK_DM,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROLr_INIT_LFSR,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROLr_FIFO,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROLr_MPR,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROLr_RD_WR,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROLr_RD_EN,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROLr_ENABLE,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_DATA_CONTROLr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_DATA_CONTROLr_PATTERN,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_DATA_CONTROLr_LENGTH,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_DQ_MASKr_MASK,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ECC_DQ_MASKr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ECC_DQ_MASKr_MASK,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_STATUSr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_STATUSr_BUSY,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_DQ_STATUSr_STATUS,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ECC_STATUSr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ECC_STATUSr_STATUS,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ERR_CNT_CONTROLr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ERR_CNT_CONTROLr_DQ_SEL,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ERR_CNT_CONTROLr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ERR_CNT_CONTROLr_CLEAR,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ERR_CNT_CONTROLr_ENABLE,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ERR_CNT_STATUSr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ERR_CNT_STATUSr_COUNT,
    DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROLr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROLr_RESERVED_for_eco1,
    DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROLr_MAX_NOISE,
    DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROLr_LOW_NOISE,
    DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROLr_LOW_VTT,
    DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROLr_HIGH_VTT,
    DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROLr_ERROR_RESET,
    DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROLr_ENABLE_CTL_IDLE,
    DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROLr_ENABLE_CS_IDLE,
    DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROLr_ENABLE_CKE_IDLE,
    DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_STATUSr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_STATUSr_ERROR,
    DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_STATUSr_ERROR_LOW,
    DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_STATUSr_ERROR_HIGH,
    DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_STATUSr_READY,
    DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_CONNECTIONSr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_CONNECTIONSr_MASK,
    DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_OVERRIDEr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_OVERRIDEr_MASK,
    DDR34_PHY_CONTROL_REGS_VREF_DAC_CONTROLr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VREF_DAC_CONTROLr_AUX_GT_INT,
    DDR34_PHY_CONTROL_REGS_VREF_DAC_CONTROLr_TESTOUT_MUX_CTL,
    DDR34_PHY_CONTROL_REGS_VREF_DAC_CONTROLr_TEST,
    DDR34_PHY_CONTROL_REGS_VREF_DAC_CONTROLr_PDN3,
    DDR34_PHY_CONTROL_REGS_VREF_DAC_CONTROLr_PDN2,
    DDR34_PHY_CONTROL_REGS_VREF_DAC_CONTROLr_PDN1,
    DDR34_PHY_CONTROL_REGS_VREF_DAC_CONTROLr_PDN0,
    DDR34_PHY_CONTROL_REGS_VREF_DAC_CONTROLr_DAC1,
    DDR34_PHY_CONTROL_REGS_VREF_DAC_CONTROLr_DAC0,
    DDR34_PHY_CONTROL_REGS_PHYBIST_CNTRLr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_PHYBIST_CNTRLr_CLK_PAD_ENB,
    DDR34_PHY_CONTROL_REGS_PHYBIST_CNTRLr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_PHYBIST_CNTRLr_FORCE_DQ_ERROR_SEL,
    DDR34_PHY_CONTROL_REGS_PHYBIST_CNTRLr_RESERVED2,
    DDR34_PHY_CONTROL_REGS_PHYBIST_CNTRLr_FORCE_BL_ERROR_SEL,
    DDR34_PHY_CONTROL_REGS_PHYBIST_CNTRLr_RESERVED3,
    DDR34_PHY_CONTROL_REGS_PHYBIST_CNTRLr_FORCE_CTL_ERROR_SEL,
    DDR34_PHY_CONTROL_REGS_PHYBIST_CNTRLr_RESERVED4,
    DDR34_PHY_CONTROL_REGS_PHYBIST_CNTRLr_FORCE_DAT_ERROR,
    DDR34_PHY_CONTROL_REGS_PHYBIST_CNTRLr_FORCE_CTL_ERROR,
    DDR34_PHY_CONTROL_REGS_PHYBIST_CNTRLr_SSO,
    DDR34_PHY_CONTROL_REGS_PHYBIST_CNTRLr_LENGTH,
    DDR34_PHY_CONTROL_REGS_PHYBIST_CNTRLr_MODE,
    DDR34_PHY_CONTROL_REGS_PHYBIST_CNTRLr_ENABLE,
    DDR34_PHY_CONTROL_REGS_PHYBIST_SEEDr_SEED,
    DDR34_PHY_CONTROL_REGS_PHYBIST_STATUSr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_PHYBIST_STATUSr_DAT_PASS,
    DDR34_PHY_CONTROL_REGS_PHYBIST_STATUSr_CTL_PASS,
    DDR34_PHY_CONTROL_REGS_PHYBIST_STATUSr_DAT_DONE,
    DDR34_PHY_CONTROL_REGS_PHYBIST_STATUSr_CTL_DONE,
    DDR34_PHY_CONTROL_REGS_PHYBIST_CTL_STATUSr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_PHYBIST_CTL_STATUSr_CTL_ERRORS,
    DDR34_PHY_CONTROL_REGS_PHYBIST_BL0_STATUSr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_PHYBIST_BL0_STATUSr_EDC,
    DDR34_PHY_CONTROL_REGS_PHYBIST_BL0_STATUSr_DM,
    DDR34_PHY_CONTROL_REGS_PHYBIST_BL0_STATUSr_DQ,
    DDR34_PHY_CONTROL_REGS_PHYBIST_BL1_STATUSr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_PHYBIST_BL1_STATUSr_EDC,
    DDR34_PHY_CONTROL_REGS_PHYBIST_BL1_STATUSr_DM,
    DDR34_PHY_CONTROL_REGS_PHYBIST_BL1_STATUSr_DQ,
    DDR34_PHY_CONTROL_REGS_PHYBIST_BL2_STATUSr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_PHYBIST_BL2_STATUSr_EDC,
    DDR34_PHY_CONTROL_REGS_PHYBIST_BL2_STATUSr_DM,
    DDR34_PHY_CONTROL_REGS_PHYBIST_BL2_STATUSr_DQ,
    DDR34_PHY_CONTROL_REGS_PHYBIST_BL3_STATUSr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_PHYBIST_BL3_STATUSr_EDC,
    DDR34_PHY_CONTROL_REGS_PHYBIST_BL3_STATUSr_DM,
    DDR34_PHY_CONTROL_REGS_PHYBIST_BL3_STATUSr_DQ,
    DDR34_PHY_CONTROL_REGS_PHYBIST_BL4_STATUSr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_PHYBIST_BL4_STATUSr_EDC,
    DDR34_PHY_CONTROL_REGS_PHYBIST_BL4_STATUSr_DM,
    DDR34_PHY_CONTROL_REGS_PHYBIST_BL4_STATUSr_DQ,
    DDR34_PHY_CONTROL_REGS_STANDBY_CONTROLr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_STANDBY_CONTROLr_STANDBY_READY,
    DDR34_PHY_CONTROL_REGS_STANDBY_CONTROLr_STANDBY_EXIT_PIN_EN,
    DDR34_PHY_CONTROL_REGS_STANDBY_CONTROLr_STANDBY_ACTIVE,
    DDR34_PHY_CONTROL_REGS_STANDBY_CONTROLr_ARMED,
    DDR34_PHY_CONTROL_REGS_STANDBY_CONTROLr_WARMSTART,
    DDR34_PHY_CONTROL_REGS_STANDBY_CONTROLr_PWRDOWN_LDO_BIAS,
    DDR34_PHY_CONTROL_REGS_STANDBY_CONTROLr_PWRDOWN_LDO_VOLTS,
    DDR34_PHY_CONTROL_REGS_STANDBY_CONTROLr_PWRDOWN_SKIP_MRS,
    DDR34_PHY_CONTROL_REGS_STANDBY_CONTROLr_PWRDOWN_RST_N,
    DDR34_PHY_CONTROL_REGS_STANDBY_CONTROLr_PWRDOWN_CKE,
    DDR34_PHY_CONTROL_REGS_STANDBY_CONTROLr_LDO_BIAS,
    DDR34_PHY_CONTROL_REGS_STANDBY_CONTROLr_LDO_VOLTS,
    DDR34_PHY_CONTROL_REGS_STANDBY_CONTROLr_SKIP_MRS,
    DDR34_PHY_CONTROL_REGS_STANDBY_CONTROLr_RST_N,
    DDR34_PHY_CONTROL_REGS_STANDBY_CONTROLr_CKE,
    DDR34_PHY_CONTROL_REGS_STANDBY_CONTROLr_STANDBY,
    DDR34_PHY_CONTROL_REGS_DEBUG_FREEZE_ENABLEr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_DEBUG_FREEZE_ENABLEr_WLECC,
    DDR34_PHY_CONTROL_REGS_DEBUG_FREEZE_ENABLEr_WL1_BL1,
    DDR34_PHY_CONTROL_REGS_DEBUG_FREEZE_ENABLEr_WL1_BL0,
    DDR34_PHY_CONTROL_REGS_DEBUG_FREEZE_ENABLEr_WL0_BL1,
    DDR34_PHY_CONTROL_REGS_DEBUG_FREEZE_ENABLEr_WL0_BL0,
    DDR34_PHY_CONTROL_REGS_DEBUG_MUX_CONTROLr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_DEBUG_MUX_CONTROLr_BYTE_SEL,
    DDR34_PHY_CONTROL_REGS_DEBUG_MUX_CONTROLr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_DEBUG_MUX_CONTROLr_PHASE_SEL,
    DDR34_PHY_CONTROL_REGS_DEBUG_MUX_CONTROLr_SOURCE_SEL,
    DDR34_PHY_CONTROL_REGS_DFI_CNTRLr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_DFI_CNTRLr_DFI_CS1,
    DDR34_PHY_CONTROL_REGS_DFI_CNTRLr_DFI_CS0,
    DDR34_PHY_CONTROL_REGS_DFI_CNTRLr_DFI_RST_N,
    DDR34_PHY_CONTROL_REGS_DFI_CNTRLr_DFI_CKE,
    DDR34_PHY_CONTROL_REGS_DFI_CNTRLr_ACK_ENABLE,
    DDR34_PHY_CONTROL_REGS_DFI_CNTRLr_ACK_STATUS,
    DDR34_PHY_CONTROL_REGS_DFI_CNTRLr_ASSERT_REQ,
    DDR34_PHY_CONTROL_REGS_WRITE_ODT_CNTRLr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_WRITE_ODT_CNTRLr_ODT_FORCE_VALUE,
    DDR34_PHY_CONTROL_REGS_WRITE_ODT_CNTRLr_ODT_FORCE,
    DDR34_PHY_CONTROL_REGS_WRITE_ODT_CNTRLr_ODT_ENABLE,
    DDR34_PHY_CONTROL_REGS_WRITE_ODT_CNTRLr_ODT_LENGTH,
    DDR34_PHY_CONTROL_REGS_WRITE_ODT_CNTRLr_ODT_DELAY,
    DDR34_PHY_CONTROL_REGS_ABI_PAR_CNTRLr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_ABI_PAR_CNTRLr_PAR_INCLUDE_AUX,
    DDR34_PHY_CONTROL_REGS_ABI_PAR_CNTRLr_PAR_ENABLE,
    DDR34_PHY_CONTROL_REGS_ABI_PAR_CNTRLr_ABI_INCLUDE_AUX,
    DDR34_PHY_CONTROL_REGS_ABI_PAR_CNTRLr_ABI_ENABLE,
    DDR34_PHY_CONTROL_REGS_ZQ_CALr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_ZQ_CALr_ZQ_PCOMP_STATUS,
    DDR34_PHY_CONTROL_REGS_ZQ_CALr_ZQ_NCOMP_STATUS,
    DDR34_PHY_CONTROL_REGS_ZQ_CALr_ZQ_IDDQ,
    DDR34_PHY_CONTROL_REGS_ZQ_CALr_ZQ_DRIVE_P,
    DDR34_PHY_CONTROL_REGS_ZQ_CALr_ZQ_DRIVE_N,
    DDR34_PHY_CONTROL_REGS_ZQ_CALr_RESERVED_for_padding1,
    DDR34_PHY_CONTROL_REGS_ZQ_CALr_ZQ_PCOMP_ENB,
    DDR34_PHY_CONTROL_REGS_ZQ_CALr_ZQ_NCOMP_ENB,
    DDR34_PHY_CONTROL_REGS_AC_SPARE_REGr_RESERVED_for_eco0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_Pr_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_Pr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_Pr_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_Pr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_Pr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_Pr_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_Pr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_Nr_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_Nr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_Nr_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_Nr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_Nr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_Nr_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_Nr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ0r_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ0r_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ0r_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ0r_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ0r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ0r_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ0r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ1r_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ1r_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ1r_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ1r_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ1r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ1r_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ1r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ2r_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ2r_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ2r_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ2r_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ2r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ2r_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ2r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ3r_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ3r_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ3r_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ3r_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ3r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ3r_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ3r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ4r_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ4r_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ4r_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ4r_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ4r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ4r_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ4r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ5r_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ5r_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ5r_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ5r_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ5r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ5r_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ5r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ6r_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ6r_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ6r_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ6r_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ6r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ6r_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ6r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ7r_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ7r_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ7r_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ7r_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ7r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ7r_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ7r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DMr_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DMr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DMr_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DMr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DMr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DMr_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DMr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_EDCr_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_EDCr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_EDCr_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_EDCr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_EDCr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_EDCr_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_EDCr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSPr_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSPr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSPr_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSPr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSPr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSPr_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSPr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSNr_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSNr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSNr_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSNr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSNr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSNr_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSNr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0Pr_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0Pr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0Pr_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0Pr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0Pr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0Pr_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0Pr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0Nr_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0Nr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0Nr_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0Nr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0Nr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0Nr_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0Nr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1Pr_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1Pr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1Pr_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1Pr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1Pr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1Pr_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1Pr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1Nr_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1Nr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1Nr_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1Nr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1Nr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1Nr_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1Nr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2Pr_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2Pr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2Pr_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2Pr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2Pr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2Pr_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2Pr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2Nr_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2Nr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2Nr_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2Nr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2Nr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2Nr_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2Nr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3Pr_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3Pr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3Pr_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3Pr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3Pr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3Pr_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3Pr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3Nr_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3Nr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3Nr_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3Nr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3Nr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3Nr_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3Nr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4Pr_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4Pr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4Pr_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4Pr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4Pr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4Pr_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4Pr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4Nr_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4Nr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4Nr_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4Nr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4Nr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4Nr_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4Nr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5Pr_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5Pr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5Pr_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5Pr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5Pr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5Pr_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5Pr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5Nr_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5Nr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5Nr_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5Nr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5Nr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5Nr_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5Nr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6Pr_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6Pr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6Pr_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6Pr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6Pr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6Pr_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6Pr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6Nr_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6Nr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6Nr_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6Nr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6Nr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6Nr_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6Nr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7Pr_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7Pr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7Pr_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7Pr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7Pr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7Pr_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7Pr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7Nr_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7Nr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7Nr_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7Nr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7Nr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7Nr_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7Nr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMPr_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMPr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMPr_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMPr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMPr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMPr_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMPr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMNr_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMNr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMNr_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMNr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMNr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMNr_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMNr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCPr_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCPr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCPr_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCPr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCPr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCPr_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCPr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCNr_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCNr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCNr_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCNr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCNr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCNr_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCNr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS0r_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS0r_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS0r_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS0r_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS0r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS0r_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS0r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS1r_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS1r_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS1r_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS1r_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS1r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS1r_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS1r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CLK_CONTROLr_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CLK_CONTROLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CLK_CONTROLr_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CLK_CONTROLr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CLK_CONTROLr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CLK_CONTROLr_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CLK_CONTROLr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_LDE_CONTROLr_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_LDE_CONTROLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_LDE_CONTROLr_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_LDE_CONTROLr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_LDE_CONTROLr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_LDE_CONTROLr_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_LDE_CONTROLr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_RD_EN_DLY_CYCr_BUSY,
    DDR34_PHY_BYTE_LANE_0_RD_EN_DLY_CYCr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_RD_EN_DLY_CYCr_FORCE,
    DDR34_PHY_BYTE_LANE_0_RD_EN_DLY_CYCr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_RD_EN_DLY_CYCr_CS1_CYCLES,
    DDR34_PHY_BYTE_LANE_0_RD_EN_DLY_CYCr_CS0_CYCLES,
    DDR34_PHY_BYTE_LANE_0_WR_CHAN_DLY_CYCr_BUSY,
    DDR34_PHY_BYTE_LANE_0_WR_CHAN_DLY_CYCr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_WR_CHAN_DLY_CYCr_FORCE,
    DDR34_PHY_BYTE_LANE_0_WR_CHAN_DLY_CYCr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_WR_CHAN_DLY_CYCr_CYCLES,
    DDR34_PHY_BYTE_LANE_0_READ_CONTROLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_READ_CONTROLr_MODE,
    DDR34_PHY_BYTE_LANE_0_READ_CONTROLr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_READ_CONTROLr_RD_DATA_DLY,
    DDR34_PHY_BYTE_LANE_0_READ_FIFO_ADDRr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_READ_FIFO_ADDRr_ADDR,
    DDR34_PHY_BYTE_LANE_0_READ_FIFO_DATAr_DATA,
    DDR34_PHY_BYTE_LANE_0_READ_FIFO_DM_DBIr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_READ_FIFO_DM_DBIr_DM_DBI,
    DDR34_PHY_BYTE_LANE_0_READ_FIFO_STATUSr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_READ_FIFO_STATUSr_OVERFLOW,
    DDR34_PHY_BYTE_LANE_0_READ_FIFO_STATUSr_UNDERFLOW,
    DDR34_PHY_BYTE_LANE_0_READ_FIFO_CLEARr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_READ_FIFO_CLEARr_CLEAR,
    DDR34_PHY_BYTE_LANE_0_IDLE_PAD_CONTROLr_IDLE,
    DDR34_PHY_BYTE_LANE_0_IDLE_PAD_CONTROLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_IDLE_PAD_CONTROLr_AUTO_DQ_RXENB_MODE,
    DDR34_PHY_BYTE_LANE_0_IDLE_PAD_CONTROLr_AUTO_DQ_IDDQ_MODE,
    DDR34_PHY_BYTE_LANE_0_IDLE_PAD_CONTROLr_RESERVED_for_padding1,
    DDR34_PHY_BYTE_LANE_0_IDLE_PAD_CONTROLr_IO_IDLE_ENABLE,
    DDR34_PHY_BYTE_LANE_0_IDLE_PAD_CONTROLr_RXENB,
    DDR34_PHY_BYTE_LANE_0_IDLE_PAD_CONTROLr_IDDQ,
    DDR34_PHY_BYTE_LANE_0_IDLE_PAD_CONTROLr_DOUT_N,
    DDR34_PHY_BYTE_LANE_0_IDLE_PAD_CONTROLr_DOUT_P,
    DDR34_PHY_BYTE_LANE_0_DRIVE_PAD_CTLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_DRIVE_PAD_CTLr_BL_PD_IDLE_STRENGTH,
    DDR34_PHY_BYTE_LANE_0_DRIVE_PAD_CTLr_BL_ND_IDLE_STRENGTH,
    DDR34_PHY_BYTE_LANE_0_DRIVE_PAD_CTLr_BL_PD_TERM_STRENGTH,
    DDR34_PHY_BYTE_LANE_0_DRIVE_PAD_CTLr_BL_ND_TERM_STRENGTH,
    DDR34_PHY_BYTE_LANE_0_DRIVE_PAD_CTLr_BL_PD_STRENGTH,
    DDR34_PHY_BYTE_LANE_0_DRIVE_PAD_CTLr_BL_ND_STRENGTH,
    DDR34_PHY_BYTE_LANE_0_RD_EN_DRIVE_PAD_CTLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_RD_EN_DRIVE_PAD_CTLr_EDC_RD_EN_PD_STRENGTH,
    DDR34_PHY_BYTE_LANE_0_RD_EN_DRIVE_PAD_CTLr_EDC_RD_EN_ND_STRENGTH,
    DDR34_PHY_BYTE_LANE_0_RD_EN_DRIVE_PAD_CTLr_BL_RD_EN_PD_STRENGTH,
    DDR34_PHY_BYTE_LANE_0_RD_EN_DRIVE_PAD_CTLr_BL_RD_EN_ND_STRENGTH,
    DDR34_PHY_BYTE_LANE_0_STATIC_PAD_CTLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_STATIC_PAD_CTLr_DQS_MODE,
    DDR34_PHY_BYTE_LANE_0_STATIC_PAD_CTLr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_STATIC_PAD_CTLr_EDC_MODE,
    DDR34_PHY_BYTE_LANE_0_STATIC_PAD_CTLr_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_STATIC_PAD_CTLr_WDBI_ENABLE,
    DDR34_PHY_BYTE_LANE_0_STATIC_PAD_CTLr_RDBI_ENABLE,
    DDR34_PHY_BYTE_LANE_0_STATIC_PAD_CTLr_DM_MODE,
    DDR34_PHY_BYTE_LANE_0_STATIC_PAD_CTLr_RESERVED3,
    DDR34_PHY_BYTE_LANE_0_STATIC_PAD_CTLr_DQS_TX_DIS,
    DDR34_PHY_BYTE_LANE_0_STATIC_PAD_CTLr_RESERVED4,
    DDR34_PHY_BYTE_LANE_0_STATIC_PAD_CTLr_RX_MODE,
    DDR34_PHY_BYTE_LANE_0_WR_PREAMBLE_MODEr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_WR_PREAMBLE_MODEr_DQ_POSTAM_BITS,
    DDR34_PHY_BYTE_LANE_0_WR_PREAMBLE_MODEr_DQ_PREAM_BITS,
    DDR34_PHY_BYTE_LANE_0_WR_PREAMBLE_MODEr_DQS,
    DDR34_PHY_BYTE_LANE_0_WR_PREAMBLE_MODEr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_WR_PREAMBLE_MODEr_DQS_POSTAM_BITS,
    DDR34_PHY_BYTE_LANE_0_WR_PREAMBLE_MODEr_DQS_PREAM_BITS,
    DDR34_PHY_BYTE_LANE_0_ODT_CONTROLr_RESERVED_for_padding0,
    DDR34_PHY_BYTE_LANE_0_ODT_CONTROLr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_ODT_CONTROLr_ODT_ENABLE,
    DDR34_PHY_BYTE_LANE_0_ODT_CONTROLr_ODT_DELAY,
    DDR34_PHY_BYTE_LANE_0_ODT_CONTROLr_ODT_POST_LENGTH,
    DDR34_PHY_BYTE_LANE_0_ODT_CONTROLr_ODT_PRE_LENGTH,
    DDR34_PHY_BYTE_LANE_0_EDC_DPD_CONTROLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_EDC_DPD_CONTROLr_DQS_PHASE_OVERRIDE_VALUE,
    DDR34_PHY_BYTE_LANE_0_EDC_DPD_CONTROLr_DQS_PHASE_OVERRIDE_ENABLE,
    DDR34_PHY_BYTE_LANE_0_EDC_DPD_CONTROLr_SET_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_EDC_DPD_CONTROLr_UPDATE,
    DDR34_PHY_BYTE_LANE_0_EDC_DPD_CONTROLr_MONITOR,
    DDR34_PHY_BYTE_LANE_0_EDC_DPD_CONTROLr_INIT,
    DDR34_PHY_BYTE_LANE_0_EDC_DPD_STATUSr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_EDC_DPD_STATUSr_CURRENT_DQS_PHASE,
    DDR34_PHY_BYTE_LANE_0_EDC_DPD_STATUSr_CURRENT_VDL_SETTING,
    DDR34_PHY_BYTE_LANE_0_EDC_DPD_STATUSr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_EDC_DPD_STATUSr_INIT_DQS_PHASE,
    DDR34_PHY_BYTE_LANE_0_EDC_DPD_STATUSr_INIT_VDL_SETTING,
    DDR34_PHY_BYTE_LANE_0_EDC_DPD_STATUSr_EDGE_ERROR,
    DDR34_PHY_BYTE_LANE_0_EDC_DPD_STATUSr_MONITOR_BUSY,
    DDR34_PHY_BYTE_LANE_0_EDC_DPD_STATUSr_INIT_BUSY,
    DDR34_PHY_BYTE_LANE_0_EDC_DPD_STATUSr_INIT_DONE,
    DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_CONTROLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_CONTROLr_LOWER_LIMIT,
    DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_CONTROLr_UPPER_LIMIT,
    DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_CONTROLr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_CONTROLr_ENABLE,
    DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_STATUSr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_STATUSr_VDL_SETTING,
    DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_STATUSr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_STATUSr_VALID,
    DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_STATUSr_CLEAR_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_STATUSr_CLEAR_CLEAR,
    DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_STATUS_CLEARr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_STATUS_CLEARr_CLEAR,
    DDR34_PHY_BYTE_LANE_0_EDC_CRC_CONTROLr_BUSY,
    DDR34_PHY_BYTE_LANE_0_EDC_CRC_CONTROLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_EDC_CRC_CONTROLr_FORCE,
    DDR34_PHY_BYTE_LANE_0_EDC_CRC_CONTROLr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_EDC_CRC_CONTROLr_CRCWL,
    DDR34_PHY_BYTE_LANE_0_EDC_CRC_CONTROLr_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_EDC_CRC_CONTROLr_CRCRL,
    DDR34_PHY_BYTE_LANE_0_EDC_CRC_CONTROLr_RESERVED3,
    DDR34_PHY_BYTE_LANE_0_EDC_CRC_CONTROLr_COUNT_MODE,
    DDR34_PHY_BYTE_LANE_0_EDC_CRC_CONTROLr_ENABLE_WR,
    DDR34_PHY_BYTE_LANE_0_EDC_CRC_CONTROLr_ENABLE_RD,
    DDR34_PHY_BYTE_LANE_0_EDC_CRC_STATUSr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_EDC_CRC_STATUSr_WR_FAIL,
    DDR34_PHY_BYTE_LANE_0_EDC_CRC_STATUSr_WR_PASS,
    DDR34_PHY_BYTE_LANE_0_EDC_CRC_STATUSr_RD_FAIL,
    DDR34_PHY_BYTE_LANE_0_EDC_CRC_STATUSr_RD_PASS,
    DDR34_PHY_BYTE_LANE_0_EDC_CRC_STATUSr_CLEAR_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_EDC_CRC_STATUSr_CLEAR_CLEAR,
    DDR34_PHY_BYTE_LANE_0_EDC_CRC_COUNTr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_EDC_CRC_COUNTr_WR_COUNT,
    DDR34_PHY_BYTE_LANE_0_EDC_CRC_COUNTr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_EDC_CRC_COUNTr_RD_COUNT,
    DDR34_PHY_BYTE_LANE_0_EDC_CRC_STATUS_CLEARr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_EDC_CRC_STATUS_CLEARr_CLEAR,
    DDR34_PHY_BYTE_LANE_0_BL_SPARE_REGr_RESERVED_for_eco0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_Pr_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_Pr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_Pr_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_Pr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_Pr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_Pr_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_Pr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_Nr_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_Nr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_Nr_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_Nr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_Nr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_Nr_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_Nr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ0r_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ0r_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ0r_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ0r_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ0r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ0r_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ0r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ1r_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ1r_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ1r_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ1r_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ1r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ1r_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ1r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ2r_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ2r_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ2r_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ2r_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ2r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ2r_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ2r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ3r_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ3r_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ3r_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ3r_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ3r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ3r_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ3r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ4r_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ4r_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ4r_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ4r_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ4r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ4r_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ4r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ5r_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ5r_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ5r_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ5r_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ5r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ5r_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ5r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ6r_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ6r_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ6r_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ6r_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ6r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ6r_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ6r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ7r_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ7r_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ7r_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ7r_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ7r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ7r_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ7r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DMr_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DMr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DMr_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DMr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DMr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DMr_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DMr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_EDCr_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_EDCr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_EDCr_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_EDCr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_EDCr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_EDCr_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_EDCr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSPr_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSPr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSPr_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSPr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSPr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSPr_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSPr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSNr_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSNr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSNr_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSNr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSNr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSNr_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSNr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0Pr_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0Pr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0Pr_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0Pr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0Pr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0Pr_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0Pr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0Nr_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0Nr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0Nr_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0Nr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0Nr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0Nr_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0Nr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1Pr_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1Pr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1Pr_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1Pr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1Pr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1Pr_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1Pr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1Nr_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1Nr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1Nr_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1Nr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1Nr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1Nr_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1Nr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2Pr_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2Pr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2Pr_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2Pr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2Pr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2Pr_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2Pr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2Nr_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2Nr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2Nr_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2Nr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2Nr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2Nr_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2Nr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3Pr_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3Pr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3Pr_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3Pr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3Pr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3Pr_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3Pr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3Nr_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3Nr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3Nr_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3Nr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3Nr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3Nr_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3Nr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4Pr_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4Pr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4Pr_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4Pr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4Pr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4Pr_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4Pr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4Nr_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4Nr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4Nr_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4Nr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4Nr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4Nr_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4Nr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5Pr_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5Pr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5Pr_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5Pr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5Pr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5Pr_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5Pr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5Nr_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5Nr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5Nr_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5Nr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5Nr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5Nr_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5Nr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6Pr_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6Pr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6Pr_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6Pr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6Pr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6Pr_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6Pr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6Nr_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6Nr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6Nr_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6Nr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6Nr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6Nr_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6Nr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7Pr_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7Pr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7Pr_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7Pr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7Pr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7Pr_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7Pr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7Nr_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7Nr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7Nr_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7Nr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7Nr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7Nr_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7Nr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMPr_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMPr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMPr_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMPr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMPr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMPr_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMPr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMNr_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMNr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMNr_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMNr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMNr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMNr_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMNr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCPr_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCPr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCPr_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCPr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCPr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCPr_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCPr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCNr_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCNr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCNr_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCNr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCNr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCNr_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCNr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS0r_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS0r_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS0r_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS0r_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS0r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS0r_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS0r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS1r_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS1r_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS1r_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS1r_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS1r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS1r_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS1r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CLK_CONTROLr_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CLK_CONTROLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CLK_CONTROLr_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CLK_CONTROLr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CLK_CONTROLr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CLK_CONTROLr_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CLK_CONTROLr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_LDE_CONTROLr_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_LDE_CONTROLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_LDE_CONTROLr_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_LDE_CONTROLr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_LDE_CONTROLr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_LDE_CONTROLr_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_LDE_CONTROLr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_RD_EN_DLY_CYCr_BUSY,
    DDR34_PHY_BYTE_LANE_1_RD_EN_DLY_CYCr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_RD_EN_DLY_CYCr_FORCE,
    DDR34_PHY_BYTE_LANE_1_RD_EN_DLY_CYCr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_RD_EN_DLY_CYCr_CS1_CYCLES,
    DDR34_PHY_BYTE_LANE_1_RD_EN_DLY_CYCr_CS0_CYCLES,
    DDR34_PHY_BYTE_LANE_1_WR_CHAN_DLY_CYCr_BUSY,
    DDR34_PHY_BYTE_LANE_1_WR_CHAN_DLY_CYCr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_WR_CHAN_DLY_CYCr_FORCE,
    DDR34_PHY_BYTE_LANE_1_WR_CHAN_DLY_CYCr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_WR_CHAN_DLY_CYCr_CYCLES,
    DDR34_PHY_BYTE_LANE_1_READ_CONTROLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_READ_CONTROLr_MODE,
    DDR34_PHY_BYTE_LANE_1_READ_CONTROLr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_READ_CONTROLr_RD_DATA_DLY,
    DDR34_PHY_BYTE_LANE_1_READ_FIFO_ADDRr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_READ_FIFO_ADDRr_ADDR,
    DDR34_PHY_BYTE_LANE_1_READ_FIFO_DATAr_DATA,
    DDR34_PHY_BYTE_LANE_1_READ_FIFO_DM_DBIr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_READ_FIFO_DM_DBIr_DM_DBI,
    DDR34_PHY_BYTE_LANE_1_READ_FIFO_STATUSr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_READ_FIFO_STATUSr_OVERFLOW,
    DDR34_PHY_BYTE_LANE_1_READ_FIFO_STATUSr_UNDERFLOW,
    DDR34_PHY_BYTE_LANE_1_READ_FIFO_CLEARr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_READ_FIFO_CLEARr_CLEAR,
    DDR34_PHY_BYTE_LANE_1_IDLE_PAD_CONTROLr_IDLE,
    DDR34_PHY_BYTE_LANE_1_IDLE_PAD_CONTROLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_IDLE_PAD_CONTROLr_AUTO_DQ_RXENB_MODE,
    DDR34_PHY_BYTE_LANE_1_IDLE_PAD_CONTROLr_AUTO_DQ_IDDQ_MODE,
    DDR34_PHY_BYTE_LANE_1_IDLE_PAD_CONTROLr_RESERVED_for_padding1,
    DDR34_PHY_BYTE_LANE_1_IDLE_PAD_CONTROLr_IO_IDLE_ENABLE,
    DDR34_PHY_BYTE_LANE_1_IDLE_PAD_CONTROLr_RXENB,
    DDR34_PHY_BYTE_LANE_1_IDLE_PAD_CONTROLr_IDDQ,
    DDR34_PHY_BYTE_LANE_1_IDLE_PAD_CONTROLr_DOUT_N,
    DDR34_PHY_BYTE_LANE_1_IDLE_PAD_CONTROLr_DOUT_P,
    DDR34_PHY_BYTE_LANE_1_DRIVE_PAD_CTLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_DRIVE_PAD_CTLr_BL_PD_IDLE_STRENGTH,
    DDR34_PHY_BYTE_LANE_1_DRIVE_PAD_CTLr_BL_ND_IDLE_STRENGTH,
    DDR34_PHY_BYTE_LANE_1_DRIVE_PAD_CTLr_BL_PD_TERM_STRENGTH,
    DDR34_PHY_BYTE_LANE_1_DRIVE_PAD_CTLr_BL_ND_TERM_STRENGTH,
    DDR34_PHY_BYTE_LANE_1_DRIVE_PAD_CTLr_BL_PD_STRENGTH,
    DDR34_PHY_BYTE_LANE_1_DRIVE_PAD_CTLr_BL_ND_STRENGTH,
    DDR34_PHY_BYTE_LANE_1_RD_EN_DRIVE_PAD_CTLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_RD_EN_DRIVE_PAD_CTLr_EDC_RD_EN_PD_STRENGTH,
    DDR34_PHY_BYTE_LANE_1_RD_EN_DRIVE_PAD_CTLr_EDC_RD_EN_ND_STRENGTH,
    DDR34_PHY_BYTE_LANE_1_RD_EN_DRIVE_PAD_CTLr_BL_RD_EN_PD_STRENGTH,
    DDR34_PHY_BYTE_LANE_1_RD_EN_DRIVE_PAD_CTLr_BL_RD_EN_ND_STRENGTH,
    DDR34_PHY_BYTE_LANE_1_STATIC_PAD_CTLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_STATIC_PAD_CTLr_DQS_MODE,
    DDR34_PHY_BYTE_LANE_1_STATIC_PAD_CTLr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_STATIC_PAD_CTLr_EDC_MODE,
    DDR34_PHY_BYTE_LANE_1_STATIC_PAD_CTLr_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_STATIC_PAD_CTLr_WDBI_ENABLE,
    DDR34_PHY_BYTE_LANE_1_STATIC_PAD_CTLr_RDBI_ENABLE,
    DDR34_PHY_BYTE_LANE_1_STATIC_PAD_CTLr_DM_MODE,
    DDR34_PHY_BYTE_LANE_1_STATIC_PAD_CTLr_RESERVED3,
    DDR34_PHY_BYTE_LANE_1_STATIC_PAD_CTLr_DQS_TX_DIS,
    DDR34_PHY_BYTE_LANE_1_STATIC_PAD_CTLr_RESERVED4,
    DDR34_PHY_BYTE_LANE_1_STATIC_PAD_CTLr_RX_MODE,
    DDR34_PHY_BYTE_LANE_1_WR_PREAMBLE_MODEr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_WR_PREAMBLE_MODEr_DQ_POSTAM_BITS,
    DDR34_PHY_BYTE_LANE_1_WR_PREAMBLE_MODEr_DQ_PREAM_BITS,
    DDR34_PHY_BYTE_LANE_1_WR_PREAMBLE_MODEr_DQS,
    DDR34_PHY_BYTE_LANE_1_WR_PREAMBLE_MODEr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_WR_PREAMBLE_MODEr_DQS_POSTAM_BITS,
    DDR34_PHY_BYTE_LANE_1_WR_PREAMBLE_MODEr_DQS_PREAM_BITS,
    DDR34_PHY_BYTE_LANE_1_ODT_CONTROLr_RESERVED_for_padding0,
    DDR34_PHY_BYTE_LANE_1_ODT_CONTROLr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_ODT_CONTROLr_ODT_ENABLE,
    DDR34_PHY_BYTE_LANE_1_ODT_CONTROLr_ODT_DELAY,
    DDR34_PHY_BYTE_LANE_1_ODT_CONTROLr_ODT_POST_LENGTH,
    DDR34_PHY_BYTE_LANE_1_ODT_CONTROLr_ODT_PRE_LENGTH,
    DDR34_PHY_BYTE_LANE_1_EDC_DPD_CONTROLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_EDC_DPD_CONTROLr_DQS_PHASE_OVERRIDE_VALUE,
    DDR34_PHY_BYTE_LANE_1_EDC_DPD_CONTROLr_DQS_PHASE_OVERRIDE_ENABLE,
    DDR34_PHY_BYTE_LANE_1_EDC_DPD_CONTROLr_SET_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_EDC_DPD_CONTROLr_UPDATE,
    DDR34_PHY_BYTE_LANE_1_EDC_DPD_CONTROLr_MONITOR,
    DDR34_PHY_BYTE_LANE_1_EDC_DPD_CONTROLr_INIT,
    DDR34_PHY_BYTE_LANE_1_EDC_DPD_STATUSr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_EDC_DPD_STATUSr_CURRENT_DQS_PHASE,
    DDR34_PHY_BYTE_LANE_1_EDC_DPD_STATUSr_CURRENT_VDL_SETTING,
    DDR34_PHY_BYTE_LANE_1_EDC_DPD_STATUSr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_EDC_DPD_STATUSr_INIT_DQS_PHASE,
    DDR34_PHY_BYTE_LANE_1_EDC_DPD_STATUSr_INIT_VDL_SETTING,
    DDR34_PHY_BYTE_LANE_1_EDC_DPD_STATUSr_EDGE_ERROR,
    DDR34_PHY_BYTE_LANE_1_EDC_DPD_STATUSr_MONITOR_BUSY,
    DDR34_PHY_BYTE_LANE_1_EDC_DPD_STATUSr_INIT_BUSY,
    DDR34_PHY_BYTE_LANE_1_EDC_DPD_STATUSr_INIT_DONE,
    DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_CONTROLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_CONTROLr_LOWER_LIMIT,
    DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_CONTROLr_UPPER_LIMIT,
    DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_CONTROLr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_CONTROLr_ENABLE,
    DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_STATUSr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_STATUSr_VDL_SETTING,
    DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_STATUSr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_STATUSr_VALID,
    DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_STATUSr_CLEAR_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_STATUSr_CLEAR_CLEAR,
    DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_STATUS_CLEARr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_STATUS_CLEARr_CLEAR,
    DDR34_PHY_BYTE_LANE_1_EDC_CRC_CONTROLr_BUSY,
    DDR34_PHY_BYTE_LANE_1_EDC_CRC_CONTROLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_EDC_CRC_CONTROLr_FORCE,
    DDR34_PHY_BYTE_LANE_1_EDC_CRC_CONTROLr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_EDC_CRC_CONTROLr_CRCWL,
    DDR34_PHY_BYTE_LANE_1_EDC_CRC_CONTROLr_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_EDC_CRC_CONTROLr_CRCRL,
    DDR34_PHY_BYTE_LANE_1_EDC_CRC_CONTROLr_RESERVED3,
    DDR34_PHY_BYTE_LANE_1_EDC_CRC_CONTROLr_COUNT_MODE,
    DDR34_PHY_BYTE_LANE_1_EDC_CRC_CONTROLr_ENABLE_WR,
    DDR34_PHY_BYTE_LANE_1_EDC_CRC_CONTROLr_ENABLE_RD,
    DDR34_PHY_BYTE_LANE_1_EDC_CRC_STATUSr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_EDC_CRC_STATUSr_WR_FAIL,
    DDR34_PHY_BYTE_LANE_1_EDC_CRC_STATUSr_WR_PASS,
    DDR34_PHY_BYTE_LANE_1_EDC_CRC_STATUSr_RD_FAIL,
    DDR34_PHY_BYTE_LANE_1_EDC_CRC_STATUSr_RD_PASS,
    DDR34_PHY_BYTE_LANE_1_EDC_CRC_STATUSr_CLEAR_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_EDC_CRC_STATUSr_CLEAR_CLEAR,
    DDR34_PHY_BYTE_LANE_1_EDC_CRC_COUNTr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_EDC_CRC_COUNTr_WR_COUNT,
    DDR34_PHY_BYTE_LANE_1_EDC_CRC_COUNTr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_EDC_CRC_COUNTr_RD_COUNT,
    DDR34_PHY_BYTE_LANE_1_EDC_CRC_STATUS_CLEARr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_EDC_CRC_STATUS_CLEARr_CLEAR,
    DDR34_PHY_BYTE_LANE_1_BL_SPARE_REGr_RESERVED_for_eco0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_Pr_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_Pr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_Pr_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_Pr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_Pr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_Pr_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_Pr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_Nr_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_Nr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_Nr_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_Nr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_Nr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_Nr_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_Nr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ0r_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ0r_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ0r_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ0r_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ0r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ0r_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ0r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ1r_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ1r_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ1r_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ1r_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ1r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ1r_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ1r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ2r_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ2r_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ2r_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ2r_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ2r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ2r_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ2r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ3r_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ3r_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ3r_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ3r_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ3r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ3r_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ3r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ4r_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ4r_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ4r_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ4r_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ4r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ4r_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ4r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ5r_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ5r_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ5r_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ5r_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ5r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ5r_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ5r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ6r_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ6r_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ6r_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ6r_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ6r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ6r_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ6r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ7r_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ7r_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ7r_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ7r_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ7r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ7r_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ7r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DMr_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DMr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DMr_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DMr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DMr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DMr_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DMr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_EDCr_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_EDCr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_EDCr_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_EDCr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_EDCr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_EDCr_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_EDCr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSPr_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSPr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSPr_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSPr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSPr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSPr_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSPr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSNr_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSNr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSNr_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSNr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSNr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSNr_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSNr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0Pr_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0Pr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0Pr_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0Pr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0Pr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0Pr_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0Pr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0Nr_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0Nr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0Nr_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0Nr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0Nr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0Nr_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0Nr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1Pr_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1Pr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1Pr_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1Pr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1Pr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1Pr_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1Pr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1Nr_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1Nr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1Nr_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1Nr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1Nr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1Nr_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1Nr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2Pr_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2Pr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2Pr_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2Pr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2Pr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2Pr_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2Pr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2Nr_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2Nr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2Nr_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2Nr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2Nr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2Nr_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2Nr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3Pr_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3Pr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3Pr_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3Pr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3Pr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3Pr_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3Pr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3Nr_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3Nr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3Nr_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3Nr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3Nr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3Nr_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3Nr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4Pr_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4Pr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4Pr_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4Pr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4Pr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4Pr_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4Pr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4Nr_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4Nr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4Nr_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4Nr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4Nr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4Nr_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4Nr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5Pr_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5Pr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5Pr_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5Pr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5Pr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5Pr_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5Pr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5Nr_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5Nr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5Nr_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5Nr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5Nr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5Nr_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5Nr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6Pr_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6Pr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6Pr_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6Pr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6Pr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6Pr_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6Pr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6Nr_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6Nr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6Nr_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6Nr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6Nr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6Nr_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6Nr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7Pr_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7Pr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7Pr_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7Pr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7Pr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7Pr_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7Pr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7Nr_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7Nr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7Nr_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7Nr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7Nr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7Nr_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7Nr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMPr_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMPr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMPr_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMPr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMPr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMPr_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMPr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMNr_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMNr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMNr_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMNr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMNr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMNr_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMNr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCPr_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCPr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCPr_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCPr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCPr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCPr_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCPr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCNr_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCNr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCNr_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCNr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCNr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCNr_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCNr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS0r_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS0r_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS0r_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS0r_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS0r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS0r_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS0r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS1r_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS1r_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS1r_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS1r_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS1r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS1r_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS1r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CLK_CONTROLr_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CLK_CONTROLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CLK_CONTROLr_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CLK_CONTROLr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CLK_CONTROLr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CLK_CONTROLr_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CLK_CONTROLr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_LDE_CONTROLr_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_LDE_CONTROLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_LDE_CONTROLr_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_LDE_CONTROLr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_LDE_CONTROLr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_LDE_CONTROLr_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_LDE_CONTROLr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_RD_EN_DLY_CYCr_BUSY,
    DDR34_PHY_BYTE_LANE_2_RD_EN_DLY_CYCr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_RD_EN_DLY_CYCr_FORCE,
    DDR34_PHY_BYTE_LANE_2_RD_EN_DLY_CYCr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_RD_EN_DLY_CYCr_CS1_CYCLES,
    DDR34_PHY_BYTE_LANE_2_RD_EN_DLY_CYCr_CS0_CYCLES,
    DDR34_PHY_BYTE_LANE_2_WR_CHAN_DLY_CYCr_BUSY,
    DDR34_PHY_BYTE_LANE_2_WR_CHAN_DLY_CYCr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_WR_CHAN_DLY_CYCr_FORCE,
    DDR34_PHY_BYTE_LANE_2_WR_CHAN_DLY_CYCr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_WR_CHAN_DLY_CYCr_CYCLES,
    DDR34_PHY_BYTE_LANE_2_READ_CONTROLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_READ_CONTROLr_MODE,
    DDR34_PHY_BYTE_LANE_2_READ_CONTROLr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_READ_CONTROLr_RD_DATA_DLY,
    DDR34_PHY_BYTE_LANE_2_READ_FIFO_ADDRr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_READ_FIFO_ADDRr_ADDR,
    DDR34_PHY_BYTE_LANE_2_READ_FIFO_DATAr_DATA,
    DDR34_PHY_BYTE_LANE_2_READ_FIFO_DM_DBIr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_READ_FIFO_DM_DBIr_DM_DBI,
    DDR34_PHY_BYTE_LANE_2_READ_FIFO_STATUSr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_READ_FIFO_STATUSr_OVERFLOW,
    DDR34_PHY_BYTE_LANE_2_READ_FIFO_STATUSr_UNDERFLOW,
    DDR34_PHY_BYTE_LANE_2_READ_FIFO_CLEARr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_READ_FIFO_CLEARr_CLEAR,
    DDR34_PHY_BYTE_LANE_2_IDLE_PAD_CONTROLr_IDLE,
    DDR34_PHY_BYTE_LANE_2_IDLE_PAD_CONTROLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_IDLE_PAD_CONTROLr_AUTO_DQ_RXENB_MODE,
    DDR34_PHY_BYTE_LANE_2_IDLE_PAD_CONTROLr_AUTO_DQ_IDDQ_MODE,
    DDR34_PHY_BYTE_LANE_2_IDLE_PAD_CONTROLr_RESERVED_for_padding1,
    DDR34_PHY_BYTE_LANE_2_IDLE_PAD_CONTROLr_IO_IDLE_ENABLE,
    DDR34_PHY_BYTE_LANE_2_IDLE_PAD_CONTROLr_RXENB,
    DDR34_PHY_BYTE_LANE_2_IDLE_PAD_CONTROLr_IDDQ,
    DDR34_PHY_BYTE_LANE_2_IDLE_PAD_CONTROLr_DOUT_N,
    DDR34_PHY_BYTE_LANE_2_IDLE_PAD_CONTROLr_DOUT_P,
    DDR34_PHY_BYTE_LANE_2_DRIVE_PAD_CTLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_DRIVE_PAD_CTLr_BL_PD_IDLE_STRENGTH,
    DDR34_PHY_BYTE_LANE_2_DRIVE_PAD_CTLr_BL_ND_IDLE_STRENGTH,
    DDR34_PHY_BYTE_LANE_2_DRIVE_PAD_CTLr_BL_PD_TERM_STRENGTH,
    DDR34_PHY_BYTE_LANE_2_DRIVE_PAD_CTLr_BL_ND_TERM_STRENGTH,
    DDR34_PHY_BYTE_LANE_2_DRIVE_PAD_CTLr_BL_PD_STRENGTH,
    DDR34_PHY_BYTE_LANE_2_DRIVE_PAD_CTLr_BL_ND_STRENGTH,
    DDR34_PHY_BYTE_LANE_2_RD_EN_DRIVE_PAD_CTLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_RD_EN_DRIVE_PAD_CTLr_EDC_RD_EN_PD_STRENGTH,
    DDR34_PHY_BYTE_LANE_2_RD_EN_DRIVE_PAD_CTLr_EDC_RD_EN_ND_STRENGTH,
    DDR34_PHY_BYTE_LANE_2_RD_EN_DRIVE_PAD_CTLr_BL_RD_EN_PD_STRENGTH,
    DDR34_PHY_BYTE_LANE_2_RD_EN_DRIVE_PAD_CTLr_BL_RD_EN_ND_STRENGTH,
    DDR34_PHY_BYTE_LANE_2_STATIC_PAD_CTLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_STATIC_PAD_CTLr_DQS_MODE,
    DDR34_PHY_BYTE_LANE_2_STATIC_PAD_CTLr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_STATIC_PAD_CTLr_EDC_MODE,
    DDR34_PHY_BYTE_LANE_2_STATIC_PAD_CTLr_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_STATIC_PAD_CTLr_WDBI_ENABLE,
    DDR34_PHY_BYTE_LANE_2_STATIC_PAD_CTLr_RDBI_ENABLE,
    DDR34_PHY_BYTE_LANE_2_STATIC_PAD_CTLr_DM_MODE,
    DDR34_PHY_BYTE_LANE_2_STATIC_PAD_CTLr_RESERVED3,
    DDR34_PHY_BYTE_LANE_2_STATIC_PAD_CTLr_DQS_TX_DIS,
    DDR34_PHY_BYTE_LANE_2_STATIC_PAD_CTLr_RESERVED4,
    DDR34_PHY_BYTE_LANE_2_STATIC_PAD_CTLr_RX_MODE,
    DDR34_PHY_BYTE_LANE_2_WR_PREAMBLE_MODEr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_WR_PREAMBLE_MODEr_DQ_POSTAM_BITS,
    DDR34_PHY_BYTE_LANE_2_WR_PREAMBLE_MODEr_DQ_PREAM_BITS,
    DDR34_PHY_BYTE_LANE_2_WR_PREAMBLE_MODEr_DQS,
    DDR34_PHY_BYTE_LANE_2_WR_PREAMBLE_MODEr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_WR_PREAMBLE_MODEr_DQS_POSTAM_BITS,
    DDR34_PHY_BYTE_LANE_2_WR_PREAMBLE_MODEr_DQS_PREAM_BITS,
    DDR34_PHY_BYTE_LANE_2_ODT_CONTROLr_RESERVED_for_padding0,
    DDR34_PHY_BYTE_LANE_2_ODT_CONTROLr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_ODT_CONTROLr_ODT_ENABLE,
    DDR34_PHY_BYTE_LANE_2_ODT_CONTROLr_ODT_DELAY,
    DDR34_PHY_BYTE_LANE_2_ODT_CONTROLr_ODT_POST_LENGTH,
    DDR34_PHY_BYTE_LANE_2_ODT_CONTROLr_ODT_PRE_LENGTH,
    DDR34_PHY_BYTE_LANE_2_EDC_DPD_CONTROLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_EDC_DPD_CONTROLr_DQS_PHASE_OVERRIDE_VALUE,
    DDR34_PHY_BYTE_LANE_2_EDC_DPD_CONTROLr_DQS_PHASE_OVERRIDE_ENABLE,
    DDR34_PHY_BYTE_LANE_2_EDC_DPD_CONTROLr_SET_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_EDC_DPD_CONTROLr_UPDATE,
    DDR34_PHY_BYTE_LANE_2_EDC_DPD_CONTROLr_MONITOR,
    DDR34_PHY_BYTE_LANE_2_EDC_DPD_CONTROLr_INIT,
    DDR34_PHY_BYTE_LANE_2_EDC_DPD_STATUSr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_EDC_DPD_STATUSr_CURRENT_DQS_PHASE,
    DDR34_PHY_BYTE_LANE_2_EDC_DPD_STATUSr_CURRENT_VDL_SETTING,
    DDR34_PHY_BYTE_LANE_2_EDC_DPD_STATUSr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_EDC_DPD_STATUSr_INIT_DQS_PHASE,
    DDR34_PHY_BYTE_LANE_2_EDC_DPD_STATUSr_INIT_VDL_SETTING,
    DDR34_PHY_BYTE_LANE_2_EDC_DPD_STATUSr_EDGE_ERROR,
    DDR34_PHY_BYTE_LANE_2_EDC_DPD_STATUSr_MONITOR_BUSY,
    DDR34_PHY_BYTE_LANE_2_EDC_DPD_STATUSr_INIT_BUSY,
    DDR34_PHY_BYTE_LANE_2_EDC_DPD_STATUSr_INIT_DONE,
    DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_CONTROLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_CONTROLr_LOWER_LIMIT,
    DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_CONTROLr_UPPER_LIMIT,
    DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_CONTROLr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_CONTROLr_ENABLE,
    DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_STATUSr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_STATUSr_VDL_SETTING,
    DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_STATUSr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_STATUSr_VALID,
    DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_STATUSr_CLEAR_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_STATUSr_CLEAR_CLEAR,
    DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_STATUS_CLEARr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_STATUS_CLEARr_CLEAR,
    DDR34_PHY_BYTE_LANE_2_EDC_CRC_CONTROLr_BUSY,
    DDR34_PHY_BYTE_LANE_2_EDC_CRC_CONTROLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_EDC_CRC_CONTROLr_FORCE,
    DDR34_PHY_BYTE_LANE_2_EDC_CRC_CONTROLr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_EDC_CRC_CONTROLr_CRCWL,
    DDR34_PHY_BYTE_LANE_2_EDC_CRC_CONTROLr_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_EDC_CRC_CONTROLr_CRCRL,
    DDR34_PHY_BYTE_LANE_2_EDC_CRC_CONTROLr_RESERVED3,
    DDR34_PHY_BYTE_LANE_2_EDC_CRC_CONTROLr_COUNT_MODE,
    DDR34_PHY_BYTE_LANE_2_EDC_CRC_CONTROLr_ENABLE_WR,
    DDR34_PHY_BYTE_LANE_2_EDC_CRC_CONTROLr_ENABLE_RD,
    DDR34_PHY_BYTE_LANE_2_EDC_CRC_STATUSr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_EDC_CRC_STATUSr_WR_FAIL,
    DDR34_PHY_BYTE_LANE_2_EDC_CRC_STATUSr_WR_PASS,
    DDR34_PHY_BYTE_LANE_2_EDC_CRC_STATUSr_RD_FAIL,
    DDR34_PHY_BYTE_LANE_2_EDC_CRC_STATUSr_RD_PASS,
    DDR34_PHY_BYTE_LANE_2_EDC_CRC_STATUSr_CLEAR_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_EDC_CRC_STATUSr_CLEAR_CLEAR,
    DDR34_PHY_BYTE_LANE_2_EDC_CRC_COUNTr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_EDC_CRC_COUNTr_WR_COUNT,
    DDR34_PHY_BYTE_LANE_2_EDC_CRC_COUNTr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_EDC_CRC_COUNTr_RD_COUNT,
    DDR34_PHY_BYTE_LANE_2_EDC_CRC_STATUS_CLEARr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_EDC_CRC_STATUS_CLEARr_CLEAR,
    DDR34_PHY_BYTE_LANE_2_BL_SPARE_REGr_RESERVED_for_eco0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_Pr_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_Pr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_Pr_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_Pr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_Pr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_Pr_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_Pr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_Nr_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_Nr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_Nr_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_Nr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_Nr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_Nr_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_Nr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ0r_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ0r_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ0r_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ0r_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ0r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ0r_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ0r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ1r_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ1r_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ1r_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ1r_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ1r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ1r_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ1r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ2r_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ2r_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ2r_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ2r_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ2r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ2r_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ2r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ3r_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ3r_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ3r_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ3r_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ3r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ3r_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ3r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ4r_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ4r_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ4r_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ4r_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ4r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ4r_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ4r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ5r_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ5r_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ5r_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ5r_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ5r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ5r_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ5r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ6r_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ6r_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ6r_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ6r_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ6r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ6r_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ6r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ7r_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ7r_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ7r_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ7r_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ7r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ7r_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ7r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DMr_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DMr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DMr_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DMr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DMr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DMr_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DMr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_EDCr_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_EDCr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_EDCr_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_EDCr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_EDCr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_EDCr_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_EDCr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSPr_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSPr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSPr_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSPr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSPr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSPr_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSPr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSNr_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSNr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSNr_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSNr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSNr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSNr_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSNr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0Pr_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0Pr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0Pr_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0Pr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0Pr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0Pr_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0Pr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0Nr_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0Nr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0Nr_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0Nr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0Nr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0Nr_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0Nr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1Pr_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1Pr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1Pr_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1Pr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1Pr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1Pr_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1Pr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1Nr_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1Nr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1Nr_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1Nr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1Nr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1Nr_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1Nr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2Pr_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2Pr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2Pr_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2Pr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2Pr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2Pr_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2Pr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2Nr_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2Nr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2Nr_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2Nr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2Nr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2Nr_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2Nr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3Pr_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3Pr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3Pr_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3Pr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3Pr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3Pr_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3Pr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3Nr_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3Nr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3Nr_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3Nr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3Nr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3Nr_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3Nr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4Pr_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4Pr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4Pr_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4Pr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4Pr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4Pr_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4Pr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4Nr_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4Nr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4Nr_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4Nr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4Nr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4Nr_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4Nr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5Pr_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5Pr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5Pr_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5Pr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5Pr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5Pr_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5Pr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5Nr_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5Nr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5Nr_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5Nr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5Nr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5Nr_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5Nr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6Pr_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6Pr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6Pr_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6Pr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6Pr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6Pr_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6Pr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6Nr_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6Nr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6Nr_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6Nr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6Nr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6Nr_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6Nr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7Pr_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7Pr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7Pr_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7Pr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7Pr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7Pr_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7Pr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7Nr_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7Nr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7Nr_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7Nr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7Nr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7Nr_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7Nr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMPr_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMPr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMPr_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMPr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMPr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMPr_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMPr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMNr_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMNr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMNr_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMNr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMNr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMNr_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMNr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCPr_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCPr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCPr_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCPr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCPr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCPr_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCPr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCNr_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCNr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCNr_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCNr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCNr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCNr_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCNr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS0r_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS0r_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS0r_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS0r_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS0r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS0r_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS0r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS1r_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS1r_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS1r_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS1r_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS1r_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS1r_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS1r_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CLK_CONTROLr_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CLK_CONTROLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CLK_CONTROLr_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CLK_CONTROLr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CLK_CONTROLr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CLK_CONTROLr_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CLK_CONTROLr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_LDE_CONTROLr_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_LDE_CONTROLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_LDE_CONTROLr_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_LDE_CONTROLr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_LDE_CONTROLr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_LDE_CONTROLr_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_LDE_CONTROLr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_RD_EN_DLY_CYCr_BUSY,
    DDR34_PHY_BYTE_LANE_3_RD_EN_DLY_CYCr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_RD_EN_DLY_CYCr_FORCE,
    DDR34_PHY_BYTE_LANE_3_RD_EN_DLY_CYCr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_RD_EN_DLY_CYCr_CS1_CYCLES,
    DDR34_PHY_BYTE_LANE_3_RD_EN_DLY_CYCr_CS0_CYCLES,
    DDR34_PHY_BYTE_LANE_3_WR_CHAN_DLY_CYCr_BUSY,
    DDR34_PHY_BYTE_LANE_3_WR_CHAN_DLY_CYCr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_WR_CHAN_DLY_CYCr_FORCE,
    DDR34_PHY_BYTE_LANE_3_WR_CHAN_DLY_CYCr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_WR_CHAN_DLY_CYCr_CYCLES,
    DDR34_PHY_BYTE_LANE_3_READ_CONTROLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_READ_CONTROLr_MODE,
    DDR34_PHY_BYTE_LANE_3_READ_CONTROLr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_READ_CONTROLr_RD_DATA_DLY,
    DDR34_PHY_BYTE_LANE_3_READ_FIFO_ADDRr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_READ_FIFO_ADDRr_ADDR,
    DDR34_PHY_BYTE_LANE_3_READ_FIFO_DATAr_DATA,
    DDR34_PHY_BYTE_LANE_3_READ_FIFO_DM_DBIr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_READ_FIFO_DM_DBIr_DM_DBI,
    DDR34_PHY_BYTE_LANE_3_READ_FIFO_STATUSr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_READ_FIFO_STATUSr_OVERFLOW,
    DDR34_PHY_BYTE_LANE_3_READ_FIFO_STATUSr_UNDERFLOW,
    DDR34_PHY_BYTE_LANE_3_READ_FIFO_CLEARr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_READ_FIFO_CLEARr_CLEAR,
    DDR34_PHY_BYTE_LANE_3_IDLE_PAD_CONTROLr_IDLE,
    DDR34_PHY_BYTE_LANE_3_IDLE_PAD_CONTROLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_IDLE_PAD_CONTROLr_AUTO_DQ_RXENB_MODE,
    DDR34_PHY_BYTE_LANE_3_IDLE_PAD_CONTROLr_AUTO_DQ_IDDQ_MODE,
    DDR34_PHY_BYTE_LANE_3_IDLE_PAD_CONTROLr_RESERVED_for_padding1,
    DDR34_PHY_BYTE_LANE_3_IDLE_PAD_CONTROLr_IO_IDLE_ENABLE,
    DDR34_PHY_BYTE_LANE_3_IDLE_PAD_CONTROLr_RXENB,
    DDR34_PHY_BYTE_LANE_3_IDLE_PAD_CONTROLr_IDDQ,
    DDR34_PHY_BYTE_LANE_3_IDLE_PAD_CONTROLr_DOUT_N,
    DDR34_PHY_BYTE_LANE_3_IDLE_PAD_CONTROLr_DOUT_P,
    DDR34_PHY_BYTE_LANE_3_DRIVE_PAD_CTLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_DRIVE_PAD_CTLr_BL_PD_IDLE_STRENGTH,
    DDR34_PHY_BYTE_LANE_3_DRIVE_PAD_CTLr_BL_ND_IDLE_STRENGTH,
    DDR34_PHY_BYTE_LANE_3_DRIVE_PAD_CTLr_BL_PD_TERM_STRENGTH,
    DDR34_PHY_BYTE_LANE_3_DRIVE_PAD_CTLr_BL_ND_TERM_STRENGTH,
    DDR34_PHY_BYTE_LANE_3_DRIVE_PAD_CTLr_BL_PD_STRENGTH,
    DDR34_PHY_BYTE_LANE_3_DRIVE_PAD_CTLr_BL_ND_STRENGTH,
    DDR34_PHY_BYTE_LANE_3_RD_EN_DRIVE_PAD_CTLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_RD_EN_DRIVE_PAD_CTLr_EDC_RD_EN_PD_STRENGTH,
    DDR34_PHY_BYTE_LANE_3_RD_EN_DRIVE_PAD_CTLr_EDC_RD_EN_ND_STRENGTH,
    DDR34_PHY_BYTE_LANE_3_RD_EN_DRIVE_PAD_CTLr_BL_RD_EN_PD_STRENGTH,
    DDR34_PHY_BYTE_LANE_3_RD_EN_DRIVE_PAD_CTLr_BL_RD_EN_ND_STRENGTH,
    DDR34_PHY_BYTE_LANE_3_STATIC_PAD_CTLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_STATIC_PAD_CTLr_DQS_MODE,
    DDR34_PHY_BYTE_LANE_3_STATIC_PAD_CTLr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_STATIC_PAD_CTLr_EDC_MODE,
    DDR34_PHY_BYTE_LANE_3_STATIC_PAD_CTLr_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_STATIC_PAD_CTLr_WDBI_ENABLE,
    DDR34_PHY_BYTE_LANE_3_STATIC_PAD_CTLr_RDBI_ENABLE,
    DDR34_PHY_BYTE_LANE_3_STATIC_PAD_CTLr_DM_MODE,
    DDR34_PHY_BYTE_LANE_3_STATIC_PAD_CTLr_RESERVED3,
    DDR34_PHY_BYTE_LANE_3_STATIC_PAD_CTLr_DQS_TX_DIS,
    DDR34_PHY_BYTE_LANE_3_STATIC_PAD_CTLr_RESERVED4,
    DDR34_PHY_BYTE_LANE_3_STATIC_PAD_CTLr_RX_MODE,
    DDR34_PHY_BYTE_LANE_3_WR_PREAMBLE_MODEr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_WR_PREAMBLE_MODEr_DQ_POSTAM_BITS,
    DDR34_PHY_BYTE_LANE_3_WR_PREAMBLE_MODEr_DQ_PREAM_BITS,
    DDR34_PHY_BYTE_LANE_3_WR_PREAMBLE_MODEr_DQS,
    DDR34_PHY_BYTE_LANE_3_WR_PREAMBLE_MODEr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_WR_PREAMBLE_MODEr_DQS_POSTAM_BITS,
    DDR34_PHY_BYTE_LANE_3_WR_PREAMBLE_MODEr_DQS_PREAM_BITS,
    DDR34_PHY_BYTE_LANE_3_ODT_CONTROLr_RESERVED_for_padding0,
    DDR34_PHY_BYTE_LANE_3_ODT_CONTROLr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_ODT_CONTROLr_ODT_ENABLE,
    DDR34_PHY_BYTE_LANE_3_ODT_CONTROLr_ODT_DELAY,
    DDR34_PHY_BYTE_LANE_3_ODT_CONTROLr_ODT_POST_LENGTH,
    DDR34_PHY_BYTE_LANE_3_ODT_CONTROLr_ODT_PRE_LENGTH,
    DDR34_PHY_BYTE_LANE_3_EDC_DPD_CONTROLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_EDC_DPD_CONTROLr_DQS_PHASE_OVERRIDE_VALUE,
    DDR34_PHY_BYTE_LANE_3_EDC_DPD_CONTROLr_DQS_PHASE_OVERRIDE_ENABLE,
    DDR34_PHY_BYTE_LANE_3_EDC_DPD_CONTROLr_SET_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_EDC_DPD_CONTROLr_UPDATE,
    DDR34_PHY_BYTE_LANE_3_EDC_DPD_CONTROLr_MONITOR,
    DDR34_PHY_BYTE_LANE_3_EDC_DPD_CONTROLr_INIT,
    DDR34_PHY_BYTE_LANE_3_EDC_DPD_STATUSr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_EDC_DPD_STATUSr_CURRENT_DQS_PHASE,
    DDR34_PHY_BYTE_LANE_3_EDC_DPD_STATUSr_CURRENT_VDL_SETTING,
    DDR34_PHY_BYTE_LANE_3_EDC_DPD_STATUSr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_EDC_DPD_STATUSr_INIT_DQS_PHASE,
    DDR34_PHY_BYTE_LANE_3_EDC_DPD_STATUSr_INIT_VDL_SETTING,
    DDR34_PHY_BYTE_LANE_3_EDC_DPD_STATUSr_EDGE_ERROR,
    DDR34_PHY_BYTE_LANE_3_EDC_DPD_STATUSr_MONITOR_BUSY,
    DDR34_PHY_BYTE_LANE_3_EDC_DPD_STATUSr_INIT_BUSY,
    DDR34_PHY_BYTE_LANE_3_EDC_DPD_STATUSr_INIT_DONE,
    DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_CONTROLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_CONTROLr_LOWER_LIMIT,
    DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_CONTROLr_UPPER_LIMIT,
    DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_CONTROLr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_CONTROLr_ENABLE,
    DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_STATUSr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_STATUSr_VDL_SETTING,
    DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_STATUSr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_STATUSr_VALID,
    DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_STATUSr_CLEAR_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_STATUSr_CLEAR_CLEAR,
    DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_STATUS_CLEARr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_STATUS_CLEARr_CLEAR,
    DDR34_PHY_BYTE_LANE_3_EDC_CRC_CONTROLr_BUSY,
    DDR34_PHY_BYTE_LANE_3_EDC_CRC_CONTROLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_EDC_CRC_CONTROLr_FORCE,
    DDR34_PHY_BYTE_LANE_3_EDC_CRC_CONTROLr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_EDC_CRC_CONTROLr_CRCWL,
    DDR34_PHY_BYTE_LANE_3_EDC_CRC_CONTROLr_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_EDC_CRC_CONTROLr_CRCRL,
    DDR34_PHY_BYTE_LANE_3_EDC_CRC_CONTROLr_RESERVED3,
    DDR34_PHY_BYTE_LANE_3_EDC_CRC_CONTROLr_COUNT_MODE,
    DDR34_PHY_BYTE_LANE_3_EDC_CRC_CONTROLr_ENABLE_WR,
    DDR34_PHY_BYTE_LANE_3_EDC_CRC_CONTROLr_ENABLE_RD,
    DDR34_PHY_BYTE_LANE_3_EDC_CRC_STATUSr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_EDC_CRC_STATUSr_WR_FAIL,
    DDR34_PHY_BYTE_LANE_3_EDC_CRC_STATUSr_WR_PASS,
    DDR34_PHY_BYTE_LANE_3_EDC_CRC_STATUSr_RD_FAIL,
    DDR34_PHY_BYTE_LANE_3_EDC_CRC_STATUSr_RD_PASS,
    DDR34_PHY_BYTE_LANE_3_EDC_CRC_STATUSr_CLEAR_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_EDC_CRC_STATUSr_CLEAR_CLEAR,
    DDR34_PHY_BYTE_LANE_3_EDC_CRC_COUNTr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_EDC_CRC_COUNTr_WR_COUNT,
    DDR34_PHY_BYTE_LANE_3_EDC_CRC_COUNTr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_EDC_CRC_COUNTr_RD_COUNT,
    DDR34_PHY_BYTE_LANE_3_EDC_CRC_STATUS_CLEARr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_EDC_CRC_STATUS_CLEARr_CLEAR,
    DDR34_PHY_BYTE_LANE_3_BL_SPARE_REGr_RESERVED_for_eco0,
    DDR34_PHY_CONTROL_REGS_PLL_CONFIGr_PLL_LDO_PWRDOWN,
    DDR34_PHY_CONTROL_REGS_PLL_CONFIGr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_PLL_CONFIGr_ENABLE_EXT_CTRL,
    DDR34_PHY_CONTROL_REGS_PLL_CONFIGr_ISO_OUT,
    DDR34_PHY_CONTROL_REGS_PLL_CONFIGr_ISO_IN,
    DDR34_PHY_CONTROL_REGS_AUX_CONTROLr_AUX0_IS_ODT,
    DDR34_PHY_CONTROL_REGS_AUX_CONTROLr_PAR_IS_ODT,
    DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTLr_2T_RESERVED0,
    DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTLr_2T_PD_IDLE_STRENGTH,
    DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTLr_2T_ND_IDLE_STRENGTH,
    DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTLr_2T_PD_TERM_STRENGTH,
    DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTLr_2T_ND_TERM_STRENGTH,
    DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTLr_2T_PD_STRENGTH,
    DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTLr_2T_ND_STRENGTH,
    DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTL_2Tr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTL_2Tr_PD_IDLE_STRENGTH,
    DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTL_2Tr_ND_IDLE_STRENGTH,
    DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTL_2Tr_PD_TERM_STRENGTH,
    DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTL_2Tr_ND_TERM_STRENGTH,
    DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTL_2Tr_PD_STRENGTH,
    DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTL_2Tr_ND_STRENGTH,
    DDR34_PHY_CONTROL_REGS_DRAM_CONFIGr_DDR4_GEARDOWN_ENABLE,
    DDR34_PHY_CONTROL_REGS_DRAM_CONFIGr_RESERVED2,
    DDR34_PHY_CONTROL_REGS_DRAM_TIMING4r_TEMP,
    DDR34_PHY_CONTROL_REGS_VDL_CALIBRATEr_CALIB_FAST,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_STATUSr_ERROR,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_STATUSr_PAUSED,
    DDR34_PHY_CONTROL_REGS_VDL_MONITOR_STATUSr_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CKr_BUSY,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CKr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CKr_FORCE,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CKr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CKr_ADJ_EN,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CKr_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CKr_VDL_STEP,
    DDR34_PHY_CONTROL_REGS_CMD_CYC_DLYr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_CMD_CYC_DLYr_CKE,
    DDR34_PHY_CONTROL_REGS_CMD_CYC_DLYr_WE_N,
    DDR34_PHY_CONTROL_REGS_CMD_CYC_DLYr_CAS_N,
    DDR34_PHY_CONTROL_REGS_CMD_CYC_DLYr_RAS_N,
    DDR34_PHY_CONTROL_REGS_CMD_CYC_DLYr_CS_N,
    DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG1r_AD16,
    DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG2r_AD16,
    DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG3r_AD16,
    DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG4r_AD16,
    DDR34_PHY_CONTROL_REGS_ALERT_STATUSr_BL3_STATE,
    DDR34_PHY_CONTROL_REGS_ALERT_STATUSr_BL2_STATE,
    DDR34_PHY_CONTROL_REGS_ALERT_STATUSr_BL1_STATE,
    DDR34_PHY_CONTROL_REGS_ALERT_STATUSr_BL0_STATE,
    DDR34_PHY_CONTROL_REGS_ALERT_STATUSr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_ALERT_STATUSr_BL3_ALERT,
    DDR34_PHY_CONTROL_REGS_ALERT_STATUSr_BL2_ALERT,
    DDR34_PHY_CONTROL_REGS_ALERT_STATUSr_BL1_ALERT,
    DDR34_PHY_CONTROL_REGS_ALERT_STATUSr_BL0_ALERT,
    DDR34_PHY_CONTROL_REGS_ALERT_DFIr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_ALERT_DFIr_MASK,
    DDR34_PHY_CONTROL_REGS_READ_ENABLE_CONTROLr_RESERVED2,
    DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROLr_RESERVED_FOR_ECO1,
    DDR34_PHY_CONTROL_REGS_PHYBIST_CNTRLr_PATTERN_SEL,
    DDR34_PHY_CONTROL_REGS_PHYBIST_CA_MASKr_MASK,
    DDR34_PHY_CONTROL_REGS_PHYBIST_DQ_MASK1r_MASK,
    DDR34_PHY_CONTROL_REGS_PHYBIST_DQ_MASK2r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_PHYBIST_DQ_MASK2r_ALERT_MASK,
    DDR34_PHY_CONTROL_REGS_PHYBIST_DQ_MASK2r_RESERVED1,
    DDR34_PHY_CONTROL_REGS_PHYBIST_DQ_MASK2r_DM_MASK,
    DDR34_PHY_CONTROL_REGS_PHYBIST_DQ_MASK2r_RESERVED2,
    DDR34_PHY_CONTROL_REGS_PHYBIST_BL0_STATUSr_ALERT,
    DDR34_PHY_CONTROL_REGS_PHYBIST_BL1_STATUSr_ALERT,
    DDR34_PHY_CONTROL_REGS_PHYBIST_BL2_STATUSr_ALERT,
    DDR34_PHY_CONTROL_REGS_PHYBIST_BL3_STATUSr_ALERT,
    DDR34_PHY_CONTROL_REGS_PHYBIST_BL4_STATUSr_ALERT,
    DDR34_PHY_CONTROL_REGS_STANDBY_CONTROLr_PHY_STANDBY,
    DDR34_PHY_CONTROL_REGS_STANDBY_CONTROLr_FORCE_CKE_RST_N,
    DDR34_PHY_CONTROL_REGS_STANDBY_CONTROLr_BIT_PIPE_RESET,
    DDR34_PHY_CONTROL_REGS_DFI_CNTRLr_SELF_REFRESH,
    DDR34_PHY_CONTROL_REGS_DFI_CNTRLr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_WRITE_ODT_CNTRLr_ODT1_FORCE_VALUE,
    DDR34_PHY_CONTROL_REGS_WRITE_ODT_CNTRLr_ODT1_FORCE,
    DDR34_PHY_CONTROL_REGS_WRITE_ODT_CNTRLr_ODT1_ENABLE,
    DDR34_PHY_CONTROL_REGS_WRITE_ODT_CNTRLr_ODT1_LENGTH,
    DDR34_PHY_CONTROL_REGS_WRITE_ODT_CNTRLr_ODT1_DELAY,
    DDR34_PHY_CONTROL_REGS_WRITE_ODT_CNTRLr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_WRITE_ODT_CNTRLr_ODT0_FORCE_VALUE,
    DDR34_PHY_CONTROL_REGS_WRITE_ODT_CNTRLr_ODT0_FORCE,
    DDR34_PHY_CONTROL_REGS_WRITE_ODT_CNTRLr_ODT0_ENABLE,
    DDR34_PHY_CONTROL_REGS_WRITE_ODT_CNTRLr_ODT0_LENGTH,
    DDR34_PHY_CONTROL_REGS_WRITE_ODT_CNTRLr_ODT0_DELAY,
    DDR34_PHY_CONTROL_REGS_ABI_PAR_CNTRLr_PAR_ERROR,
    DDR34_PHY_CONTROL_REGS_ZQ_CALr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_ZQ_CAL2r_RESERVED0,
    DDR34_PHY_CONTROL_REGS_ZQ_CAL2r_ZQ_CAL_RESET,
    DDR34_PHY_CONTROL_REGS_ZQ_CAL2r_ZQ_CAL_ERROR_N,
    DDR34_PHY_CONTROL_REGS_ZQ_CAL2r_ZQ_CAL_ERROR_P,
    DDR34_PHY_CONTROL_REGS_ZQ_CAL2r_ZQ_NSETTING,
    DDR34_PHY_CONTROL_REGS_ZQ_CAL2r_ZQ_PSETTING,
    DDR34_PHY_CONTROL_REGS_ZQ_CAL2r_ZQ_CAL_OP_COMPLETE_N,
    DDR34_PHY_CONTROL_REGS_ZQ_CAL2r_ZQ_CAL_OP_COMPLETE_P,
    DDR34_PHY_CONTROL_REGS_ZQ_CAL2r_ZQ_CAL_START_N,
    DDR34_PHY_CONTROL_REGS_ZQ_CAL2r_ZQ_CAL_START_P,
    DDR34_PHY_CONTROL_REGS_RO_PROC_MON_CTLr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_RO_PROC_MON_CTLr_BUSY,
    DDR34_PHY_CONTROL_REGS_RO_PROC_MON_CTLr_TRIGGER,
    DDR34_PHY_CONTROL_REGS_RO_PROC_MON_CTLr_RESET,
    DDR34_PHY_CONTROL_REGS_RO_PROC_MON_CTLr_SEL,
    DDR34_PHY_CONTROL_REGS_RO_PROC_MON_CTLr_EN,
    DDR34_PHY_CONTROL_REGS_RO_PROC_MON_STATUSr_CLK_COUNT,
    DDR34_PHY_CONTROL_REGS_RO_PROC_MON_STATUSr_RO_COUNT,
    DDR34_PHY_CONTROL_REGS_CK_DRIVE_PAD_CTLr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_CK_DRIVE_PAD_CTLr_ADDR_CTL_PD_IDLE_STRENGTH,
    DDR34_PHY_CONTROL_REGS_CK_DRIVE_PAD_CTLr_ADDR_CTL_ND_IDLE_STRENGTH,
    DDR34_PHY_CONTROL_REGS_CK_DRIVE_PAD_CTLr_ADDR_CTL_PD_TERM_STRENGTH,
    DDR34_PHY_CONTROL_REGS_CK_DRIVE_PAD_CTLr_ADDR_CTL_ND_TERM_STRENGTH,
    DDR34_PHY_CONTROL_REGS_CK_DRIVE_PAD_CTLr_ADDR_CTL_PD_STRENGTH,
    DDR34_PHY_CONTROL_REGS_CK_DRIVE_PAD_CTLr_ADDR_CTL_ND_STRENGTH,
    DDR34_PHY_CONTROL_REGS_BYTE_LANE_ALIGN_MASKSr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_BYTE_LANE_ALIGN_MASKSr_MASK_ENABLE,
    DDR34_PHY_CONTROL_REGS_BYTE_LANE_ALIGN_MASKSr_RESERVED1,
    DDR34_PHY_CONTROL_REGS_BYTE_LANE_ALIGN_MASKSr_BL3_MASK,
    DDR34_PHY_CONTROL_REGS_BYTE_LANE_ALIGN_MASKSr_BL2_MASK,
    DDR34_PHY_CONTROL_REGS_BYTE_LANE_ALIGN_MASKSr_BL1_MASK,
    DDR34_PHY_CONTROL_REGS_BYTE_LANE_ALIGN_MASKSr_BL0_MASK,
    DDR34_PHY_CONTROL_REGS_CLOCK_IDLEr_RESERVED0,
    DDR34_PHY_CONTROL_REGS_CLOCK_IDLEr_BIT_CLOCK_CS,
    DDR34_PHY_CONTROL_REGS_CLOCK_IDLEr_BIT_CLOCK_LS,
    DDR34_PHY_CONTROL_REGS_CLOCK_IDLEr_TO_MC_CLOCK,
    DDR34_PHY_CONTROL_REGS_CLOCK_IDLEr_FROM_MC_CLOCK,
    DDR34_PHY_CONTROL_REGS_CLOCK_IDLEr_BIT_CLOCK,
    DDR34_PHY_CONTROL_REGS_AC_SPARE_REGr_RESERVED_FOR_ECO0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQSr_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQSr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQSr_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQSr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQSr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQSr_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQSr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_ALERTr_BUSY,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_ALERTr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_ALERTr_FORCE,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_ALERTr_RESERVED1,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_ALERTr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_ALERTr_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_ALERTr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_0_RD_EN_DLY_CYCr_ENABLE_CS1,
    DDR34_PHY_BYTE_LANE_0_WR_CHAN_DLY_CYCr_DQ_CYCLES,
    DDR34_PHY_BYTE_LANE_0_WR_CHAN_DLY_CYCr_RESERVED2,
    DDR34_PHY_BYTE_LANE_0_READ_CONTROLr_RD_EN_MODE,
    DDR34_PHY_BYTE_LANE_0_DQSP_DRIVE_PAD_CTLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_DQSP_DRIVE_PAD_CTLr_BL_PD_TERM_STRENGTH,
    DDR34_PHY_BYTE_LANE_0_DQSP_DRIVE_PAD_CTLr_BL_ND_TERM_STRENGTH,
    DDR34_PHY_BYTE_LANE_0_DQSP_DRIVE_PAD_CTLr_BL_PD_STRENGTH,
    DDR34_PHY_BYTE_LANE_0_DQSP_DRIVE_PAD_CTLr_BL_ND_STRENGTH,
    DDR34_PHY_BYTE_LANE_0_DQSN_DRIVE_PAD_CTLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_DQSN_DRIVE_PAD_CTLr_BL_PD_TERM_STRENGTH,
    DDR34_PHY_BYTE_LANE_0_DQSN_DRIVE_PAD_CTLr_BL_ND_TERM_STRENGTH,
    DDR34_PHY_BYTE_LANE_0_DQSN_DRIVE_PAD_CTLr_BL_PD_STRENGTH,
    DDR34_PHY_BYTE_LANE_0_DQSN_DRIVE_PAD_CTLr_BL_ND_STRENGTH,
    DDR34_PHY_BYTE_LANE_0_ALERT_DRIVE_PAD_CTLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_ALERT_DRIVE_PAD_CTLr_BL_PD_TERM_STRENGTH,
    DDR34_PHY_BYTE_LANE_0_ALERT_DRIVE_PAD_CTLr_BL_ND_TERM_STRENGTH,
    DDR34_PHY_BYTE_LANE_0_ALERT_DRIVE_PAD_CTLr_BL_PD_STRENGTH,
    DDR34_PHY_BYTE_LANE_0_ALERT_DRIVE_PAD_CTLr_BL_ND_STRENGTH,
    DDR34_PHY_BYTE_LANE_0_STATIC_PAD_CTLr_DM_INVERT,
    DDR34_PHY_BYTE_LANE_0_WR_PREAMBLE_MODEr_POSTAM_TERM_BITS,
    DDR34_PHY_BYTE_LANE_0_WR_PREAMBLE_MODEr_DM_POSTAM_BITS,
    DDR34_PHY_BYTE_LANE_0_WR_PREAMBLE_MODEr_DM_PREAM_BITS,
    DDR34_PHY_BYTE_LANE_0_ODT_CONTROLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_CLOCK_ENABLEr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_CLOCK_ENABLEr_TO_MC_CLOCK,
    DDR34_PHY_BYTE_LANE_0_CLOCK_ENABLEr_FROM_MC_CLOCK,
    DDR34_PHY_BYTE_LANE_0_CLOCK_ENABLEr_BIT_CLOCK,
    DDR34_PHY_BYTE_LANE_0_CLOCK_IDLEr_RESERVED0,
    DDR34_PHY_BYTE_LANE_0_CLOCK_IDLEr_TO_MC_CLOCK,
    DDR34_PHY_BYTE_LANE_0_CLOCK_IDLEr_FROM_MC_CLOCK,
    DDR34_PHY_BYTE_LANE_0_CLOCK_IDLEr_BIT_CLOCK,
    DDR34_PHY_BYTE_LANE_0_BL_SPARE_REGr_RESERVED_FOR_ECO0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQSr_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQSr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQSr_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQSr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQSr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQSr_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQSr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_ALERTr_BUSY,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_ALERTr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_ALERTr_FORCE,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_ALERTr_RESERVED1,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_ALERTr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_ALERTr_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_ALERTr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_1_RD_EN_DLY_CYCr_ENABLE_CS1,
    DDR34_PHY_BYTE_LANE_1_WR_CHAN_DLY_CYCr_DQ_CYCLES,
    DDR34_PHY_BYTE_LANE_1_WR_CHAN_DLY_CYCr_RESERVED2,
    DDR34_PHY_BYTE_LANE_1_READ_CONTROLr_RD_EN_MODE,
    DDR34_PHY_BYTE_LANE_1_DQSP_DRIVE_PAD_CTLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_DQSP_DRIVE_PAD_CTLr_BL_PD_TERM_STRENGTH,
    DDR34_PHY_BYTE_LANE_1_DQSP_DRIVE_PAD_CTLr_BL_ND_TERM_STRENGTH,
    DDR34_PHY_BYTE_LANE_1_DQSP_DRIVE_PAD_CTLr_BL_PD_STRENGTH,
    DDR34_PHY_BYTE_LANE_1_DQSP_DRIVE_PAD_CTLr_BL_ND_STRENGTH,
    DDR34_PHY_BYTE_LANE_1_DQSN_DRIVE_PAD_CTLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_DQSN_DRIVE_PAD_CTLr_BL_PD_TERM_STRENGTH,
    DDR34_PHY_BYTE_LANE_1_DQSN_DRIVE_PAD_CTLr_BL_ND_TERM_STRENGTH,
    DDR34_PHY_BYTE_LANE_1_DQSN_DRIVE_PAD_CTLr_BL_PD_STRENGTH,
    DDR34_PHY_BYTE_LANE_1_DQSN_DRIVE_PAD_CTLr_BL_ND_STRENGTH,
    DDR34_PHY_BYTE_LANE_1_ALERT_DRIVE_PAD_CTLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_ALERT_DRIVE_PAD_CTLr_BL_PD_TERM_STRENGTH,
    DDR34_PHY_BYTE_LANE_1_ALERT_DRIVE_PAD_CTLr_BL_ND_TERM_STRENGTH,
    DDR34_PHY_BYTE_LANE_1_ALERT_DRIVE_PAD_CTLr_BL_PD_STRENGTH,
    DDR34_PHY_BYTE_LANE_1_ALERT_DRIVE_PAD_CTLr_BL_ND_STRENGTH,
    DDR34_PHY_BYTE_LANE_1_STATIC_PAD_CTLr_DM_INVERT,
    DDR34_PHY_BYTE_LANE_1_WR_PREAMBLE_MODEr_POSTAM_TERM_BITS,
    DDR34_PHY_BYTE_LANE_1_WR_PREAMBLE_MODEr_DM_POSTAM_BITS,
    DDR34_PHY_BYTE_LANE_1_WR_PREAMBLE_MODEr_DM_PREAM_BITS,
    DDR34_PHY_BYTE_LANE_1_ODT_CONTROLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_CLOCK_ENABLEr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_CLOCK_ENABLEr_TO_MC_CLOCK,
    DDR34_PHY_BYTE_LANE_1_CLOCK_ENABLEr_FROM_MC_CLOCK,
    DDR34_PHY_BYTE_LANE_1_CLOCK_ENABLEr_BIT_CLOCK,
    DDR34_PHY_BYTE_LANE_1_CLOCK_IDLEr_RESERVED0,
    DDR34_PHY_BYTE_LANE_1_CLOCK_IDLEr_TO_MC_CLOCK,
    DDR34_PHY_BYTE_LANE_1_CLOCK_IDLEr_FROM_MC_CLOCK,
    DDR34_PHY_BYTE_LANE_1_CLOCK_IDLEr_BIT_CLOCK,
    DDR34_PHY_BYTE_LANE_1_BL_SPARE_REGr_RESERVED_FOR_ECO0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQSr_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQSr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQSr_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQSr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQSr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQSr_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQSr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_ALERTr_BUSY,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_ALERTr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_ALERTr_FORCE,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_ALERTr_RESERVED1,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_ALERTr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_ALERTr_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_ALERTr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_2_RD_EN_DLY_CYCr_ENABLE_CS1,
    DDR34_PHY_BYTE_LANE_2_WR_CHAN_DLY_CYCr_DQ_CYCLES,
    DDR34_PHY_BYTE_LANE_2_WR_CHAN_DLY_CYCr_RESERVED2,
    DDR34_PHY_BYTE_LANE_2_READ_CONTROLr_RD_EN_MODE,
    DDR34_PHY_BYTE_LANE_2_DQSP_DRIVE_PAD_CTLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_DQSP_DRIVE_PAD_CTLr_BL_PD_TERM_STRENGTH,
    DDR34_PHY_BYTE_LANE_2_DQSP_DRIVE_PAD_CTLr_BL_ND_TERM_STRENGTH,
    DDR34_PHY_BYTE_LANE_2_DQSP_DRIVE_PAD_CTLr_BL_PD_STRENGTH,
    DDR34_PHY_BYTE_LANE_2_DQSP_DRIVE_PAD_CTLr_BL_ND_STRENGTH,
    DDR34_PHY_BYTE_LANE_2_DQSN_DRIVE_PAD_CTLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_DQSN_DRIVE_PAD_CTLr_BL_PD_TERM_STRENGTH,
    DDR34_PHY_BYTE_LANE_2_DQSN_DRIVE_PAD_CTLr_BL_ND_TERM_STRENGTH,
    DDR34_PHY_BYTE_LANE_2_DQSN_DRIVE_PAD_CTLr_BL_PD_STRENGTH,
    DDR34_PHY_BYTE_LANE_2_DQSN_DRIVE_PAD_CTLr_BL_ND_STRENGTH,
    DDR34_PHY_BYTE_LANE_2_ALERT_DRIVE_PAD_CTLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_ALERT_DRIVE_PAD_CTLr_BL_PD_TERM_STRENGTH,
    DDR34_PHY_BYTE_LANE_2_ALERT_DRIVE_PAD_CTLr_BL_ND_TERM_STRENGTH,
    DDR34_PHY_BYTE_LANE_2_ALERT_DRIVE_PAD_CTLr_BL_PD_STRENGTH,
    DDR34_PHY_BYTE_LANE_2_ALERT_DRIVE_PAD_CTLr_BL_ND_STRENGTH,
    DDR34_PHY_BYTE_LANE_2_STATIC_PAD_CTLr_DM_INVERT,
    DDR34_PHY_BYTE_LANE_2_WR_PREAMBLE_MODEr_POSTAM_TERM_BITS,
    DDR34_PHY_BYTE_LANE_2_WR_PREAMBLE_MODEr_DM_POSTAM_BITS,
    DDR34_PHY_BYTE_LANE_2_WR_PREAMBLE_MODEr_DM_PREAM_BITS,
    DDR34_PHY_BYTE_LANE_2_ODT_CONTROLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_CLOCK_ENABLEr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_CLOCK_ENABLEr_TO_MC_CLOCK,
    DDR34_PHY_BYTE_LANE_2_CLOCK_ENABLEr_FROM_MC_CLOCK,
    DDR34_PHY_BYTE_LANE_2_CLOCK_ENABLEr_BIT_CLOCK,
    DDR34_PHY_BYTE_LANE_2_CLOCK_IDLEr_RESERVED0,
    DDR34_PHY_BYTE_LANE_2_CLOCK_IDLEr_TO_MC_CLOCK,
    DDR34_PHY_BYTE_LANE_2_CLOCK_IDLEr_FROM_MC_CLOCK,
    DDR34_PHY_BYTE_LANE_2_CLOCK_IDLEr_BIT_CLOCK,
    DDR34_PHY_BYTE_LANE_2_BL_SPARE_REGr_RESERVED_FOR_ECO0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQSr_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQSr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQSr_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQSr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQSr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQSr_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQSr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_ALERTr_BUSY,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_ALERTr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_ALERTr_FORCE,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_ALERTr_RESERVED1,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_ALERTr_ADJ_EN,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_ALERTr_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_ALERTr_VDL_STEP,
    DDR34_PHY_BYTE_LANE_3_RD_EN_DLY_CYCr_ENABLE_CS1,
    DDR34_PHY_BYTE_LANE_3_WR_CHAN_DLY_CYCr_DQ_CYCLES,
    DDR34_PHY_BYTE_LANE_3_WR_CHAN_DLY_CYCr_RESERVED2,
    DDR34_PHY_BYTE_LANE_3_READ_CONTROLr_RD_EN_MODE,
    DDR34_PHY_BYTE_LANE_3_DQSP_DRIVE_PAD_CTLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_DQSP_DRIVE_PAD_CTLr_BL_PD_TERM_STRENGTH,
    DDR34_PHY_BYTE_LANE_3_DQSP_DRIVE_PAD_CTLr_BL_ND_TERM_STRENGTH,
    DDR34_PHY_BYTE_LANE_3_DQSP_DRIVE_PAD_CTLr_BL_PD_STRENGTH,
    DDR34_PHY_BYTE_LANE_3_DQSP_DRIVE_PAD_CTLr_BL_ND_STRENGTH,
    DDR34_PHY_BYTE_LANE_3_DQSN_DRIVE_PAD_CTLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_DQSN_DRIVE_PAD_CTLr_BL_PD_TERM_STRENGTH,
    DDR34_PHY_BYTE_LANE_3_DQSN_DRIVE_PAD_CTLr_BL_ND_TERM_STRENGTH,
    DDR34_PHY_BYTE_LANE_3_DQSN_DRIVE_PAD_CTLr_BL_PD_STRENGTH,
    DDR34_PHY_BYTE_LANE_3_DQSN_DRIVE_PAD_CTLr_BL_ND_STRENGTH,
    DDR34_PHY_BYTE_LANE_3_ALERT_DRIVE_PAD_CTLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_ALERT_DRIVE_PAD_CTLr_BL_PD_TERM_STRENGTH,
    DDR34_PHY_BYTE_LANE_3_ALERT_DRIVE_PAD_CTLr_BL_ND_TERM_STRENGTH,
    DDR34_PHY_BYTE_LANE_3_ALERT_DRIVE_PAD_CTLr_BL_PD_STRENGTH,
    DDR34_PHY_BYTE_LANE_3_ALERT_DRIVE_PAD_CTLr_BL_ND_STRENGTH,
    DDR34_PHY_BYTE_LANE_3_STATIC_PAD_CTLr_DM_INVERT,
    DDR34_PHY_BYTE_LANE_3_WR_PREAMBLE_MODEr_POSTAM_TERM_BITS,
    DDR34_PHY_BYTE_LANE_3_WR_PREAMBLE_MODEr_DM_POSTAM_BITS,
    DDR34_PHY_BYTE_LANE_3_WR_PREAMBLE_MODEr_DM_PREAM_BITS,
    DDR34_PHY_BYTE_LANE_3_ODT_CONTROLr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_CLOCK_ENABLEr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_CLOCK_ENABLEr_TO_MC_CLOCK,
    DDR34_PHY_BYTE_LANE_3_CLOCK_ENABLEr_FROM_MC_CLOCK,
    DDR34_PHY_BYTE_LANE_3_CLOCK_ENABLEr_BIT_CLOCK,
    DDR34_PHY_BYTE_LANE_3_CLOCK_IDLEr_RESERVED0,
    DDR34_PHY_BYTE_LANE_3_CLOCK_IDLEr_TO_MC_CLOCK,
    DDR34_PHY_BYTE_LANE_3_CLOCK_IDLEr_FROM_MC_CLOCK,
    DDR34_PHY_BYTE_LANE_3_CLOCK_IDLEr_BIT_CLOCK,
    DDR34_PHY_BYTE_LANE_3_BL_SPARE_REGr_RESERVED_FOR_ECO0,
    DDR34_PHY_MAX_FIELD_INDEX
} ddr34_phy_reg_fields_t;

/*******************************************************************************
 *
 ******************************************************************************/
#define INIT_DDR34_E0_PHY_REGS_TABLE()   { \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_REVISIONr) = DDR34_E0_PHY_CONTROL_REGS_REVISION; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_STATUSr) = DDR34_E0_PHY_CONTROL_REGS_PLL_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_CONFIGr) = DDR34_E0_PHY_CONTROL_REGS_PLL_CONFIG; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_CONTROL1r) = DDR34_E0_PHY_CONTROL_REGS_PLL_CONTROL1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_CONTROL2r) = DDR34_E0_PHY_CONTROL_REGS_PLL_CONTROL2; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_CONTROL3r) = DDR34_E0_PHY_CONTROL_REGS_PLL_CONTROL3; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_DIVIDERSr) = DDR34_E0_PHY_CONTROL_REGS_PLL_DIVIDERS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_FRAC_DIVIDERr) = DDR34_E0_PHY_CONTROL_REGS_PLL_FRAC_DIVIDER; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_SS_CONTROLr) = DDR34_E0_PHY_CONTROL_REGS_PLL_SS_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_SS_LIMITr) = DDR34_E0_PHY_CONTROL_REGS_PLL_SS_LIMIT; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_AUX_CONTROLr) = DDR34_E0_PHY_CONTROL_REGS_AUX_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_IDLE_PAD_CONTROLr) = DDR34_E0_PHY_CONTROL_REGS_IDLE_PAD_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_IDLE_PAD_ENABLE0r) = DDR34_E0_PHY_CONTROL_REGS_IDLE_PAD_ENABLE0; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_IDLE_PAD_ENABLE1r) = DDR34_E0_PHY_CONTROL_REGS_IDLE_PAD_ENABLE1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTLr) = DDR34_E0_PHY_CONTROL_REGS_DRIVE_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_STATIC_PAD_CTLr) = DDR34_E0_PHY_CONTROL_REGS_STATIC_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DRAM_CONFIGr) = DDR34_E0_PHY_CONTROL_REGS_DRAM_CONFIG; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DRAM_TIMING1r) = DDR34_E0_PHY_CONTROL_REGS_DRAM_TIMING1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DRAM_TIMING2r) = DDR34_E0_PHY_CONTROL_REGS_DRAM_TIMING2; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DRAM_TIMING3r) = DDR34_E0_PHY_CONTROL_REGS_DRAM_TIMING3; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DRAM_TIMING4r) = DDR34_E0_PHY_CONTROL_REGS_DRAM_TIMING4; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CALIBRATEr) = DDR34_E0_PHY_CONTROL_REGS_VDL_CALIBRATE; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CALIB_STATUS1r) = DDR34_E0_PHY_CONTROL_REGS_VDL_CALIB_STATUS1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CALIB_STATUS2r) = DDR34_E0_PHY_CONTROL_REGS_VDL_CALIB_STATUS2; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_MONITOR_CONTROLr) = DDR34_E0_PHY_CONTROL_REGS_VDL_MONITOR_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_MONITOR_REFr) = DDR34_E0_PHY_CONTROL_REGS_VDL_MONITOR_REF; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_MONITOR_STATUSr) = DDR34_E0_PHY_CONTROL_REGS_VDL_MONITOR_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OVERRIDEr) = DDR34_E0_PHY_CONTROL_REGS_VDL_MONITOR_OVERRIDE; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_CONTROLr) = DDR34_E0_PHY_CONTROL_REGS_VDL_MONITOR_OUT_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUSr) = DDR34_E0_PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUS_CLEARr) = DDR34_E0_PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUS_CLEAR; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD00r) = DDR34_E0_PHY_CONTROL_REGS_VDL_CONTROL_AD00; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD01r) = DDR34_E0_PHY_CONTROL_REGS_VDL_CONTROL_AD01; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD02r) = DDR34_E0_PHY_CONTROL_REGS_VDL_CONTROL_AD02; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD03r) = DDR34_E0_PHY_CONTROL_REGS_VDL_CONTROL_AD03; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD04r) = DDR34_E0_PHY_CONTROL_REGS_VDL_CONTROL_AD04; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD05r) = DDR34_E0_PHY_CONTROL_REGS_VDL_CONTROL_AD05; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD06r) = DDR34_E0_PHY_CONTROL_REGS_VDL_CONTROL_AD06; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD07r) = DDR34_E0_PHY_CONTROL_REGS_VDL_CONTROL_AD07; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD08r) = DDR34_E0_PHY_CONTROL_REGS_VDL_CONTROL_AD08; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD09r) = DDR34_E0_PHY_CONTROL_REGS_VDL_CONTROL_AD09; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD10r) = DDR34_E0_PHY_CONTROL_REGS_VDL_CONTROL_AD10; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD11r) = DDR34_E0_PHY_CONTROL_REGS_VDL_CONTROL_AD11; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD12r) = DDR34_E0_PHY_CONTROL_REGS_VDL_CONTROL_AD12; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD13r) = DDR34_E0_PHY_CONTROL_REGS_VDL_CONTROL_AD13; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD14r) = DDR34_E0_PHY_CONTROL_REGS_VDL_CONTROL_AD14; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD15r) = DDR34_E0_PHY_CONTROL_REGS_VDL_CONTROL_AD15; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA0r) = DDR34_E0_PHY_CONTROL_REGS_VDL_CONTROL_BA0; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA1r) = DDR34_E0_PHY_CONTROL_REGS_VDL_CONTROL_BA1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA2r) = DDR34_E0_PHY_CONTROL_REGS_VDL_CONTROL_BA2; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX0r) = DDR34_E0_PHY_CONTROL_REGS_VDL_CONTROL_AUX0; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX1r) = DDR34_E0_PHY_CONTROL_REGS_VDL_CONTROL_AUX1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX2r) = DDR34_E0_PHY_CONTROL_REGS_VDL_CONTROL_AUX2; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CS0r) = DDR34_E0_PHY_CONTROL_REGS_VDL_CONTROL_CS0; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CS1r) = DDR34_E0_PHY_CONTROL_REGS_VDL_CONTROL_CS1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_PARr) = DDR34_E0_PHY_CONTROL_REGS_VDL_CONTROL_PAR; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_RAS_Nr) = DDR34_E0_PHY_CONTROL_REGS_VDL_CONTROL_RAS_N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CAS_Nr) = DDR34_E0_PHY_CONTROL_REGS_VDL_CONTROL_CAS_N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CKEr) = DDR34_E0_PHY_CONTROL_REGS_VDL_CONTROL_CKE; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_RST_Nr) = DDR34_E0_PHY_CONTROL_REGS_VDL_CONTROL_RST_N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_ODTr) = DDR34_E0_PHY_CONTROL_REGS_VDL_CONTROL_ODT; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_WE_Nr) = DDR34_E0_PHY_CONTROL_REGS_VDL_CONTROL_WE_N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_Pr) = DDR34_E0_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_Nr) = DDR34_E0_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CLK_CONTROLr) = DDR34_E0_PHY_CONTROL_REGS_VDL_CLK_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_LDE_CONTROLr) = DDR34_E0_PHY_CONTROL_REGS_VDL_LDE_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_REFRESHr) = DDR34_E0_PHY_CONTROL_REGS_REFRESH; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_UPDATE_VDLr) = DDR34_E0_PHY_CONTROL_REGS_UPDATE_VDL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP1r) = DDR34_E0_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP2r) = DDR34_E0_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP2; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_REG1r) = DDR34_E0_PHY_CONTROL_REGS_COMMAND_REG1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG1r) = DDR34_E0_PHY_CONTROL_REGS_COMMAND_AUX_REG1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_REG2r) = DDR34_E0_PHY_CONTROL_REGS_COMMAND_REG2; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG2r) = DDR34_E0_PHY_CONTROL_REGS_COMMAND_AUX_REG2; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_REG3r) = DDR34_E0_PHY_CONTROL_REGS_COMMAND_REG3; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG3r) = DDR34_E0_PHY_CONTROL_REGS_COMMAND_AUX_REG3; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_REG4r) = DDR34_E0_PHY_CONTROL_REGS_COMMAND_REG4; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG4r) = DDR34_E0_PHY_CONTROL_REGS_COMMAND_AUX_REG4; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_REG_TIMERr) = DDR34_E0_PHY_CONTROL_REGS_COMMAND_REG_TIMER; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG0r) = DDR34_E0_PHY_CONTROL_REGS_MODE_REG0; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG1r) = DDR34_E0_PHY_CONTROL_REGS_MODE_REG1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG2r) = DDR34_E0_PHY_CONTROL_REGS_MODE_REG2; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG3r) = DDR34_E0_PHY_CONTROL_REGS_MODE_REG3; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG4r) = DDR34_E0_PHY_CONTROL_REGS_MODE_REG4; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG5r) = DDR34_E0_PHY_CONTROL_REGS_MODE_REG5; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG6r) = DDR34_E0_PHY_CONTROL_REGS_MODE_REG6; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG7r) = DDR34_E0_PHY_CONTROL_REGS_MODE_REG7; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG8r) = DDR34_E0_PHY_CONTROL_REGS_MODE_REG8; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG15r) = DDR34_E0_PHY_CONTROL_REGS_MODE_REG15; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG63r) = DDR34_E0_PHY_CONTROL_REGS_MODE_REG63; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_ALERT_CLEARr) = DDR34_E0_PHY_CONTROL_REGS_ALERT_CLEAR; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_ALERT_STATUSr) = DDR34_E0_PHY_CONTROL_REGS_ALERT_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_CA_PARITYr) = DDR34_E0_PHY_CONTROL_REGS_CA_PARITY; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_CONTROLr) = DDR34_E0_PHY_CONTROL_REGS_CA_PLAYBACK_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_STATUS0r) = DDR34_E0_PHY_CONTROL_REGS_CA_PLAYBACK_STATUS0; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_STATUS1r) = DDR34_E0_PHY_CONTROL_REGS_CA_PLAYBACK_STATUS1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_WRITE_LEVELING_CONTROLr) = DDR34_E0_PHY_CONTROL_REGS_WRITE_LEVELING_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_WRITE_LEVELING_STATUSr) = DDR34_E0_PHY_CONTROL_REGS_WRITE_LEVELING_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_READ_ENABLE_CONTROLr) = DDR34_E0_PHY_CONTROL_REGS_READ_ENABLE_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_READ_ENABLE_STATUSr) = DDR34_E0_PHY_CONTROL_REGS_READ_ENABLE_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_LFSR_SEEDr) = DDR34_E0_PHY_CONTROL_REGS_TRAFFIC_GEN_LFSR_SEED; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ADDRESS1r) = DDR34_E0_PHY_CONTROL_REGS_TRAFFIC_GEN_ADDRESS1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ADDRESS2r) = DDR34_E0_PHY_CONTROL_REGS_TRAFFIC_GEN_ADDRESS2; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROLr) = DDR34_E0_PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_DATA_CONTROLr) = DDR34_E0_PHY_CONTROL_REGS_TRAFFIC_GEN_DATA_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_DQ_MASKr) = DDR34_E0_PHY_CONTROL_REGS_TRAFFIC_GEN_DQ_MASK; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ECC_DQ_MASKr) = DDR34_E0_PHY_CONTROL_REGS_TRAFFIC_GEN_ECC_DQ_MASK; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_STATUSr) = DDR34_E0_PHY_CONTROL_REGS_TRAFFIC_GEN_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_DQ_STATUSr) = DDR34_E0_PHY_CONTROL_REGS_TRAFFIC_GEN_DQ_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ECC_STATUSr) = DDR34_E0_PHY_CONTROL_REGS_TRAFFIC_GEN_ECC_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ERR_CNT_CONTROLr) = DDR34_E0_PHY_CONTROL_REGS_TRAFFIC_GEN_ERR_CNT_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ERR_CNT_STATUSr) = DDR34_E0_PHY_CONTROL_REGS_TRAFFIC_GEN_ERR_CNT_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROLr) = DDR34_E0_PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_STATUSr) = DDR34_E0_PHY_CONTROL_REGS_VIRTUAL_VTT_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_CONNECTIONSr) = DDR34_E0_PHY_CONTROL_REGS_VIRTUAL_VTT_CONNECTIONS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_OVERRIDEr) = DDR34_E0_PHY_CONTROL_REGS_VIRTUAL_VTT_OVERRIDE; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VREF_DAC_CONTROLr) = DDR34_E0_PHY_CONTROL_REGS_VREF_DAC_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_CNTRLr) = DDR34_E0_PHY_CONTROL_REGS_PHYBIST_CNTRL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_SEEDr) = DDR34_E0_PHY_CONTROL_REGS_PHYBIST_SEED; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_STATUSr) = DDR34_E0_PHY_CONTROL_REGS_PHYBIST_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_CTL_STATUSr) = DDR34_E0_PHY_CONTROL_REGS_PHYBIST_CTL_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_BL0_STATUSr) = DDR34_E0_PHY_CONTROL_REGS_PHYBIST_BL0_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_BL1_STATUSr) = DDR34_E0_PHY_CONTROL_REGS_PHYBIST_BL1_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_BL2_STATUSr) = DDR34_E0_PHY_CONTROL_REGS_PHYBIST_BL2_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_BL3_STATUSr) = DDR34_E0_PHY_CONTROL_REGS_PHYBIST_BL3_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_BL4_STATUSr) = DDR34_E0_PHY_CONTROL_REGS_PHYBIST_BL4_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_STANDBY_CONTROLr) = DDR34_E0_PHY_CONTROL_REGS_STANDBY_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DEBUG_FREEZE_ENABLEr) = DDR34_E0_PHY_CONTROL_REGS_DEBUG_FREEZE_ENABLE; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DEBUG_MUX_CONTROLr) = DDR34_E0_PHY_CONTROL_REGS_DEBUG_MUX_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DFI_CNTRLr) = DDR34_E0_PHY_CONTROL_REGS_DFI_CNTRL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_WRITE_ODT_CNTRLr) = DDR34_E0_PHY_CONTROL_REGS_WRITE_ODT_CNTRL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_ABI_PAR_CNTRLr) = DDR34_E0_PHY_CONTROL_REGS_ABI_PAR_CNTRL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_ZQ_CALr) = DDR34_E0_PHY_CONTROL_REGS_ZQ_CAL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_AC_SPARE_REGr) = DDR34_E0_PHY_CONTROL_REGS_AC_SPARE_REG; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_Pr) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_Nr) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ0r) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ0; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ1r) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ2r) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ2; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ3r) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ3; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ4r) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ4; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ5r) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ5; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ6r) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ6; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ7r) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ7; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DMr) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DM; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_EDCr) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CONTROL_WR_EDC; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSPr) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSP; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSNr) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSN; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0Pr) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0Nr) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1Pr) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1Nr) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2Pr) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2Nr) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3Pr) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3Nr) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4Pr) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4Nr) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5Pr) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5Nr) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6Pr) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6Nr) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7Pr) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7Nr) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMPr) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMP; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMNr) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMN; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCPr) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCP; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCNr) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCN; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS0r) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS0; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS1r) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CLK_CONTROLr) = DDR34_E0_PHY_BYTE_LANE_0_VDL_CLK_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_LDE_CONTROLr) = DDR34_E0_PHY_BYTE_LANE_0_VDL_LDE_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_RD_EN_DLY_CYCr) = DDR34_E0_PHY_BYTE_LANE_0_RD_EN_DLY_CYC; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_WR_CHAN_DLY_CYCr) = DDR34_E0_PHY_BYTE_LANE_0_WR_CHAN_DLY_CYC; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_READ_CONTROLr) = DDR34_E0_PHY_BYTE_LANE_0_READ_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_READ_FIFO_ADDRr) = DDR34_E0_PHY_BYTE_LANE_0_READ_FIFO_ADDR; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_READ_FIFO_DATAr) = DDR34_E0_PHY_BYTE_LANE_0_READ_FIFO_DATA; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_READ_FIFO_DM_DBIr) = DDR34_E0_PHY_BYTE_LANE_0_READ_FIFO_DM_DBI; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_READ_FIFO_STATUSr) = DDR34_E0_PHY_BYTE_LANE_0_READ_FIFO_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_READ_FIFO_CLEARr) = DDR34_E0_PHY_BYTE_LANE_0_READ_FIFO_CLEAR; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_IDLE_PAD_CONTROLr) = DDR34_E0_PHY_BYTE_LANE_0_IDLE_PAD_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_DRIVE_PAD_CTLr) = DDR34_E0_PHY_BYTE_LANE_0_DRIVE_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_RD_EN_DRIVE_PAD_CTLr) = DDR34_E0_PHY_BYTE_LANE_0_RD_EN_DRIVE_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_STATIC_PAD_CTLr) = DDR34_E0_PHY_BYTE_LANE_0_STATIC_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_WR_PREAMBLE_MODEr) = DDR34_E0_PHY_BYTE_LANE_0_WR_PREAMBLE_MODE; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_ODT_CONTROLr) = DDR34_E0_PHY_BYTE_LANE_0_ODT_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_EDC_DPD_CONTROLr) = DDR34_E0_PHY_BYTE_LANE_0_EDC_DPD_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_EDC_DPD_STATUSr) = DDR34_E0_PHY_BYTE_LANE_0_EDC_DPD_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_CONTROLr) = DDR34_E0_PHY_BYTE_LANE_0_EDC_DPD_OUT_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_STATUSr) = DDR34_E0_PHY_BYTE_LANE_0_EDC_DPD_OUT_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_STATUS_CLEARr) = DDR34_E0_PHY_BYTE_LANE_0_EDC_DPD_OUT_STATUS_CLEAR; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_EDC_CRC_CONTROLr) = DDR34_E0_PHY_BYTE_LANE_0_EDC_CRC_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_EDC_CRC_STATUSr) = DDR34_E0_PHY_BYTE_LANE_0_EDC_CRC_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_EDC_CRC_COUNTr) = DDR34_E0_PHY_BYTE_LANE_0_EDC_CRC_COUNT; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_EDC_CRC_STATUS_CLEARr) = DDR34_E0_PHY_BYTE_LANE_0_EDC_CRC_STATUS_CLEAR; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_BL_SPARE_REGr) = DDR34_E0_PHY_BYTE_LANE_0_BL_SPARE_REG; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_Pr) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_Nr) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ0r) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ0; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ1r) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ2r) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ2; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ3r) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ3; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ4r) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ4; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ5r) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ5; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ6r) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ6; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ7r) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ7; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DMr) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DM; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_EDCr) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CONTROL_WR_EDC; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSPr) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSP; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSNr) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSN; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0Pr) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0Nr) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1Pr) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1Nr) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2Pr) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2Nr) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3Pr) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3Nr) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4Pr) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4Nr) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5Pr) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5Nr) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6Pr) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6Nr) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7Pr) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7Nr) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMPr) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMP; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMNr) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMN; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCPr) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCP; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCNr) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCN; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS0r) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS0; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS1r) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CLK_CONTROLr) = DDR34_E0_PHY_BYTE_LANE_1_VDL_CLK_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_LDE_CONTROLr) = DDR34_E0_PHY_BYTE_LANE_1_VDL_LDE_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_RD_EN_DLY_CYCr) = DDR34_E0_PHY_BYTE_LANE_1_RD_EN_DLY_CYC; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_WR_CHAN_DLY_CYCr) = DDR34_E0_PHY_BYTE_LANE_1_WR_CHAN_DLY_CYC; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_READ_CONTROLr) = DDR34_E0_PHY_BYTE_LANE_1_READ_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_READ_FIFO_ADDRr) = DDR34_E0_PHY_BYTE_LANE_1_READ_FIFO_ADDR; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_READ_FIFO_DATAr) = DDR34_E0_PHY_BYTE_LANE_1_READ_FIFO_DATA; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_READ_FIFO_DM_DBIr) = DDR34_E0_PHY_BYTE_LANE_1_READ_FIFO_DM_DBI; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_READ_FIFO_STATUSr) = DDR34_E0_PHY_BYTE_LANE_1_READ_FIFO_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_READ_FIFO_CLEARr) = DDR34_E0_PHY_BYTE_LANE_1_READ_FIFO_CLEAR; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_IDLE_PAD_CONTROLr) = DDR34_E0_PHY_BYTE_LANE_1_IDLE_PAD_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_DRIVE_PAD_CTLr) = DDR34_E0_PHY_BYTE_LANE_1_DRIVE_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_RD_EN_DRIVE_PAD_CTLr) = DDR34_E0_PHY_BYTE_LANE_1_RD_EN_DRIVE_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_STATIC_PAD_CTLr) = DDR34_E0_PHY_BYTE_LANE_1_STATIC_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_WR_PREAMBLE_MODEr) = DDR34_E0_PHY_BYTE_LANE_1_WR_PREAMBLE_MODE; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_ODT_CONTROLr) = DDR34_E0_PHY_BYTE_LANE_1_ODT_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_EDC_DPD_CONTROLr) = DDR34_E0_PHY_BYTE_LANE_1_EDC_DPD_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_EDC_DPD_STATUSr) = DDR34_E0_PHY_BYTE_LANE_1_EDC_DPD_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_CONTROLr) = DDR34_E0_PHY_BYTE_LANE_1_EDC_DPD_OUT_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_STATUSr) = DDR34_E0_PHY_BYTE_LANE_1_EDC_DPD_OUT_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_STATUS_CLEARr) = DDR34_E0_PHY_BYTE_LANE_1_EDC_DPD_OUT_STATUS_CLEAR; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_EDC_CRC_CONTROLr) = DDR34_E0_PHY_BYTE_LANE_1_EDC_CRC_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_EDC_CRC_STATUSr) = DDR34_E0_PHY_BYTE_LANE_1_EDC_CRC_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_EDC_CRC_COUNTr) = DDR34_E0_PHY_BYTE_LANE_1_EDC_CRC_COUNT; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_EDC_CRC_STATUS_CLEARr) = DDR34_E0_PHY_BYTE_LANE_1_EDC_CRC_STATUS_CLEAR; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_BL_SPARE_REGr) = DDR34_E0_PHY_BYTE_LANE_1_BL_SPARE_REG; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_Pr) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_Nr) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ0r) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ0; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ1r) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ2r) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ2; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ3r) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ3; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ4r) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ4; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ5r) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ5; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ6r) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ6; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ7r) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ7; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DMr) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DM; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_EDCr) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CONTROL_WR_EDC; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSPr) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSP; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSNr) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSN; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0Pr) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0Nr) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1Pr) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1Nr) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2Pr) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2Nr) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3Pr) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3Nr) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4Pr) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4Nr) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5Pr) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5Nr) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6Pr) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6Nr) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7Pr) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7Nr) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMPr) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMP; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMNr) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMN; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCPr) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCP; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCNr) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCN; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS0r) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS0; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS1r) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CLK_CONTROLr) = DDR34_E0_PHY_BYTE_LANE_2_VDL_CLK_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_LDE_CONTROLr) = DDR34_E0_PHY_BYTE_LANE_2_VDL_LDE_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_RD_EN_DLY_CYCr) = DDR34_E0_PHY_BYTE_LANE_2_RD_EN_DLY_CYC; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_WR_CHAN_DLY_CYCr) = DDR34_E0_PHY_BYTE_LANE_2_WR_CHAN_DLY_CYC; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_READ_CONTROLr) = DDR34_E0_PHY_BYTE_LANE_2_READ_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_READ_FIFO_ADDRr) = DDR34_E0_PHY_BYTE_LANE_2_READ_FIFO_ADDR; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_READ_FIFO_DATAr) = DDR34_E0_PHY_BYTE_LANE_2_READ_FIFO_DATA; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_READ_FIFO_DM_DBIr) = DDR34_E0_PHY_BYTE_LANE_2_READ_FIFO_DM_DBI; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_READ_FIFO_STATUSr) = DDR34_E0_PHY_BYTE_LANE_2_READ_FIFO_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_READ_FIFO_CLEARr) = DDR34_E0_PHY_BYTE_LANE_2_READ_FIFO_CLEAR; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_IDLE_PAD_CONTROLr) = DDR34_E0_PHY_BYTE_LANE_2_IDLE_PAD_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_DRIVE_PAD_CTLr) = DDR34_E0_PHY_BYTE_LANE_2_DRIVE_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_RD_EN_DRIVE_PAD_CTLr) = DDR34_E0_PHY_BYTE_LANE_2_RD_EN_DRIVE_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_STATIC_PAD_CTLr) = DDR34_E0_PHY_BYTE_LANE_2_STATIC_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_WR_PREAMBLE_MODEr) = DDR34_E0_PHY_BYTE_LANE_2_WR_PREAMBLE_MODE; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_ODT_CONTROLr) = DDR34_E0_PHY_BYTE_LANE_2_ODT_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_EDC_DPD_CONTROLr) = DDR34_E0_PHY_BYTE_LANE_2_EDC_DPD_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_EDC_DPD_STATUSr) = DDR34_E0_PHY_BYTE_LANE_2_EDC_DPD_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_CONTROLr) = DDR34_E0_PHY_BYTE_LANE_2_EDC_DPD_OUT_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_STATUSr) = DDR34_E0_PHY_BYTE_LANE_2_EDC_DPD_OUT_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_STATUS_CLEARr) = DDR34_E0_PHY_BYTE_LANE_2_EDC_DPD_OUT_STATUS_CLEAR; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_EDC_CRC_CONTROLr) = DDR34_E0_PHY_BYTE_LANE_2_EDC_CRC_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_EDC_CRC_STATUSr) = DDR34_E0_PHY_BYTE_LANE_2_EDC_CRC_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_EDC_CRC_COUNTr) = DDR34_E0_PHY_BYTE_LANE_2_EDC_CRC_COUNT; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_EDC_CRC_STATUS_CLEARr) = DDR34_E0_PHY_BYTE_LANE_2_EDC_CRC_STATUS_CLEAR; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_BL_SPARE_REGr) = DDR34_E0_PHY_BYTE_LANE_2_BL_SPARE_REG; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_Pr) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_Nr) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ0r) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ0; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ1r) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ2r) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ2; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ3r) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ3; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ4r) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ4; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ5r) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ5; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ6r) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ6; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ7r) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ7; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DMr) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DM; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_EDCr) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CONTROL_WR_EDC; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSPr) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSP; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSNr) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSN; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0Pr) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0Nr) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1Pr) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1Nr) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2Pr) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2Nr) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3Pr) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3Nr) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4Pr) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4Nr) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5Pr) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5Nr) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6Pr) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6Nr) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7Pr) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7Nr) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMPr) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMP; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMNr) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMN; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCPr) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCP; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCNr) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCN; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS0r) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS0; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS1r) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CLK_CONTROLr) = DDR34_E0_PHY_BYTE_LANE_3_VDL_CLK_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_LDE_CONTROLr) = DDR34_E0_PHY_BYTE_LANE_3_VDL_LDE_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_RD_EN_DLY_CYCr) = DDR34_E0_PHY_BYTE_LANE_3_RD_EN_DLY_CYC; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_WR_CHAN_DLY_CYCr) = DDR34_E0_PHY_BYTE_LANE_3_WR_CHAN_DLY_CYC; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_READ_CONTROLr) = DDR34_E0_PHY_BYTE_LANE_3_READ_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_READ_FIFO_ADDRr) = DDR34_E0_PHY_BYTE_LANE_3_READ_FIFO_ADDR; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_READ_FIFO_DATAr) = DDR34_E0_PHY_BYTE_LANE_3_READ_FIFO_DATA; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_READ_FIFO_DM_DBIr) = DDR34_E0_PHY_BYTE_LANE_3_READ_FIFO_DM_DBI; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_READ_FIFO_STATUSr) = DDR34_E0_PHY_BYTE_LANE_3_READ_FIFO_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_READ_FIFO_CLEARr) = DDR34_E0_PHY_BYTE_LANE_3_READ_FIFO_CLEAR; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_IDLE_PAD_CONTROLr) = DDR34_E0_PHY_BYTE_LANE_3_IDLE_PAD_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_DRIVE_PAD_CTLr) = DDR34_E0_PHY_BYTE_LANE_3_DRIVE_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_RD_EN_DRIVE_PAD_CTLr) = DDR34_E0_PHY_BYTE_LANE_3_RD_EN_DRIVE_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_STATIC_PAD_CTLr) = DDR34_E0_PHY_BYTE_LANE_3_STATIC_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_WR_PREAMBLE_MODEr) = DDR34_E0_PHY_BYTE_LANE_3_WR_PREAMBLE_MODE; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_ODT_CONTROLr) = DDR34_E0_PHY_BYTE_LANE_3_ODT_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_EDC_DPD_CONTROLr) = DDR34_E0_PHY_BYTE_LANE_3_EDC_DPD_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_EDC_DPD_STATUSr) = DDR34_E0_PHY_BYTE_LANE_3_EDC_DPD_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_CONTROLr) = DDR34_E0_PHY_BYTE_LANE_3_EDC_DPD_OUT_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_STATUSr) = DDR34_E0_PHY_BYTE_LANE_3_EDC_DPD_OUT_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_STATUS_CLEARr) = DDR34_E0_PHY_BYTE_LANE_3_EDC_DPD_OUT_STATUS_CLEAR; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_EDC_CRC_CONTROLr) = DDR34_E0_PHY_BYTE_LANE_3_EDC_CRC_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_EDC_CRC_STATUSr) = DDR34_E0_PHY_BYTE_LANE_3_EDC_CRC_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_EDC_CRC_COUNTr) = DDR34_E0_PHY_BYTE_LANE_3_EDC_CRC_COUNT; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_EDC_CRC_STATUS_CLEARr) = DDR34_E0_PHY_BYTE_LANE_3_EDC_CRC_STATUS_CLEAR; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_BL_SPARE_REGr) = DDR34_E0_PHY_BYTE_LANE_3_BL_SPARE_REG; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_IDLE_PAD_ENABLE0r) = DDR34_E0_PHY_CONTROL_REGS_IDLE_PAD_ENABLE0; \
}


/**********************************************/
#define INIT_DDR34_F0_PHY_REGS_TABLE()   { \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_REVISIONr) = DDR34_F0_PHY_CONTROL_REGS_REVISION; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_STATUSr) = DDR34_F0_PHY_CONTROL_REGS_PLL_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_CONFIGr) = DDR34_F0_PHY_CONTROL_REGS_PLL_CONFIG; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_CONTROL1r) = DDR34_F0_PHY_CONTROL_REGS_PLL_CONTROL1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_CONTROL2r) = DDR34_F0_PHY_CONTROL_REGS_PLL_CONTROL2; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_CONTROL3r) = DDR34_F0_PHY_CONTROL_REGS_PLL_CONTROL3; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_DIVIDERSr) = DDR34_F0_PHY_CONTROL_REGS_PLL_DIVIDERS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_FRAC_DIVIDERr) = DDR34_F0_PHY_CONTROL_REGS_PLL_FRAC_DIVIDER; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_SS_CONTROLr) = DDR34_F0_PHY_CONTROL_REGS_PLL_SS_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_SS_LIMITr) = DDR34_F0_PHY_CONTROL_REGS_PLL_SS_LIMIT; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_AUX_CONTROLr) = DDR34_F0_PHY_CONTROL_REGS_AUX_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_IDLE_PAD_CONTROLr) = DDR34_F0_PHY_CONTROL_REGS_IDLE_PAD_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_IDLE_PAD_ENABLE0r) = DDR34_F0_PHY_CONTROL_REGS_IDLE_PAD_ENABLE0; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_IDLE_PAD_ENABLE1r) = DDR34_F0_PHY_CONTROL_REGS_IDLE_PAD_ENABLE1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTLr) = DDR34_F0_PHY_CONTROL_REGS_DRIVE_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTL_2Tr) = DDR34_F0_PHY_CONTROL_REGS_DRIVE_PAD_CTL_2T; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_STATIC_PAD_CTLr) = DDR34_F0_PHY_CONTROL_REGS_STATIC_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DRAM_CONFIGr) = DDR34_F0_PHY_CONTROL_REGS_DRAM_CONFIG; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DRAM_TIMING1r) = DDR34_F0_PHY_CONTROL_REGS_DRAM_TIMING1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DRAM_TIMING2r) = DDR34_F0_PHY_CONTROL_REGS_DRAM_TIMING2; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DRAM_TIMING3r) = DDR34_F0_PHY_CONTROL_REGS_DRAM_TIMING3; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DRAM_TIMING4r) = DDR34_F0_PHY_CONTROL_REGS_DRAM_TIMING4; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CALIBRATEr) = DDR34_F0_PHY_CONTROL_REGS_VDL_CALIBRATE; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CALIB_STATUS1r) = DDR34_F0_PHY_CONTROL_REGS_VDL_CALIB_STATUS1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CALIB_STATUS2r) = DDR34_F0_PHY_CONTROL_REGS_VDL_CALIB_STATUS2; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_MONITOR_CONTROLr) = DDR34_F0_PHY_CONTROL_REGS_VDL_MONITOR_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_MONITOR_REFr) = DDR34_F0_PHY_CONTROL_REGS_VDL_MONITOR_REF; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_MONITOR_STATUSr) = DDR34_F0_PHY_CONTROL_REGS_VDL_MONITOR_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OVERRIDEr) = DDR34_F0_PHY_CONTROL_REGS_VDL_MONITOR_OVERRIDE; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_CONTROLr) = DDR34_F0_PHY_CONTROL_REGS_VDL_MONITOR_OUT_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUSr) = DDR34_F0_PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUS_CLEARr) = DDR34_F0_PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUS_CLEAR; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD00r) = DDR34_F0_PHY_CONTROL_REGS_VDL_CONTROL_AD00; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD01r) = DDR34_F0_PHY_CONTROL_REGS_VDL_CONTROL_AD01; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD02r) = DDR34_F0_PHY_CONTROL_REGS_VDL_CONTROL_AD02; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD03r) = DDR34_F0_PHY_CONTROL_REGS_VDL_CONTROL_AD03; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD04r) = DDR34_F0_PHY_CONTROL_REGS_VDL_CONTROL_AD04; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD05r) = DDR34_F0_PHY_CONTROL_REGS_VDL_CONTROL_AD05; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD06r) = DDR34_F0_PHY_CONTROL_REGS_VDL_CONTROL_AD06; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD07r) = DDR34_F0_PHY_CONTROL_REGS_VDL_CONTROL_AD07; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD08r) = DDR34_F0_PHY_CONTROL_REGS_VDL_CONTROL_AD08; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD09r) = DDR34_F0_PHY_CONTROL_REGS_VDL_CONTROL_AD09; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD10r) = DDR34_F0_PHY_CONTROL_REGS_VDL_CONTROL_AD10; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD11r) = DDR34_F0_PHY_CONTROL_REGS_VDL_CONTROL_AD11; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD12r) = DDR34_F0_PHY_CONTROL_REGS_VDL_CONTROL_AD12; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD13r) = DDR34_F0_PHY_CONTROL_REGS_VDL_CONTROL_AD13; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD14r) = DDR34_F0_PHY_CONTROL_REGS_VDL_CONTROL_AD14; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD15r) = DDR34_F0_PHY_CONTROL_REGS_VDL_CONTROL_AD15; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA0r) = DDR34_F0_PHY_CONTROL_REGS_VDL_CONTROL_BA0; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA1r) = DDR34_F0_PHY_CONTROL_REGS_VDL_CONTROL_BA1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA2r) = DDR34_F0_PHY_CONTROL_REGS_VDL_CONTROL_BA2; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX0r) = DDR34_F0_PHY_CONTROL_REGS_VDL_CONTROL_AUX0; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX1r) = DDR34_F0_PHY_CONTROL_REGS_VDL_CONTROL_AUX1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX2r) = DDR34_F0_PHY_CONTROL_REGS_VDL_CONTROL_AUX2; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CS0r) = DDR34_F0_PHY_CONTROL_REGS_VDL_CONTROL_CS0; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CS1r) = DDR34_F0_PHY_CONTROL_REGS_VDL_CONTROL_CS1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_PARr) = DDR34_F0_PHY_CONTROL_REGS_VDL_CONTROL_PAR; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_RAS_Nr) = DDR34_F0_PHY_CONTROL_REGS_VDL_CONTROL_RAS_N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CAS_Nr) = DDR34_F0_PHY_CONTROL_REGS_VDL_CONTROL_CAS_N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CKEr) = DDR34_F0_PHY_CONTROL_REGS_VDL_CONTROL_CKE; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_RST_Nr) = DDR34_F0_PHY_CONTROL_REGS_VDL_CONTROL_RST_N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_ODTr) = DDR34_F0_PHY_CONTROL_REGS_VDL_CONTROL_ODT; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_WE_Nr) = DDR34_F0_PHY_CONTROL_REGS_VDL_CONTROL_WE_N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CKr) = DDR34_F0_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CLK_CONTROLr) = DDR34_F0_PHY_CONTROL_REGS_VDL_CLK_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_LDE_CONTROLr) = DDR34_F0_PHY_CONTROL_REGS_VDL_LDE_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_CMD_CYC_DLYr) = DDR34_F0_PHY_CONTROL_REGS_CMD_CYC_DLY; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_UPDATE_VDLr) = DDR34_F0_PHY_CONTROL_REGS_UPDATE_VDL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP1r) = DDR34_F0_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP2r) = DDR34_F0_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP2; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_REG1r) = DDR34_F0_PHY_CONTROL_REGS_COMMAND_REG1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG1r) = DDR34_F0_PHY_CONTROL_REGS_COMMAND_AUX_REG1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_REG2r) = DDR34_F0_PHY_CONTROL_REGS_COMMAND_REG2; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG2r) = DDR34_F0_PHY_CONTROL_REGS_COMMAND_AUX_REG2; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_REG3r) = DDR34_F0_PHY_CONTROL_REGS_COMMAND_REG3; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG3r) = DDR34_F0_PHY_CONTROL_REGS_COMMAND_AUX_REG3; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_REG4r) = DDR34_F0_PHY_CONTROL_REGS_COMMAND_REG4; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG4r) = DDR34_F0_PHY_CONTROL_REGS_COMMAND_AUX_REG4; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG0r) = DDR34_F0_PHY_CONTROL_REGS_MODE_REG0; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG1r) = DDR34_F0_PHY_CONTROL_REGS_MODE_REG1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG2r) = DDR34_F0_PHY_CONTROL_REGS_MODE_REG2; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG3r) = DDR34_F0_PHY_CONTROL_REGS_MODE_REG3; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG4r) = DDR34_F0_PHY_CONTROL_REGS_MODE_REG4; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG5r) = DDR34_F0_PHY_CONTROL_REGS_MODE_REG5; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG6r) = DDR34_F0_PHY_CONTROL_REGS_MODE_REG6; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG7r) = DDR34_F0_PHY_CONTROL_REGS_MODE_REG7; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG8r) = DDR34_F0_PHY_CONTROL_REGS_MODE_REG8; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG15r) = DDR34_F0_PHY_CONTROL_REGS_MODE_REG15; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG63r) = DDR34_F0_PHY_CONTROL_REGS_MODE_REG63; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_ALERT_CLEARr) = DDR34_F0_PHY_CONTROL_REGS_ALERT_CLEAR; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_ALERT_STATUSr) = DDR34_F0_PHY_CONTROL_REGS_ALERT_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_ALERT_DFIr) = DDR34_F0_PHY_CONTROL_REGS_ALERT_DFI; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_CONTROLr) = DDR34_F0_PHY_CONTROL_REGS_CA_PLAYBACK_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_STATUS0r) = DDR34_F0_PHY_CONTROL_REGS_CA_PLAYBACK_STATUS0; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_STATUS1r) = DDR34_F0_PHY_CONTROL_REGS_CA_PLAYBACK_STATUS1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_WRITE_LEVELING_CONTROLr) = DDR34_F0_PHY_CONTROL_REGS_WRITE_LEVELING_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_WRITE_LEVELING_STATUSr) = DDR34_F0_PHY_CONTROL_REGS_WRITE_LEVELING_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_READ_ENABLE_CONTROLr) = DDR34_F0_PHY_CONTROL_REGS_READ_ENABLE_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_READ_ENABLE_STATUSr) = DDR34_F0_PHY_CONTROL_REGS_READ_ENABLE_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROLr) = DDR34_F0_PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_STATUSr) = DDR34_F0_PHY_CONTROL_REGS_VIRTUAL_VTT_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_CONNECTIONSr) = DDR34_F0_PHY_CONTROL_REGS_VIRTUAL_VTT_CONNECTIONS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_OVERRIDEr) = DDR34_F0_PHY_CONTROL_REGS_VIRTUAL_VTT_OVERRIDE; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VREF_DAC_CONTROLr) = DDR34_F0_PHY_CONTROL_REGS_VREF_DAC_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_CNTRLr) = DDR34_F0_PHY_CONTROL_REGS_PHYBIST_CNTRL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_SEEDr) = DDR34_F0_PHY_CONTROL_REGS_PHYBIST_SEED; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_CA_MASKr) = DDR34_F0_PHY_CONTROL_REGS_PHYBIST_CA_MASK; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_DQ_MASK1r) = DDR34_F0_PHY_CONTROL_REGS_PHYBIST_DQ_MASK1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_DQ_MASK2r) = DDR34_F0_PHY_CONTROL_REGS_PHYBIST_DQ_MASK2; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_STATUSr) = DDR34_F0_PHY_CONTROL_REGS_PHYBIST_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_CTL_STATUSr) = DDR34_F0_PHY_CONTROL_REGS_PHYBIST_CTL_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_BL0_STATUSr) = DDR34_F0_PHY_CONTROL_REGS_PHYBIST_BL0_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_BL1_STATUSr) = DDR34_F0_PHY_CONTROL_REGS_PHYBIST_BL1_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_BL2_STATUSr) = DDR34_F0_PHY_CONTROL_REGS_PHYBIST_BL2_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_BL3_STATUSr) = DDR34_F0_PHY_CONTROL_REGS_PHYBIST_BL3_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_BL4_STATUSr) = DDR34_F0_PHY_CONTROL_REGS_PHYBIST_BL4_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_STANDBY_CONTROLr) = DDR34_F0_PHY_CONTROL_REGS_STANDBY_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DEBUG_FREEZE_ENABLEr) = DDR34_F0_PHY_CONTROL_REGS_DEBUG_FREEZE_ENABLE; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DEBUG_MUX_CONTROLr) = DDR34_F0_PHY_CONTROL_REGS_DEBUG_MUX_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DFI_CNTRLr) = DDR34_F0_PHY_CONTROL_REGS_DFI_CNTRL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_WRITE_ODT_CNTRLr) = DDR34_F0_PHY_CONTROL_REGS_WRITE_ODT_CNTRL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_ABI_PAR_CNTRLr) = DDR34_F0_PHY_CONTROL_REGS_ABI_PAR_CNTRL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_ZQ_CALr) = DDR34_F0_PHY_CONTROL_REGS_ZQ_CAL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_ZQ_CAL2r) = DDR34_F0_PHY_CONTROL_REGS_ZQ_CAL2; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_RO_PROC_MON_CTLr) = DDR34_F0_PHY_CONTROL_REGS_RO_PROC_MON_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_RO_PROC_MON_STATUSr) = DDR34_F0_PHY_CONTROL_REGS_RO_PROC_MON_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_CK_DRIVE_PAD_CTLr) = DDR34_F0_PHY_CONTROL_REGS_CK_DRIVE_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_BYTE_LANE_ALIGN_MASKSr) = DDR34_F0_PHY_CONTROL_REGS_BYTE_LANE_ALIGN_MASKS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_CLOCK_IDLEr) = DDR34_F0_PHY_CONTROL_REGS_CLOCK_IDLE; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_AC_SPARE_REGr) = DDR34_F0_PHY_CONTROL_REGS_AC_SPARE_REG; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQSr) = DDR34_F0_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ0r) = DDR34_F0_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ0; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ1r) = DDR34_F0_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ2r) = DDR34_F0_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ2; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ3r) = DDR34_F0_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ3; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ4r) = DDR34_F0_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ4; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ5r) = DDR34_F0_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ5; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ6r) = DDR34_F0_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ6; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ7r) = DDR34_F0_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ7; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DMr) = DDR34_F0_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DM; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_ALERTr) = DDR34_F0_PHY_BYTE_LANE_0_VDL_CONTROL_WR_ALERT; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSPr) = DDR34_F0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSP; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSNr) = DDR34_F0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSN; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0Pr) = DDR34_F0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0Nr) = DDR34_F0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1Pr) = DDR34_F0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1Nr) = DDR34_F0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2Pr) = DDR34_F0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2Nr) = DDR34_F0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3Pr) = DDR34_F0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3Nr) = DDR34_F0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4Pr) = DDR34_F0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4Nr) = DDR34_F0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5Pr) = DDR34_F0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5Nr) = DDR34_F0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6Pr) = DDR34_F0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6Nr) = DDR34_F0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7Pr) = DDR34_F0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7Nr) = DDR34_F0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMPr) = DDR34_F0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMP; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMNr) = DDR34_F0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMN; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS0r) = DDR34_F0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS0; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS1r) = DDR34_F0_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CLK_CONTROLr) = DDR34_F0_PHY_BYTE_LANE_0_VDL_CLK_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_LDE_CONTROLr) = DDR34_F0_PHY_BYTE_LANE_0_VDL_LDE_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_RD_EN_DLY_CYCr) = DDR34_F0_PHY_BYTE_LANE_0_RD_EN_DLY_CYC; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_WR_CHAN_DLY_CYCr) = DDR34_F0_PHY_BYTE_LANE_0_WR_CHAN_DLY_CYC; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_READ_CONTROLr) = DDR34_F0_PHY_BYTE_LANE_0_READ_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_READ_FIFO_ADDRr) = DDR34_F0_PHY_BYTE_LANE_0_READ_FIFO_ADDR; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_READ_FIFO_DATAr) = DDR34_F0_PHY_BYTE_LANE_0_READ_FIFO_DATA; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_READ_FIFO_DM_DBIr) = DDR34_F0_PHY_BYTE_LANE_0_READ_FIFO_DM_DBI; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_READ_FIFO_STATUSr) = DDR34_F0_PHY_BYTE_LANE_0_READ_FIFO_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_READ_FIFO_CLEARr) = DDR34_F0_PHY_BYTE_LANE_0_READ_FIFO_CLEAR; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_IDLE_PAD_CONTROLr) = DDR34_F0_PHY_BYTE_LANE_0_IDLE_PAD_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_DRIVE_PAD_CTLr) = DDR34_F0_PHY_BYTE_LANE_0_DRIVE_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_DQSP_DRIVE_PAD_CTLr) = DDR34_F0_PHY_BYTE_LANE_0_DQSP_DRIVE_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_DQSN_DRIVE_PAD_CTLr) = DDR34_F0_PHY_BYTE_LANE_0_DQSN_DRIVE_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_ALERT_DRIVE_PAD_CTLr) = DDR34_F0_PHY_BYTE_LANE_0_ALERT_DRIVE_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_RD_EN_DRIVE_PAD_CTLr) = DDR34_F0_PHY_BYTE_LANE_0_RD_EN_DRIVE_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_STATIC_PAD_CTLr) = DDR34_F0_PHY_BYTE_LANE_0_STATIC_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_WR_PREAMBLE_MODEr) = DDR34_F0_PHY_BYTE_LANE_0_WR_PREAMBLE_MODE; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_ODT_CONTROLr) = DDR34_F0_PHY_BYTE_LANE_0_ODT_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_CLOCK_ENABLEr) = DDR34_F0_PHY_BYTE_LANE_0_CLOCK_ENABLE; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_CLOCK_IDLEr) = DDR34_F0_PHY_BYTE_LANE_0_CLOCK_IDLE; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_BL_SPARE_REGr) = DDR34_F0_PHY_BYTE_LANE_0_BL_SPARE_REG; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQSr) = DDR34_F0_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ0r) = DDR34_F0_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ0; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ1r) = DDR34_F0_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ2r) = DDR34_F0_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ2; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ3r) = DDR34_F0_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ3; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ4r) = DDR34_F0_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ4; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ5r) = DDR34_F0_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ5; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ6r) = DDR34_F0_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ6; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ7r) = DDR34_F0_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ7; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DMr) = DDR34_F0_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DM; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_ALERTr) = DDR34_F0_PHY_BYTE_LANE_1_VDL_CONTROL_WR_ALERT; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSPr) = DDR34_F0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSP; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSNr) = DDR34_F0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSN; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0Pr) = DDR34_F0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0Nr) = DDR34_F0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1Pr) = DDR34_F0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1Nr) = DDR34_F0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2Pr) = DDR34_F0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2Nr) = DDR34_F0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3Pr) = DDR34_F0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3Nr) = DDR34_F0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4Pr) = DDR34_F0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4Nr) = DDR34_F0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5Pr) = DDR34_F0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5Nr) = DDR34_F0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6Pr) = DDR34_F0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6Nr) = DDR34_F0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7Pr) = DDR34_F0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7Nr) = DDR34_F0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMPr) = DDR34_F0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMP; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMNr) = DDR34_F0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMN; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS0r) = DDR34_F0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS0; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS1r) = DDR34_F0_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CLK_CONTROLr) = DDR34_F0_PHY_BYTE_LANE_1_VDL_CLK_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_LDE_CONTROLr) = DDR34_F0_PHY_BYTE_LANE_1_VDL_LDE_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_RD_EN_DLY_CYCr) = DDR34_F0_PHY_BYTE_LANE_1_RD_EN_DLY_CYC; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_WR_CHAN_DLY_CYCr) = DDR34_F0_PHY_BYTE_LANE_1_WR_CHAN_DLY_CYC; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_READ_CONTROLr) = DDR34_F0_PHY_BYTE_LANE_1_READ_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_READ_FIFO_ADDRr) = DDR34_F0_PHY_BYTE_LANE_1_READ_FIFO_ADDR; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_READ_FIFO_DATAr) = DDR34_F0_PHY_BYTE_LANE_1_READ_FIFO_DATA; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_READ_FIFO_DM_DBIr) = DDR34_F0_PHY_BYTE_LANE_1_READ_FIFO_DM_DBI; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_READ_FIFO_STATUSr) = DDR34_F0_PHY_BYTE_LANE_1_READ_FIFO_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_READ_FIFO_CLEARr) = DDR34_F0_PHY_BYTE_LANE_1_READ_FIFO_CLEAR; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_IDLE_PAD_CONTROLr) = DDR34_F0_PHY_BYTE_LANE_1_IDLE_PAD_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_DRIVE_PAD_CTLr) = DDR34_F0_PHY_BYTE_LANE_1_DRIVE_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_DQSP_DRIVE_PAD_CTLr) = DDR34_F0_PHY_BYTE_LANE_1_DQSP_DRIVE_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_DQSN_DRIVE_PAD_CTLr) = DDR34_F0_PHY_BYTE_LANE_1_DQSN_DRIVE_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_ALERT_DRIVE_PAD_CTLr) = DDR34_F0_PHY_BYTE_LANE_1_ALERT_DRIVE_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_RD_EN_DRIVE_PAD_CTLr) = DDR34_F0_PHY_BYTE_LANE_1_RD_EN_DRIVE_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_STATIC_PAD_CTLr) = DDR34_F0_PHY_BYTE_LANE_1_STATIC_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_WR_PREAMBLE_MODEr) = DDR34_F0_PHY_BYTE_LANE_1_WR_PREAMBLE_MODE; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_ODT_CONTROLr) = DDR34_F0_PHY_BYTE_LANE_1_ODT_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_CLOCK_ENABLEr) = DDR34_F0_PHY_BYTE_LANE_1_CLOCK_ENABLE; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_CLOCK_IDLEr) = DDR34_F0_PHY_BYTE_LANE_1_CLOCK_IDLE; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_BL_SPARE_REGr) = DDR34_F0_PHY_BYTE_LANE_1_BL_SPARE_REG; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQSr) = DDR34_F0_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ0r) = DDR34_F0_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ0; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ1r) = DDR34_F0_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ2r) = DDR34_F0_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ2; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ3r) = DDR34_F0_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ3; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ4r) = DDR34_F0_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ4; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ5r) = DDR34_F0_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ5; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ6r) = DDR34_F0_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ6; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ7r) = DDR34_F0_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ7; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DMr) = DDR34_F0_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DM; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_ALERTr) = DDR34_F0_PHY_BYTE_LANE_2_VDL_CONTROL_WR_ALERT; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSPr) = DDR34_F0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSP; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSNr) = DDR34_F0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSN; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0Pr) = DDR34_F0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0Nr) = DDR34_F0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1Pr) = DDR34_F0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1Nr) = DDR34_F0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2Pr) = DDR34_F0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2Nr) = DDR34_F0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3Pr) = DDR34_F0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3Nr) = DDR34_F0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4Pr) = DDR34_F0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4Nr) = DDR34_F0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5Pr) = DDR34_F0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5Nr) = DDR34_F0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6Pr) = DDR34_F0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6Nr) = DDR34_F0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7Pr) = DDR34_F0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7Nr) = DDR34_F0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMPr) = DDR34_F0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMP; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMNr) = DDR34_F0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMN; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS0r) = DDR34_F0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS0; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS1r) = DDR34_F0_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CLK_CONTROLr) = DDR34_F0_PHY_BYTE_LANE_2_VDL_CLK_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_LDE_CONTROLr) = DDR34_F0_PHY_BYTE_LANE_2_VDL_LDE_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_RD_EN_DLY_CYCr) = DDR34_F0_PHY_BYTE_LANE_2_RD_EN_DLY_CYC; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_WR_CHAN_DLY_CYCr) = DDR34_F0_PHY_BYTE_LANE_2_WR_CHAN_DLY_CYC; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_READ_CONTROLr) = DDR34_F0_PHY_BYTE_LANE_2_READ_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_READ_FIFO_ADDRr) = DDR34_F0_PHY_BYTE_LANE_2_READ_FIFO_ADDR; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_READ_FIFO_DATAr) = DDR34_F0_PHY_BYTE_LANE_2_READ_FIFO_DATA; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_READ_FIFO_DM_DBIr) = DDR34_F0_PHY_BYTE_LANE_2_READ_FIFO_DM_DBI; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_READ_FIFO_STATUSr) = DDR34_F0_PHY_BYTE_LANE_2_READ_FIFO_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_READ_FIFO_CLEARr) = DDR34_F0_PHY_BYTE_LANE_2_READ_FIFO_CLEAR; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_IDLE_PAD_CONTROLr) = DDR34_F0_PHY_BYTE_LANE_2_IDLE_PAD_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_DRIVE_PAD_CTLr) = DDR34_F0_PHY_BYTE_LANE_2_DRIVE_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_DQSP_DRIVE_PAD_CTLr) = DDR34_F0_PHY_BYTE_LANE_2_DQSP_DRIVE_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_DQSN_DRIVE_PAD_CTLr) = DDR34_F0_PHY_BYTE_LANE_2_DQSN_DRIVE_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_ALERT_DRIVE_PAD_CTLr) = DDR34_F0_PHY_BYTE_LANE_2_ALERT_DRIVE_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_RD_EN_DRIVE_PAD_CTLr) = DDR34_F0_PHY_BYTE_LANE_2_RD_EN_DRIVE_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_STATIC_PAD_CTLr) = DDR34_F0_PHY_BYTE_LANE_2_STATIC_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_WR_PREAMBLE_MODEr) = DDR34_F0_PHY_BYTE_LANE_2_WR_PREAMBLE_MODE; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_ODT_CONTROLr) = DDR34_F0_PHY_BYTE_LANE_2_ODT_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_CLOCK_ENABLEr) = DDR34_F0_PHY_BYTE_LANE_2_CLOCK_ENABLE; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_CLOCK_IDLEr) = DDR34_F0_PHY_BYTE_LANE_2_CLOCK_IDLE; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_BL_SPARE_REGr) = DDR34_F0_PHY_BYTE_LANE_2_BL_SPARE_REG; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQSr) = DDR34_F0_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ0r) = DDR34_F0_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ0; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ1r) = DDR34_F0_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ2r) = DDR34_F0_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ2; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ3r) = DDR34_F0_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ3; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ4r) = DDR34_F0_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ4; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ5r) = DDR34_F0_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ5; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ6r) = DDR34_F0_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ6; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ7r) = DDR34_F0_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ7; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DMr) = DDR34_F0_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DM; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_ALERTr) = DDR34_F0_PHY_BYTE_LANE_3_VDL_CONTROL_WR_ALERT; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSPr) = DDR34_F0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSP; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSNr) = DDR34_F0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSN; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0Pr) = DDR34_F0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0Nr) = DDR34_F0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1Pr) = DDR34_F0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1Nr) = DDR34_F0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2Pr) = DDR34_F0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2Nr) = DDR34_F0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3Pr) = DDR34_F0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3Nr) = DDR34_F0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4Pr) = DDR34_F0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4Nr) = DDR34_F0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5Pr) = DDR34_F0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5Nr) = DDR34_F0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6Pr) = DDR34_F0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6Nr) = DDR34_F0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7Pr) = DDR34_F0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7P; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7Nr) = DDR34_F0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7N; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMPr) = DDR34_F0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMP; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMNr) = DDR34_F0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMN; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS0r) = DDR34_F0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS0; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS1r) = DDR34_F0_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS1; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CLK_CONTROLr) = DDR34_F0_PHY_BYTE_LANE_3_VDL_CLK_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_LDE_CONTROLr) = DDR34_F0_PHY_BYTE_LANE_3_VDL_LDE_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_RD_EN_DLY_CYCr) = DDR34_F0_PHY_BYTE_LANE_3_RD_EN_DLY_CYC; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_WR_CHAN_DLY_CYCr) = DDR34_F0_PHY_BYTE_LANE_3_WR_CHAN_DLY_CYC; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_READ_CONTROLr) = DDR34_F0_PHY_BYTE_LANE_3_READ_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_READ_FIFO_ADDRr) = DDR34_F0_PHY_BYTE_LANE_3_READ_FIFO_ADDR; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_READ_FIFO_DATAr) = DDR34_F0_PHY_BYTE_LANE_3_READ_FIFO_DATA; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_READ_FIFO_DM_DBIr) = DDR34_F0_PHY_BYTE_LANE_3_READ_FIFO_DM_DBI; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_READ_FIFO_STATUSr) = DDR34_F0_PHY_BYTE_LANE_3_READ_FIFO_STATUS; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_READ_FIFO_CLEARr) = DDR34_F0_PHY_BYTE_LANE_3_READ_FIFO_CLEAR; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_IDLE_PAD_CONTROLr) = DDR34_F0_PHY_BYTE_LANE_3_IDLE_PAD_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_DRIVE_PAD_CTLr) = DDR34_F0_PHY_BYTE_LANE_3_DRIVE_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_DQSP_DRIVE_PAD_CTLr) = DDR34_F0_PHY_BYTE_LANE_3_DQSP_DRIVE_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_DQSN_DRIVE_PAD_CTLr) = DDR34_F0_PHY_BYTE_LANE_3_DQSN_DRIVE_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_ALERT_DRIVE_PAD_CTLr) = DDR34_F0_PHY_BYTE_LANE_3_ALERT_DRIVE_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_RD_EN_DRIVE_PAD_CTLr) = DDR34_F0_PHY_BYTE_LANE_3_RD_EN_DRIVE_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_STATIC_PAD_CTLr) = DDR34_F0_PHY_BYTE_LANE_3_STATIC_PAD_CTL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_WR_PREAMBLE_MODEr) = DDR34_F0_PHY_BYTE_LANE_3_WR_PREAMBLE_MODE; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_ODT_CONTROLr) = DDR34_F0_PHY_BYTE_LANE_3_ODT_CONTROL; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_CLOCK_ENABLEr) = DDR34_F0_PHY_BYTE_LANE_3_CLOCK_ENABLE; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_CLOCK_IDLEr) = DDR34_F0_PHY_BYTE_LANE_3_CLOCK_IDLE; \
    DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_BL_SPARE_REGr) = DDR34_F0_PHY_BYTE_LANE_3_BL_SPARE_REG; \
}

/**************************************************************
***************************************************************/
#define INIT_DDR34_SIGNE_FIELD(v, r, f) \
        DDR34_PHY_FIELDS_TABLE(DDR34_##r##r_##f, 0) = DDR34_##v##_##r##_##f##_MASK; \
        DDR34_PHY_FIELDS_TABLE(DDR34_##r##r_##f, 1) = DDR34_##v##_##r##_##f##_ALIGN; \
        DDR34_PHY_FIELDS_TABLE(DDR34_##r##r_##f, 2) = DDR34_##v##_##r##_##f##_BITS; \
        DDR34_PHY_FIELDS_TABLE(DDR34_##r##r_##f, 3) = DDR34_##v##_##r##_##f##_SHIFT; 

#define INIT_DDR34_E0_PHY_FIELDS_TABLE()   { \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_REVISION, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_REVISION, PERFORMANCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_REVISION, TECHNOLOGY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_REVISION, WB); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_REVISION, BITS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_REVISION, MAJOR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_REVISION, MINOR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_STATUS, LOCK_LOST); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_STATUS, CLOCKING_8X); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_STATUS, CLOCKING_4X); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_STATUS, CLOCKING_2X); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_STATUS, STATUS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_STATUS, LOCK); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_CONFIG, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_CONFIG, RESERVED_for_eco1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_CONFIG, CK_LDO_REF_CTRL); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_CONFIG, CK_LDO_BIAS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_CONFIG, PLL_LDO_REF_SEL); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_CONFIG, PLL_LDO_REF_CTRL); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_CONFIG, PLL_LDO_BIAS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_CONFIG, HOLD); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_CONFIG, ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_CONFIG, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_CONFIG, FB_OFFSET); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_CONFIG, RESERVED3); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_CONFIG, RESET_POST_DIV); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_CONFIG, RESERVED4); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_CONFIG, RESET); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_CONFIG, PWRDN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_CONTROL1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_CONTROL1, I_KP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_CONTROL1, I_KI); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_CONTROL1, I_KA); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_CONTROL2, VCO_RANGE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_CONTROL2, UNUSED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_CONTROL2, NDIV_RELOCK); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_CONTROL2, FAST_LOCK); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_CONTROL2, VCO_FB_DIV2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_CONTROL2, POST_CTRL_RESETB); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_CONTROL2, PWM_RATE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_CONTROL2, STAT_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_CONTROL2, UNUSED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_CONTROL2, STAT_UPDATE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_CONTROL2, STAT_SELECT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_CONTROL2, STAT_RESET); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_CONTROL2, DCO_CTRL_BYPASS_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_CONTROL2, DCO_CTRL_BYPASS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_CONTROL3, PLL_CONTROL); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_DIVIDERS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_DIVIDERS, MDIV); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_DIVIDERS, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_DIVIDERS, PDIV); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_DIVIDERS, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_DIVIDERS, NDIV_INT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_FRAC_DIVIDER, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_FRAC_DIVIDER, NDIV_FRAC); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_SS_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_SS_CONTROL, SSC_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_SS_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_SS_CONTROL, SSC_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_SS_LIMIT, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_SS_LIMIT, SSC_LIMIT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PLL_SS_LIMIT, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_AUX_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_AUX_CONTROL, IS_ODT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_AUX_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_AUX_CONTROL, IS_CS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_AUX_CONTROL, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_AUX_CONTROL, IS_AD); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_IDLE_PAD_CONTROL, IDLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_IDLE_PAD_CONTROL, DIB_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_IDLE_PAD_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_IDLE_PAD_CONTROL, RXENB); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_IDLE_PAD_CONTROL, IDDQ); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_IDLE_PAD_CONTROL, DOUT_N); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_IDLE_PAD_CONTROL, DOUT_P); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_IDLE_PAD_ENABLE0, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_IDLE_PAD_ENABLE0, IO_IDLE_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_IDLE_PAD_ENABLE1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_IDLE_PAD_ENABLE1, IO_IDLE_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DRIVE_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DRIVE_PAD_CTL, ADDR_CTL_PD_IDLE_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DRIVE_PAD_CTL, ADDR_CTL_ND_IDLE_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DRIVE_PAD_CTL, ADDR_CTL_PD_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DRIVE_PAD_CTL, ADDR_CTL_ND_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DRIVE_PAD_CTL, ADDR_CTL_PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DRIVE_PAD_CTL, ADDR_CTL_ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STATIC_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STATIC_PAD_CTL, AUTO_OEB); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STATIC_PAD_CTL, IDDQ_GDDR5); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STATIC_PAD_CTL, IDDQ_LPDDR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STATIC_PAD_CTL, IDDQ_CLK1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STATIC_PAD_CTL, IDDQ_CLK0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STATIC_PAD_CTL, IDDQ_ODT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STATIC_PAD_CTL, IDDQ_PAR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STATIC_PAD_CTL, IDDQ_BA); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STATIC_PAD_CTL, IDDQ_AUX2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STATIC_PAD_CTL, IDDQ_AUX1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STATIC_PAD_CTL, IDDQ_AUX0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STATIC_PAD_CTL, IDDQ_CS1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STATIC_PAD_CTL, IDDQ_A15); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STATIC_PAD_CTL, IDDQ_A14); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STATIC_PAD_CTL, IDDQ_A13); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STATIC_PAD_CTL, IDDQ_A12); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STATIC_PAD_CTL, IDDQ_A11); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STATIC_PAD_CTL, IDDQ_A10); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STATIC_PAD_CTL, IDDQ_A09); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STATIC_PAD_CTL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STATIC_PAD_CTL, RX_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DRAM_CONFIG, INIT_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DRAM_CONFIG, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DRAM_CONFIG, ECC_ENABLED); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DRAM_CONFIG, SPLIT_DQ_BUS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DRAM_CONFIG, BUS16); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DRAM_CONFIG, BUS8); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DRAM_CONFIG, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DRAM_CONFIG, EDC_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DRAM_CONFIG, RDQS_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DRAM_CONFIG, GROUP_BITS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DRAM_CONFIG, BANK_BITS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DRAM_CONFIG, COL_BITS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DRAM_CONFIG, ROW_BITS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DRAM_CONFIG, DRAM_TYPE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DRAM_TIMING1, TRAS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DRAM_TIMING1, TRRD); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DRAM_TIMING1, TRP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DRAM_TIMING1, TRCD); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DRAM_TIMING2, TRTP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DRAM_TIMING2, TWR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DRAM_TIMING2, TCWL); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DRAM_TIMING2, TCAS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DRAM_TIMING3, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DRAM_TIMING3, TCAL); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DRAM_TIMING3, TRTW); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DRAM_TIMING3, TWTR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DRAM_TIMING3, TRFC); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DRAM_TIMING4, temp); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CALIBRATE, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CALIBRATE, RESERVED_for_padding1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CALIBRATE, HALF_STEPS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CALIBRATE, UPDATE_FAST); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CALIBRATE, UPDATE_REGS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CALIBRATE, CALIB_FTM2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CALIBRATE, CALIB_PHYBIST); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CALIBRATE, CALIB_ONCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CALIB_STATUS1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CALIB_STATUS1, CALIB_TOTAL_STEPS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CALIB_STATUS1, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CALIB_STATUS1, CALIB_BUS_ERROR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CALIB_STATUS1, CALIB_REGS_DONE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CALIB_STATUS1, CALIB_LOCK_6B); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CALIB_STATUS1, CALIB_LOCK_4B); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CALIB_STATUS1, CALIB_LOCK_2B); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CALIB_STATUS1, CALIB_IDLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CALIB_STATUS2, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CALIB_STATUS2, CALIB_4B_STEPS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CALIB_STATUS2, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CALIB_STATUS2, CALIB_2B_STEPS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_CONTROL, INTERVAL); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_CONTROL, UPDATE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_CONTROL, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_CONTROL, DATA_RATE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_CONTROL, ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_REF, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_REF, MONITOR_4B_STEPS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_REF, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_REF, MONITOR_2B_STEPS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_STATUS, MONITOR_BUS_ERROR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_STATUS, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_STATUS, MONITOR_ADJ); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_STATUS, MONITOR_CHANGE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_STATUS, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_STATUS, MONITOR_TOTAL); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_OVERRIDE, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_OVERRIDE, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_OVERRIDE, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_OVERRIDE, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_OVERRIDE, ADJ); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_OVERRIDE, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_OVERRIDE, ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_OUT_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_OUT_CONTROL, LOWER_LIMIT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_OUT_CONTROL, UPPER_LIMIT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_OUT_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_OUT_CONTROL, ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUS, MONITOR_CHANGE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUS, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUS, MONITOR_TOTAL); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUS, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUS, VALID); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUS, CLEAR_RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUS, CLEAR_CLEAR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUS_CLEAR, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUS_CLEAR, CLEAR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD00, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD00, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD00, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD00, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD00, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD00, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD00, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD01, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD01, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD01, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD01, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD01, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD01, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD01, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD02, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD02, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD02, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD02, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD02, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD02, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD02, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD03, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD03, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD03, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD03, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD03, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD03, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD03, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD04, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD04, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD04, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD04, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD04, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD04, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD04, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD05, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD05, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD05, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD05, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD05, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD05, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD05, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD06, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD06, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD06, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD06, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD06, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD06, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD06, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD07, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD07, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD07, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD07, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD07, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD07, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD07, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD08, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD08, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD08, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD08, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD08, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD08, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD08, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD09, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD09, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD09, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD09, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD09, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD09, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD09, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD10, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD10, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD10, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD10, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD10, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD10, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD10, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD11, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD11, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD11, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD11, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD11, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD11, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD11, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD12, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD12, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD12, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD12, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD12, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD12, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD12, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD13, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD13, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD13, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD13, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD13, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD13, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD13, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD14, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD14, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD14, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD14, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD14, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD14, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD14, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD15, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD15, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD15, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD15, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD15, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD15, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AD15, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_BA0, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_BA0, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_BA0, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_BA0, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_BA0, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_BA0, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_BA0, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_BA1, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_BA1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_BA1, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_BA1, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_BA1, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_BA1, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_BA1, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_BA2, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_BA2, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_BA2, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_BA2, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_BA2, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_BA2, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_BA2, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AUX0, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AUX0, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AUX0, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AUX0, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AUX0, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AUX0, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AUX0, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AUX1, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AUX1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AUX1, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AUX1, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AUX1, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AUX1, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AUX1, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AUX2, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AUX2, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AUX2, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AUX2, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AUX2, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AUX2, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_AUX2, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_CS0, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_CS0, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_CS0, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_CS0, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_CS0, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_CS0, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_CS0, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_CS1, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_CS1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_CS1, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_CS1, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_CS1, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_CS1, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_CS1, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_PAR, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_PAR, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_PAR, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_PAR, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_PAR, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_PAR, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_PAR, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_RAS_N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_RAS_N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_RAS_N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_RAS_N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_RAS_N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_RAS_N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_RAS_N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_CAS_N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_CAS_N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_CAS_N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_CAS_N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_CAS_N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_CAS_N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_CAS_N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_CKE, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_CKE, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_CKE, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_CKE, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_CKE, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_CKE, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_CKE, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_RST_N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_RST_N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_RST_N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_RST_N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_RST_N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_RST_N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_RST_N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_ODT, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_ODT, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_ODT, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_ODT, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_ODT, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_ODT, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_ODT, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_WE_N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_WE_N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_WE_N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_WE_N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_WE_N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_WE_N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_WE_N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CLK_CONTROL, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CLK_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CLK_CONTROL, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CLK_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CLK_CONTROL, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CLK_CONTROL, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_CLK_CONTROL, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_LDE_CONTROL, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_LDE_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_LDE_CONTROL, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_LDE_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_LDE_CONTROL, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_LDE_CONTROL, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VDL_LDE_CONTROL, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_REFRESH, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_REFRESH, ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_REFRESH, PERIOD); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL, MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL, DISABLE_INPUT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL, ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL, SNOOP1_RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL, SNOOP1_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL, SNOOP1_RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL, SNOOP1_MASK); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL, SNOOP1_RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL, SNOOP1_CMD); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL, SNOOP1_RESERVED3); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL, SNOOP1_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL, SNOOP2_RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL, SNOOP2_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL, SNOOP2_RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL, SNOOP2_MASK); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL, SNOOP2_RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL, SNOOP2_CMD); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL, SNOOP2_RESERVED3); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL, SNOOP2_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL_SNOOP1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL_SNOOP1, MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL_SNOOP1, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL_SNOOP1, MASK); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL_SNOOP1, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL_SNOOP1, CMD); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL_SNOOP1, RESERVED3); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL_SNOOP1, ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL_SNOOP2, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL_SNOOP2, MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL_SNOOP2, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL_SNOOP2, MASK); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL_SNOOP2, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL_SNOOP2, CMD); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL_SNOOP2, RESERVED3); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_UPDATE_VDL_SNOOP2, ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG1, MCP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG1, CS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG1, AUX); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG1, ACT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG1, WE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG1, CAS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG1, RAS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG1, BA); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG1, AD); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_AUX_REG1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_AUX_REG1, AUX); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG2, MCP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG2, CS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG2, AUX); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG2, ACT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG2, WE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG2, CAS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG2, RAS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG2, BA); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG2, AD); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_AUX_REG2, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_AUX_REG2, AUX); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG3, MCP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG3, CS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG3, AUX); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG3, ACT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG3, WE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG3, CAS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG3, RAS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG3, BA); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG3, AD); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_AUX_REG3, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_AUX_REG3, AUX); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG4, MCP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG4, CS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG4, AUX); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG4, ACT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG4, WE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG4, CAS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG4, RAS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG4, BA); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG4, AD); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_AUX_REG4, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_AUX_REG4, AUX); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG_TIMER, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG_TIMER, INIT_VAL); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_COMMAND_REG_TIMER, COUNT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_MODE_REG0, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_MODE_REG0, VALID); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_MODE_REG0, AD); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_MODE_REG1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_MODE_REG1, VALID); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_MODE_REG1, AD); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_MODE_REG2, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_MODE_REG2, VALID); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_MODE_REG2, AD); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_MODE_REG3, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_MODE_REG3, VALID); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_MODE_REG3, AD); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_MODE_REG4, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_MODE_REG4, VALID); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_MODE_REG4, AD); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_MODE_REG5, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_MODE_REG5, VALID); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_MODE_REG5, AD); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_MODE_REG6, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_MODE_REG6, VALID); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_MODE_REG6, AD); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_MODE_REG7, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_MODE_REG7, VALID); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_MODE_REG7, AD); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_MODE_REG8, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_MODE_REG8, VALID); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_MODE_REG8, AD); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_MODE_REG15, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_MODE_REG15, VALID); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_MODE_REG15, AD); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_MODE_REG63, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_MODE_REG63, VALID); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_MODE_REG63, AD); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_ALERT_CLEAR, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_ALERT_CLEAR, CLEAR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_ALERT_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_ALERT_STATUS, STATUS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_CA_PARITY, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_CA_PARITY, ERROR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_CA_PARITY, ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_CA_PLAYBACK_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_CA_PLAYBACK_CONTROL, COUNT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_CA_PLAYBACK_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_CA_PLAYBACK_CONTROL, SAMPLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_CA_PLAYBACK_STATUS0, VALID); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_CA_PLAYBACK_STATUS0, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_CA_PLAYBACK_STATUS0, DATA1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_CA_PLAYBACK_STATUS0, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_CA_PLAYBACK_STATUS0, DATA0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_CA_PLAYBACK_STATUS1, VALID); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_CA_PLAYBACK_STATUS1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_CA_PLAYBACK_STATUS1, DATA1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_CA_PLAYBACK_STATUS1, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_CA_PLAYBACK_STATUS1, DATA0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_WRITE_LEVELING_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_WRITE_LEVELING_CONTROL, COUNT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_WRITE_LEVELING_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_WRITE_LEVELING_CONTROL, SAMPLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_WRITE_LEVELING_CONTROL, CONTINUOUS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_WRITE_LEVELING_CONTROL, ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_WRITE_LEVELING_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_WRITE_LEVELING_STATUS, EDC); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_WRITE_LEVELING_STATUS, STATUS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_WRITE_LEVELING_STATUS, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_WRITE_LEVELING_STATUS, VALID); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_READ_ENABLE_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_READ_ENABLE_CONTROL, TEST_CYCLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_READ_ENABLE_CONTROL, SELECT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_READ_ENABLE_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_READ_ENABLE_CONTROL, CS_N); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_READ_ENABLE_CONTROL, EDC_DATA); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_READ_ENABLE_CONTROL, EDC_PHASE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_READ_ENABLE_CONTROL, DQS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_READ_ENABLE_CONTROL, ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_READ_ENABLE_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_READ_ENABLE_STATUS, DATA); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_READ_ENABLE_STATUS, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_READ_ENABLE_STATUS, BL4_STATUS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_READ_ENABLE_STATUS, BL3_STATUS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_READ_ENABLE_STATUS, BL2_STATUS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_READ_ENABLE_STATUS, BL1_STATUS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_READ_ENABLE_STATUS, BL0_STATUS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_READ_ENABLE_STATUS, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_READ_ENABLE_STATUS, VALID); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_LFSR_SEED, SEED); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_ADDRESS1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_ADDRESS1, BANK); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_ADDRESS1, ROW); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_ADDRESS2, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_ADDRESS2, BANK); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_ADDRESS2, ROW); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROL, DIAG_WRO); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROL, DIAG_WRO_RD); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROL, DIAG_WR_RD); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROL, WR_NOISE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROL, RD_NOISE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROL, CLEAR_DRAM); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROL, MASK_DM); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROL, INIT_LFSR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROL, FIFO); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROL, MPR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROL, RD_WR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROL, RD_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROL, ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_DATA_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_DATA_CONTROL, PATTERN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_DATA_CONTROL, LENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_DQ_MASK, MASK); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_ECC_DQ_MASK, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_ECC_DQ_MASK, MASK); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_STATUS, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_DQ_STATUS, STATUS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_ECC_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_ECC_STATUS, STATUS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_ERR_CNT_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_ERR_CNT_CONTROL, DQ_SEL); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_ERR_CNT_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_ERR_CNT_CONTROL, CLEAR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_ERR_CNT_CONTROL, ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_ERR_CNT_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_TRAFFIC_GEN_ERR_CNT_STATUS, COUNT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROL, RESERVED_for_eco1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROL, MAX_NOISE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROL, LOW_NOISE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROL, LOW_VTT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROL, HIGH_VTT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROL, ERROR_RESET); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROL, ENABLE_CTL_IDLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROL, ENABLE_CS_IDLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROL, ENABLE_CKE_IDLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VIRTUAL_VTT_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VIRTUAL_VTT_STATUS, ERROR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VIRTUAL_VTT_STATUS, ERROR_LOW); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VIRTUAL_VTT_STATUS, ERROR_HIGH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VIRTUAL_VTT_STATUS, READY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VIRTUAL_VTT_CONNECTIONS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VIRTUAL_VTT_CONNECTIONS, MASK); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VIRTUAL_VTT_OVERRIDE, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VIRTUAL_VTT_OVERRIDE, MASK); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VREF_DAC_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VREF_DAC_CONTROL, AUX_GT_INT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VREF_DAC_CONTROL, TESTOUT_MUX_CTL); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VREF_DAC_CONTROL, TEST); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VREF_DAC_CONTROL, PDN3); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VREF_DAC_CONTROL, PDN2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VREF_DAC_CONTROL, PDN1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VREF_DAC_CONTROL, PDN0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VREF_DAC_CONTROL, DAC1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_VREF_DAC_CONTROL, DAC0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_CNTRL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_CNTRL, CLK_PAD_ENB); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_CNTRL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_CNTRL, FORCE_DQ_ERROR_SEL); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_CNTRL, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_CNTRL, FORCE_BL_ERROR_SEL); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_CNTRL, RESERVED3); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_CNTRL, FORCE_CTL_ERROR_SEL); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_CNTRL, RESERVED4); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_CNTRL, FORCE_DAT_ERROR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_CNTRL, FORCE_CTL_ERROR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_CNTRL, SSO); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_CNTRL, LENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_CNTRL, MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_CNTRL, ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_SEED, SEED); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_STATUS, DAT_PASS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_STATUS, CTL_PASS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_STATUS, DAT_DONE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_STATUS, CTL_DONE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_CTL_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_CTL_STATUS, CTL_ERRORS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_BL0_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_BL0_STATUS, EDC); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_BL0_STATUS, DM); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_BL0_STATUS, DQ); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_BL1_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_BL1_STATUS, EDC); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_BL1_STATUS, DM); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_BL1_STATUS, DQ); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_BL2_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_BL2_STATUS, EDC); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_BL2_STATUS, DM); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_BL2_STATUS, DQ); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_BL3_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_BL3_STATUS, EDC); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_BL3_STATUS, DM); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_BL3_STATUS, DQ); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_BL4_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_BL4_STATUS, EDC); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_BL4_STATUS, DM); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_PHYBIST_BL4_STATUS, DQ); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STANDBY_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STANDBY_CONTROL, STANDBY_READY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STANDBY_CONTROL, STANDBY_EXIT_PIN_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STANDBY_CONTROL, STANDBY_ACTIVE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STANDBY_CONTROL, ARMED); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STANDBY_CONTROL, WARMSTART); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STANDBY_CONTROL, PWRDOWN_LDO_BIAS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STANDBY_CONTROL, PWRDOWN_LDO_VOLTS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STANDBY_CONTROL, PWRDOWN_SKIP_MRS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STANDBY_CONTROL, PWRDOWN_RST_N); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STANDBY_CONTROL, PWRDOWN_CKE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STANDBY_CONTROL, LDO_BIAS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STANDBY_CONTROL, LDO_VOLTS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STANDBY_CONTROL, SKIP_MRS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STANDBY_CONTROL, RST_N); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STANDBY_CONTROL, CKE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_STANDBY_CONTROL, STANDBY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DEBUG_FREEZE_ENABLE, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DEBUG_FREEZE_ENABLE, WLECC); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DEBUG_FREEZE_ENABLE, WL1_BL1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DEBUG_FREEZE_ENABLE, WL1_BL0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DEBUG_FREEZE_ENABLE, WL0_BL1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DEBUG_FREEZE_ENABLE, WL0_BL0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DEBUG_MUX_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DEBUG_MUX_CONTROL, BYTE_SEL); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DEBUG_MUX_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DEBUG_MUX_CONTROL, PHASE_SEL); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DEBUG_MUX_CONTROL, SOURCE_SEL); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DFI_CNTRL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DFI_CNTRL, DFI_CS1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DFI_CNTRL, DFI_CS0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DFI_CNTRL, DFI_RST_N); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DFI_CNTRL, DFI_CKE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DFI_CNTRL, ACK_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DFI_CNTRL, ACK_STATUS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_DFI_CNTRL, ASSERT_REQ); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_WRITE_ODT_CNTRL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_WRITE_ODT_CNTRL, ODT_FORCE_VALUE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_WRITE_ODT_CNTRL, ODT_FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_WRITE_ODT_CNTRL, ODT_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_WRITE_ODT_CNTRL, ODT_LENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_WRITE_ODT_CNTRL, ODT_DELAY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_ABI_PAR_CNTRL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_ABI_PAR_CNTRL, PAR_INCLUDE_AUX); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_ABI_PAR_CNTRL, PAR_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_ABI_PAR_CNTRL, ABI_INCLUDE_AUX); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_ABI_PAR_CNTRL, ABI_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_ZQ_CAL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_ZQ_CAL, ZQ_PCOMP_STATUS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_ZQ_CAL, ZQ_NCOMP_STATUS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_ZQ_CAL, ZQ_IDDQ); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_ZQ_CAL, ZQ_DRIVE_P); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_ZQ_CAL, ZQ_DRIVE_N); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_ZQ_CAL, RESERVED_for_padding1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_ZQ_CAL, ZQ_PCOMP_ENB); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_ZQ_CAL, ZQ_NCOMP_ENB); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_AC_SPARE_REG, RESERVED_for_eco0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ0, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ0, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ0, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ0, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ0, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ0, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ0, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ1, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ1, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ1, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ1, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ1, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ1, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ2, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ2, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ2, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ2, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ2, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ2, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ2, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ3, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ3, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ3, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ3, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ3, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ3, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ3, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ4, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ4, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ4, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ4, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ4, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ4, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ4, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ5, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ5, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ5, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ5, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ5, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ5, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ5, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ6, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ6, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ6, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ6, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ6, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ6, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ6, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ7, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ7, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ7, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ7, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ7, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ7, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ7, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DM, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DM, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DM, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DM, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DM, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DM, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DM, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_EDC, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_EDC, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_EDC, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_EDC, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_EDC, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_EDC, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_EDC, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSP, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSP, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSP, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSP, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSP, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSP, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSP, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSN, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSN, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSN, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSN, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSN, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSN, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSN, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMP, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMP, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMP, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMP, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMP, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMP, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMP, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMN, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMN, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMN, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMN, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMN, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMN, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMN, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCP, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCP, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCP, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCP, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCP, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCP, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCP, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCN, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCN, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCN, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCN, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCN, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCN, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCN, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS0, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS0, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS0, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS0, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS0, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS0, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS0, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS1, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS1, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS1, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS1, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS1, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS1, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CLK_CONTROL, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CLK_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CLK_CONTROL, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CLK_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CLK_CONTROL, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CLK_CONTROL, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_CLK_CONTROL, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_LDE_CONTROL, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_LDE_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_LDE_CONTROL, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_LDE_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_LDE_CONTROL, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_LDE_CONTROL, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_VDL_LDE_CONTROL, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_RD_EN_DLY_CYC, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_RD_EN_DLY_CYC, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_RD_EN_DLY_CYC, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_RD_EN_DLY_CYC, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_RD_EN_DLY_CYC, CS1_CYCLES); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_RD_EN_DLY_CYC, CS0_CYCLES); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_WR_CHAN_DLY_CYC, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_WR_CHAN_DLY_CYC, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_WR_CHAN_DLY_CYC, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_WR_CHAN_DLY_CYC, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_WR_CHAN_DLY_CYC, CYCLES); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_READ_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_READ_CONTROL, MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_READ_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_READ_CONTROL, RD_DATA_DLY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_READ_FIFO_ADDR, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_READ_FIFO_ADDR, ADDR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_READ_FIFO_DATA, DATA); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_READ_FIFO_DM_DBI, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_READ_FIFO_DM_DBI, DM_DBI); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_READ_FIFO_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_READ_FIFO_STATUS, OVERFLOW); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_READ_FIFO_STATUS, UNDERFLOW); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_READ_FIFO_CLEAR, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_READ_FIFO_CLEAR, CLEAR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_IDLE_PAD_CONTROL, IDLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_IDLE_PAD_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_IDLE_PAD_CONTROL, AUTO_DQ_RXENB_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_IDLE_PAD_CONTROL, AUTO_DQ_IDDQ_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_IDLE_PAD_CONTROL, RESERVED_for_padding1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_IDLE_PAD_CONTROL, IO_IDLE_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_IDLE_PAD_CONTROL, RXENB); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_IDLE_PAD_CONTROL, IDDQ); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_IDLE_PAD_CONTROL, DOUT_N); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_IDLE_PAD_CONTROL, DOUT_P); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_DRIVE_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_DRIVE_PAD_CTL, BL_PD_IDLE_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_DRIVE_PAD_CTL, BL_ND_IDLE_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_DRIVE_PAD_CTL, BL_PD_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_DRIVE_PAD_CTL, BL_ND_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_DRIVE_PAD_CTL, BL_PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_DRIVE_PAD_CTL, BL_ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_RD_EN_DRIVE_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_RD_EN_DRIVE_PAD_CTL, EDC_RD_EN_PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_RD_EN_DRIVE_PAD_CTL, EDC_RD_EN_ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_RD_EN_DRIVE_PAD_CTL, BL_RD_EN_PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_RD_EN_DRIVE_PAD_CTL, BL_RD_EN_ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_STATIC_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_STATIC_PAD_CTL, DQS_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_STATIC_PAD_CTL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_STATIC_PAD_CTL, EDC_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_STATIC_PAD_CTL, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_STATIC_PAD_CTL, WDBI_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_STATIC_PAD_CTL, RDBI_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_STATIC_PAD_CTL, DM_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_STATIC_PAD_CTL, RESERVED3); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_STATIC_PAD_CTL, DQS_TX_DIS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_STATIC_PAD_CTL, RESERVED4); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_STATIC_PAD_CTL, RX_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_WR_PREAMBLE_MODE, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_WR_PREAMBLE_MODE, DQ_POSTAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_WR_PREAMBLE_MODE, DQ_PREAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_WR_PREAMBLE_MODE, DQS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_WR_PREAMBLE_MODE, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_WR_PREAMBLE_MODE, DQS_POSTAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_WR_PREAMBLE_MODE, DQS_PREAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_ODT_CONTROL, RESERVED_for_padding0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_ODT_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_ODT_CONTROL, ODT_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_ODT_CONTROL, ODT_DELAY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_ODT_CONTROL, ODT_POST_LENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_ODT_CONTROL, ODT_PRE_LENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_DPD_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_DPD_CONTROL, DQS_PHASE_OVERRIDE_VALUE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_DPD_CONTROL, DQS_PHASE_OVERRIDE_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_DPD_CONTROL, SET_ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_DPD_CONTROL, UPDATE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_DPD_CONTROL, MONITOR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_DPD_CONTROL, INIT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_DPD_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_DPD_STATUS, CURRENT_DQS_PHASE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_DPD_STATUS, CURRENT_VDL_SETTING); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_DPD_STATUS, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_DPD_STATUS, INIT_DQS_PHASE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_DPD_STATUS, INIT_VDL_SETTING); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_DPD_STATUS, EDGE_ERROR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_DPD_STATUS, MONITOR_BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_DPD_STATUS, INIT_BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_DPD_STATUS, INIT_DONE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_DPD_OUT_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_DPD_OUT_CONTROL, LOWER_LIMIT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_DPD_OUT_CONTROL, UPPER_LIMIT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_DPD_OUT_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_DPD_OUT_CONTROL, ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_DPD_OUT_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_DPD_OUT_STATUS, VDL_SETTING); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_DPD_OUT_STATUS, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_DPD_OUT_STATUS, VALID); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_DPD_OUT_STATUS, CLEAR_RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_DPD_OUT_STATUS, CLEAR_CLEAR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_DPD_OUT_STATUS_CLEAR, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_DPD_OUT_STATUS_CLEAR, CLEAR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_CRC_CONTROL, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_CRC_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_CRC_CONTROL, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_CRC_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_CRC_CONTROL, CRCWL); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_CRC_CONTROL, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_CRC_CONTROL, CRCRL); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_CRC_CONTROL, RESERVED3); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_CRC_CONTROL, COUNT_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_CRC_CONTROL, ENABLE_WR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_CRC_CONTROL, ENABLE_RD); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_CRC_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_CRC_STATUS, WR_FAIL); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_CRC_STATUS, WR_PASS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_CRC_STATUS, RD_FAIL); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_CRC_STATUS, RD_PASS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_CRC_STATUS, CLEAR_RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_CRC_STATUS, CLEAR_CLEAR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_CRC_COUNT, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_CRC_COUNT, WR_COUNT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_CRC_COUNT, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_CRC_COUNT, RD_COUNT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_CRC_STATUS_CLEAR, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_EDC_CRC_STATUS_CLEAR, CLEAR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_0_BL_SPARE_REG, RESERVED_for_eco0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ0, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ0, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ0, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ0, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ0, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ0, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ0, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ1, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ1, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ1, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ1, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ1, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ1, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ2, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ2, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ2, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ2, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ2, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ2, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ2, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ3, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ3, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ3, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ3, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ3, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ3, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ3, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ4, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ4, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ4, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ4, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ4, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ4, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ4, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ5, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ5, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ5, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ5, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ5, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ5, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ5, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ6, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ6, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ6, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ6, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ6, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ6, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ6, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ7, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ7, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ7, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ7, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ7, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ7, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ7, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DM, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DM, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DM, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DM, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DM, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DM, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DM, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_EDC, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_EDC, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_EDC, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_EDC, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_EDC, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_EDC, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_EDC, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSP, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSP, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSP, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSP, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSP, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSP, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSP, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSN, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSN, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSN, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSN, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSN, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSN, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSN, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMP, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMP, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMP, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMP, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMP, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMP, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMP, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMN, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMN, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMN, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMN, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMN, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMN, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMN, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCP, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCP, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCP, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCP, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCP, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCP, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCP, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCN, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCN, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCN, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCN, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCN, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCN, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCN, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS0, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS0, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS0, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS0, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS0, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS0, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS0, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS1, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS1, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS1, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS1, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS1, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS1, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CLK_CONTROL, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CLK_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CLK_CONTROL, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CLK_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CLK_CONTROL, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CLK_CONTROL, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_CLK_CONTROL, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_LDE_CONTROL, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_LDE_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_LDE_CONTROL, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_LDE_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_LDE_CONTROL, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_LDE_CONTROL, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_VDL_LDE_CONTROL, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_RD_EN_DLY_CYC, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_RD_EN_DLY_CYC, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_RD_EN_DLY_CYC, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_RD_EN_DLY_CYC, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_RD_EN_DLY_CYC, CS1_CYCLES); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_RD_EN_DLY_CYC, CS0_CYCLES); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_WR_CHAN_DLY_CYC, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_WR_CHAN_DLY_CYC, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_WR_CHAN_DLY_CYC, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_WR_CHAN_DLY_CYC, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_WR_CHAN_DLY_CYC, CYCLES); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_READ_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_READ_CONTROL, MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_READ_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_READ_CONTROL, RD_DATA_DLY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_READ_FIFO_ADDR, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_READ_FIFO_ADDR, ADDR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_READ_FIFO_DATA, DATA); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_READ_FIFO_DM_DBI, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_READ_FIFO_DM_DBI, DM_DBI); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_READ_FIFO_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_READ_FIFO_STATUS, OVERFLOW); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_READ_FIFO_STATUS, UNDERFLOW); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_READ_FIFO_CLEAR, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_READ_FIFO_CLEAR, CLEAR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_IDLE_PAD_CONTROL, IDLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_IDLE_PAD_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_IDLE_PAD_CONTROL, AUTO_DQ_RXENB_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_IDLE_PAD_CONTROL, AUTO_DQ_IDDQ_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_IDLE_PAD_CONTROL, RESERVED_for_padding1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_IDLE_PAD_CONTROL, IO_IDLE_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_IDLE_PAD_CONTROL, RXENB); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_IDLE_PAD_CONTROL, IDDQ); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_IDLE_PAD_CONTROL, DOUT_N); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_IDLE_PAD_CONTROL, DOUT_P); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_DRIVE_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_DRIVE_PAD_CTL, BL_PD_IDLE_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_DRIVE_PAD_CTL, BL_ND_IDLE_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_DRIVE_PAD_CTL, BL_PD_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_DRIVE_PAD_CTL, BL_ND_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_DRIVE_PAD_CTL, BL_PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_DRIVE_PAD_CTL, BL_ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_RD_EN_DRIVE_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_RD_EN_DRIVE_PAD_CTL, EDC_RD_EN_PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_RD_EN_DRIVE_PAD_CTL, EDC_RD_EN_ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_RD_EN_DRIVE_PAD_CTL, BL_RD_EN_PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_RD_EN_DRIVE_PAD_CTL, BL_RD_EN_ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_STATIC_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_STATIC_PAD_CTL, DQS_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_STATIC_PAD_CTL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_STATIC_PAD_CTL, EDC_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_STATIC_PAD_CTL, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_STATIC_PAD_CTL, WDBI_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_STATIC_PAD_CTL, RDBI_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_STATIC_PAD_CTL, DM_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_STATIC_PAD_CTL, RESERVED3); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_STATIC_PAD_CTL, DQS_TX_DIS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_STATIC_PAD_CTL, RESERVED4); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_STATIC_PAD_CTL, RX_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_WR_PREAMBLE_MODE, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_WR_PREAMBLE_MODE, DQ_POSTAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_WR_PREAMBLE_MODE, DQ_PREAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_WR_PREAMBLE_MODE, DQS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_WR_PREAMBLE_MODE, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_WR_PREAMBLE_MODE, DQS_POSTAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_WR_PREAMBLE_MODE, DQS_PREAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_ODT_CONTROL, RESERVED_for_padding0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_ODT_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_ODT_CONTROL, ODT_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_ODT_CONTROL, ODT_DELAY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_ODT_CONTROL, ODT_POST_LENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_ODT_CONTROL, ODT_PRE_LENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_DPD_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_DPD_CONTROL, DQS_PHASE_OVERRIDE_VALUE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_DPD_CONTROL, DQS_PHASE_OVERRIDE_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_DPD_CONTROL, SET_ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_DPD_CONTROL, UPDATE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_DPD_CONTROL, MONITOR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_DPD_CONTROL, INIT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_DPD_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_DPD_STATUS, CURRENT_DQS_PHASE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_DPD_STATUS, CURRENT_VDL_SETTING); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_DPD_STATUS, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_DPD_STATUS, INIT_DQS_PHASE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_DPD_STATUS, INIT_VDL_SETTING); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_DPD_STATUS, EDGE_ERROR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_DPD_STATUS, MONITOR_BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_DPD_STATUS, INIT_BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_DPD_STATUS, INIT_DONE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_DPD_OUT_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_DPD_OUT_CONTROL, LOWER_LIMIT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_DPD_OUT_CONTROL, UPPER_LIMIT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_DPD_OUT_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_DPD_OUT_CONTROL, ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_DPD_OUT_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_DPD_OUT_STATUS, VDL_SETTING); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_DPD_OUT_STATUS, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_DPD_OUT_STATUS, VALID); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_DPD_OUT_STATUS, CLEAR_RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_DPD_OUT_STATUS, CLEAR_CLEAR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_DPD_OUT_STATUS_CLEAR, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_DPD_OUT_STATUS_CLEAR, CLEAR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_CRC_CONTROL, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_CRC_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_CRC_CONTROL, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_CRC_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_CRC_CONTROL, CRCWL); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_CRC_CONTROL, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_CRC_CONTROL, CRCRL); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_CRC_CONTROL, RESERVED3); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_CRC_CONTROL, COUNT_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_CRC_CONTROL, ENABLE_WR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_CRC_CONTROL, ENABLE_RD); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_CRC_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_CRC_STATUS, WR_FAIL); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_CRC_STATUS, WR_PASS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_CRC_STATUS, RD_FAIL); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_CRC_STATUS, RD_PASS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_CRC_STATUS, CLEAR_RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_CRC_STATUS, CLEAR_CLEAR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_CRC_COUNT, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_CRC_COUNT, WR_COUNT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_CRC_COUNT, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_CRC_COUNT, RD_COUNT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_CRC_STATUS_CLEAR, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_EDC_CRC_STATUS_CLEAR, CLEAR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_1_BL_SPARE_REG, RESERVED_for_eco0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ0, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ0, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ0, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ0, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ0, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ0, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ0, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ1, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ1, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ1, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ1, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ1, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ1, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ2, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ2, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ2, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ2, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ2, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ2, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ2, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ3, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ3, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ3, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ3, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ3, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ3, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ3, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ4, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ4, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ4, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ4, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ4, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ4, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ4, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ5, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ5, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ5, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ5, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ5, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ5, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ5, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ6, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ6, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ6, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ6, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ6, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ6, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ6, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ7, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ7, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ7, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ7, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ7, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ7, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ7, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DM, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DM, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DM, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DM, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DM, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DM, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DM, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_EDC, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_EDC, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_EDC, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_EDC, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_EDC, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_EDC, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_EDC, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSP, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSP, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSP, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSP, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSP, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSP, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSP, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSN, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSN, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSN, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSN, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSN, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSN, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSN, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMP, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMP, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMP, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMP, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMP, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMP, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMP, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMN, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMN, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMN, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMN, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMN, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMN, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMN, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCP, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCP, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCP, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCP, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCP, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCP, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCP, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCN, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCN, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCN, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCN, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCN, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCN, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCN, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS0, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS0, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS0, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS0, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS0, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS0, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS0, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS1, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS1, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS1, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS1, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS1, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS1, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CLK_CONTROL, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CLK_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CLK_CONTROL, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CLK_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CLK_CONTROL, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CLK_CONTROL, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_CLK_CONTROL, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_LDE_CONTROL, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_LDE_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_LDE_CONTROL, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_LDE_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_LDE_CONTROL, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_LDE_CONTROL, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_VDL_LDE_CONTROL, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_RD_EN_DLY_CYC, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_RD_EN_DLY_CYC, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_RD_EN_DLY_CYC, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_RD_EN_DLY_CYC, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_RD_EN_DLY_CYC, CS1_CYCLES); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_RD_EN_DLY_CYC, CS0_CYCLES); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_WR_CHAN_DLY_CYC, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_WR_CHAN_DLY_CYC, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_WR_CHAN_DLY_CYC, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_WR_CHAN_DLY_CYC, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_WR_CHAN_DLY_CYC, CYCLES); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_READ_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_READ_CONTROL, MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_READ_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_READ_CONTROL, RD_DATA_DLY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_READ_FIFO_ADDR, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_READ_FIFO_ADDR, ADDR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_READ_FIFO_DATA, DATA); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_READ_FIFO_DM_DBI, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_READ_FIFO_DM_DBI, DM_DBI); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_READ_FIFO_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_READ_FIFO_STATUS, OVERFLOW); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_READ_FIFO_STATUS, UNDERFLOW); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_READ_FIFO_CLEAR, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_READ_FIFO_CLEAR, CLEAR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_IDLE_PAD_CONTROL, IDLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_IDLE_PAD_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_IDLE_PAD_CONTROL, AUTO_DQ_RXENB_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_IDLE_PAD_CONTROL, AUTO_DQ_IDDQ_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_IDLE_PAD_CONTROL, RESERVED_for_padding1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_IDLE_PAD_CONTROL, IO_IDLE_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_IDLE_PAD_CONTROL, RXENB); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_IDLE_PAD_CONTROL, IDDQ); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_IDLE_PAD_CONTROL, DOUT_N); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_IDLE_PAD_CONTROL, DOUT_P); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_DRIVE_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_DRIVE_PAD_CTL, BL_PD_IDLE_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_DRIVE_PAD_CTL, BL_ND_IDLE_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_DRIVE_PAD_CTL, BL_PD_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_DRIVE_PAD_CTL, BL_ND_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_DRIVE_PAD_CTL, BL_PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_DRIVE_PAD_CTL, BL_ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_RD_EN_DRIVE_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_RD_EN_DRIVE_PAD_CTL, EDC_RD_EN_PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_RD_EN_DRIVE_PAD_CTL, EDC_RD_EN_ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_RD_EN_DRIVE_PAD_CTL, BL_RD_EN_PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_RD_EN_DRIVE_PAD_CTL, BL_RD_EN_ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_STATIC_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_STATIC_PAD_CTL, DQS_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_STATIC_PAD_CTL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_STATIC_PAD_CTL, EDC_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_STATIC_PAD_CTL, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_STATIC_PAD_CTL, WDBI_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_STATIC_PAD_CTL, RDBI_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_STATIC_PAD_CTL, DM_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_STATIC_PAD_CTL, RESERVED3); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_STATIC_PAD_CTL, DQS_TX_DIS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_STATIC_PAD_CTL, RESERVED4); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_STATIC_PAD_CTL, RX_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_WR_PREAMBLE_MODE, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_WR_PREAMBLE_MODE, DQ_POSTAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_WR_PREAMBLE_MODE, DQ_PREAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_WR_PREAMBLE_MODE, DQS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_WR_PREAMBLE_MODE, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_WR_PREAMBLE_MODE, DQS_POSTAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_WR_PREAMBLE_MODE, DQS_PREAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_ODT_CONTROL, RESERVED_for_padding0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_ODT_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_ODT_CONTROL, ODT_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_ODT_CONTROL, ODT_DELAY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_ODT_CONTROL, ODT_POST_LENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_ODT_CONTROL, ODT_PRE_LENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_DPD_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_DPD_CONTROL, DQS_PHASE_OVERRIDE_VALUE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_DPD_CONTROL, DQS_PHASE_OVERRIDE_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_DPD_CONTROL, SET_ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_DPD_CONTROL, UPDATE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_DPD_CONTROL, MONITOR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_DPD_CONTROL, INIT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_DPD_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_DPD_STATUS, CURRENT_DQS_PHASE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_DPD_STATUS, CURRENT_VDL_SETTING); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_DPD_STATUS, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_DPD_STATUS, INIT_DQS_PHASE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_DPD_STATUS, INIT_VDL_SETTING); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_DPD_STATUS, EDGE_ERROR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_DPD_STATUS, MONITOR_BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_DPD_STATUS, INIT_BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_DPD_STATUS, INIT_DONE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_DPD_OUT_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_DPD_OUT_CONTROL, LOWER_LIMIT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_DPD_OUT_CONTROL, UPPER_LIMIT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_DPD_OUT_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_DPD_OUT_CONTROL, ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_DPD_OUT_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_DPD_OUT_STATUS, VDL_SETTING); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_DPD_OUT_STATUS, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_DPD_OUT_STATUS, VALID); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_DPD_OUT_STATUS, CLEAR_RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_DPD_OUT_STATUS, CLEAR_CLEAR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_DPD_OUT_STATUS_CLEAR, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_DPD_OUT_STATUS_CLEAR, CLEAR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_CRC_CONTROL, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_CRC_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_CRC_CONTROL, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_CRC_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_CRC_CONTROL, CRCWL); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_CRC_CONTROL, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_CRC_CONTROL, CRCRL); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_CRC_CONTROL, RESERVED3); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_CRC_CONTROL, COUNT_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_CRC_CONTROL, ENABLE_WR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_CRC_CONTROL, ENABLE_RD); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_CRC_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_CRC_STATUS, WR_FAIL); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_CRC_STATUS, WR_PASS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_CRC_STATUS, RD_FAIL); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_CRC_STATUS, RD_PASS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_CRC_STATUS, CLEAR_RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_CRC_STATUS, CLEAR_CLEAR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_CRC_COUNT, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_CRC_COUNT, WR_COUNT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_CRC_COUNT, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_CRC_COUNT, RD_COUNT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_CRC_STATUS_CLEAR, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_EDC_CRC_STATUS_CLEAR, CLEAR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_2_BL_SPARE_REG, RESERVED_for_eco0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ0, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ0, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ0, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ0, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ0, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ0, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ0, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ1, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ1, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ1, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ1, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ1, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ1, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ2, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ2, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ2, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ2, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ2, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ2, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ2, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ3, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ3, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ3, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ3, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ3, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ3, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ3, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ4, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ4, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ4, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ4, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ4, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ4, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ4, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ5, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ5, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ5, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ5, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ5, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ5, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ5, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ6, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ6, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ6, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ6, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ6, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ6, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ6, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ7, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ7, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ7, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ7, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ7, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ7, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ7, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DM, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DM, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DM, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DM, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DM, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DM, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DM, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_EDC, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_EDC, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_EDC, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_EDC, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_EDC, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_EDC, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_EDC, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSP, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSP, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSP, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSP, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSP, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSP, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSP, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSN, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSN, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSN, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSN, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSN, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSN, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSN, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMP, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMP, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMP, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMP, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMP, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMP, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMP, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMN, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMN, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMN, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMN, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMN, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMN, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMN, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCP, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCP, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCP, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCP, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCP, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCP, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCP, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCN, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCN, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCN, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCN, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCN, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCN, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCN, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS0, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS0, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS0, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS0, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS0, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS0, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS0, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS1, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS1, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS1, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS1, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS1, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS1, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CLK_CONTROL, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CLK_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CLK_CONTROL, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CLK_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CLK_CONTROL, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CLK_CONTROL, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_CLK_CONTROL, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_LDE_CONTROL, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_LDE_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_LDE_CONTROL, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_LDE_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_LDE_CONTROL, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_LDE_CONTROL, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_VDL_LDE_CONTROL, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_RD_EN_DLY_CYC, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_RD_EN_DLY_CYC, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_RD_EN_DLY_CYC, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_RD_EN_DLY_CYC, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_RD_EN_DLY_CYC, CS1_CYCLES); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_RD_EN_DLY_CYC, CS0_CYCLES); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_WR_CHAN_DLY_CYC, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_WR_CHAN_DLY_CYC, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_WR_CHAN_DLY_CYC, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_WR_CHAN_DLY_CYC, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_WR_CHAN_DLY_CYC, CYCLES); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_READ_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_READ_CONTROL, MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_READ_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_READ_CONTROL, RD_DATA_DLY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_READ_FIFO_ADDR, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_READ_FIFO_ADDR, ADDR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_READ_FIFO_DATA, DATA); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_READ_FIFO_DM_DBI, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_READ_FIFO_DM_DBI, DM_DBI); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_READ_FIFO_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_READ_FIFO_STATUS, OVERFLOW); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_READ_FIFO_STATUS, UNDERFLOW); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_READ_FIFO_CLEAR, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_READ_FIFO_CLEAR, CLEAR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_IDLE_PAD_CONTROL, IDLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_IDLE_PAD_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_IDLE_PAD_CONTROL, AUTO_DQ_RXENB_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_IDLE_PAD_CONTROL, AUTO_DQ_IDDQ_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_IDLE_PAD_CONTROL, RESERVED_for_padding1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_IDLE_PAD_CONTROL, IO_IDLE_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_IDLE_PAD_CONTROL, RXENB); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_IDLE_PAD_CONTROL, IDDQ); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_IDLE_PAD_CONTROL, DOUT_N); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_IDLE_PAD_CONTROL, DOUT_P); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_DRIVE_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_DRIVE_PAD_CTL, BL_PD_IDLE_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_DRIVE_PAD_CTL, BL_ND_IDLE_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_DRIVE_PAD_CTL, BL_PD_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_DRIVE_PAD_CTL, BL_ND_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_DRIVE_PAD_CTL, BL_PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_DRIVE_PAD_CTL, BL_ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_RD_EN_DRIVE_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_RD_EN_DRIVE_PAD_CTL, EDC_RD_EN_PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_RD_EN_DRIVE_PAD_CTL, EDC_RD_EN_ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_RD_EN_DRIVE_PAD_CTL, BL_RD_EN_PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_RD_EN_DRIVE_PAD_CTL, BL_RD_EN_ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_STATIC_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_STATIC_PAD_CTL, DQS_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_STATIC_PAD_CTL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_STATIC_PAD_CTL, EDC_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_STATIC_PAD_CTL, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_STATIC_PAD_CTL, WDBI_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_STATIC_PAD_CTL, RDBI_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_STATIC_PAD_CTL, DM_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_STATIC_PAD_CTL, RESERVED3); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_STATIC_PAD_CTL, DQS_TX_DIS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_STATIC_PAD_CTL, RESERVED4); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_STATIC_PAD_CTL, RX_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_WR_PREAMBLE_MODE, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_WR_PREAMBLE_MODE, DQ_POSTAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_WR_PREAMBLE_MODE, DQ_PREAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_WR_PREAMBLE_MODE, DQS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_WR_PREAMBLE_MODE, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_WR_PREAMBLE_MODE, DQS_POSTAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_WR_PREAMBLE_MODE, DQS_PREAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_ODT_CONTROL, RESERVED_for_padding0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_ODT_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_ODT_CONTROL, ODT_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_ODT_CONTROL, ODT_DELAY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_ODT_CONTROL, ODT_POST_LENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_ODT_CONTROL, ODT_PRE_LENGTH); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_DPD_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_DPD_CONTROL, DQS_PHASE_OVERRIDE_VALUE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_DPD_CONTROL, DQS_PHASE_OVERRIDE_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_DPD_CONTROL, SET_ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_DPD_CONTROL, UPDATE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_DPD_CONTROL, MONITOR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_DPD_CONTROL, INIT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_DPD_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_DPD_STATUS, CURRENT_DQS_PHASE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_DPD_STATUS, CURRENT_VDL_SETTING); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_DPD_STATUS, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_DPD_STATUS, INIT_DQS_PHASE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_DPD_STATUS, INIT_VDL_SETTING); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_DPD_STATUS, EDGE_ERROR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_DPD_STATUS, MONITOR_BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_DPD_STATUS, INIT_BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_DPD_STATUS, INIT_DONE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_DPD_OUT_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_DPD_OUT_CONTROL, LOWER_LIMIT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_DPD_OUT_CONTROL, UPPER_LIMIT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_DPD_OUT_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_DPD_OUT_CONTROL, ENABLE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_DPD_OUT_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_DPD_OUT_STATUS, VDL_SETTING); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_DPD_OUT_STATUS, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_DPD_OUT_STATUS, VALID); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_DPD_OUT_STATUS, CLEAR_RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_DPD_OUT_STATUS, CLEAR_CLEAR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_DPD_OUT_STATUS_CLEAR, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_DPD_OUT_STATUS_CLEAR, CLEAR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_CRC_CONTROL, BUSY); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_CRC_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_CRC_CONTROL, FORCE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_CRC_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_CRC_CONTROL, CRCWL); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_CRC_CONTROL, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_CRC_CONTROL, CRCRL); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_CRC_CONTROL, RESERVED3); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_CRC_CONTROL, COUNT_MODE); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_CRC_CONTROL, ENABLE_WR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_CRC_CONTROL, ENABLE_RD); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_CRC_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_CRC_STATUS, WR_FAIL); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_CRC_STATUS, WR_PASS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_CRC_STATUS, RD_FAIL); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_CRC_STATUS, RD_PASS); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_CRC_STATUS, CLEAR_RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_CRC_STATUS, CLEAR_CLEAR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_CRC_COUNT, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_CRC_COUNT, WR_COUNT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_CRC_COUNT, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_CRC_COUNT, RD_COUNT); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_CRC_STATUS_CLEAR, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_EDC_CRC_STATUS_CLEAR, CLEAR); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_BYTE_LANE_3_BL_SPARE_REG, RESERVED_for_eco0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_IDLE_PAD_ENABLE0, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(E0, PHY_CONTROL_REGS_IDLE_PAD_ENABLE0, IO_IDLE_ENABLE); \
}


/**********************************************/
#define INIT_DDR34_F0_PHY_FIELDS_TABLE()   { \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_REVISION, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_REVISION, PERFORMANCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_REVISION, TECHNOLOGY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_REVISION, WB); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_REVISION, BITS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_REVISION, MAJOR); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_REVISION, MINOR); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_STATUS, LOCK_LOST); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_STATUS, CLOCKING_8X); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_STATUS, CLOCKING_4X); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_STATUS, CLOCKING_2X); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_STATUS, STATUS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_STATUS, LOCK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_CONFIG, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_CONFIG, PLL_LDO_REF_SEL); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_CONFIG, PLL_LDO_REF_CTRL); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_CONFIG, PLL_LDO_BIAS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_CONFIG, PLL_LDO_PWRDOWN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_CONFIG, HOLD); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_CONFIG, ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_CONFIG, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_CONFIG, ENABLE_EXT_CTRL); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_CONFIG, RESET_POST_DIV); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_CONFIG, ISO_OUT); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_CONFIG, ISO_IN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_CONFIG, RESET); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_CONFIG, PWRDN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_CONTROL1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_CONTROL1, I_KP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_CONTROL1, I_KI); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_CONTROL1, I_KA); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_CONTROL2, VCO_RANGE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_CONTROL2, UNUSED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_CONTROL2, NDIV_RELOCK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_CONTROL2, FAST_LOCK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_CONTROL2, VCO_FB_DIV2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_CONTROL2, POST_CTRL_RESETB); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_CONTROL2, PWM_RATE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_CONTROL2, STAT_MODE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_CONTROL2, UNUSED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_CONTROL2, STAT_UPDATE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_CONTROL2, STAT_SELECT); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_CONTROL2, STAT_RESET); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_CONTROL2, DCO_CTRL_BYPASS_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_CONTROL2, DCO_CTRL_BYPASS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_CONTROL3, PLL_CONTROL); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_DIVIDERS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_DIVIDERS, MDIV); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_DIVIDERS, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_DIVIDERS, PDIV); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_DIVIDERS, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_DIVIDERS, NDIV_INT); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_FRAC_DIVIDER, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_FRAC_DIVIDER, NDIV_FRAC); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_SS_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_SS_CONTROL, SSC_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_SS_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_SS_CONTROL, SSC_MODE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_SS_LIMIT, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_SS_LIMIT, SSC_LIMIT); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PLL_SS_LIMIT, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_AUX_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_AUX_CONTROL, AUX0_IS_ODT); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_AUX_CONTROL, PAR_IS_ODT); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_AUX_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_IDLE_PAD_CONTROL, IDLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_IDLE_PAD_CONTROL, DIB_MODE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_IDLE_PAD_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_IDLE_PAD_CONTROL, RXENB); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_IDLE_PAD_CONTROL, IDDQ); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_IDLE_PAD_CONTROL, DOUT_N); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_IDLE_PAD_CONTROL, DOUT_P); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_IDLE_PAD_ENABLE0, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_IDLE_PAD_ENABLE0, IO_IDLE_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_IDLE_PAD_ENABLE1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_IDLE_PAD_ENABLE1, IO_IDLE_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRIVE_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRIVE_PAD_CTL, ADDR_CTL_PD_IDLE_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRIVE_PAD_CTL, ADDR_CTL_ND_IDLE_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRIVE_PAD_CTL, ADDR_CTL_PD_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRIVE_PAD_CTL, ADDR_CTL_ND_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRIVE_PAD_CTL, ADDR_CTL_PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRIVE_PAD_CTL, ADDR_CTL_ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRIVE_PAD_CTL, 2T_RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRIVE_PAD_CTL, 2T_PD_IDLE_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRIVE_PAD_CTL, 2T_ND_IDLE_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRIVE_PAD_CTL, 2T_PD_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRIVE_PAD_CTL, 2T_ND_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRIVE_PAD_CTL, 2T_PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRIVE_PAD_CTL, 2T_ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRIVE_PAD_CTL_2T, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRIVE_PAD_CTL_2T, PD_IDLE_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRIVE_PAD_CTL_2T, ND_IDLE_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRIVE_PAD_CTL_2T, PD_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRIVE_PAD_CTL_2T, ND_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRIVE_PAD_CTL_2T, PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRIVE_PAD_CTL_2T, ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_STATIC_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_STATIC_PAD_CTL, AUTO_OEB); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_STATIC_PAD_CTL, IDDQ_LPDDR); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_STATIC_PAD_CTL, IDDQ_CLK0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_STATIC_PAD_CTL, IDDQ_ODT); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_STATIC_PAD_CTL, IDDQ_PAR); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_STATIC_PAD_CTL, IDDQ_BA); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_STATIC_PAD_CTL, IDDQ_AUX2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_STATIC_PAD_CTL, IDDQ_AUX1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_STATIC_PAD_CTL, IDDQ_AUX0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_STATIC_PAD_CTL, IDDQ_CS1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_STATIC_PAD_CTL, IDDQ_A15); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_STATIC_PAD_CTL, IDDQ_A14); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_STATIC_PAD_CTL, IDDQ_A13); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_STATIC_PAD_CTL, IDDQ_A12); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_STATIC_PAD_CTL, IDDQ_A11); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_STATIC_PAD_CTL, IDDQ_A10); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_STATIC_PAD_CTL, IDDQ_A09); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_STATIC_PAD_CTL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_STATIC_PAD_CTL, RX_MODE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRAM_CONFIG, INIT_MODE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRAM_CONFIG, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRAM_CONFIG, SPLIT_DQ_BUS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRAM_CONFIG, BUS16); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRAM_CONFIG, BUS8); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRAM_CONFIG, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRAM_CONFIG, DDR4_GEARDOWN_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRAM_CONFIG, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRAM_CONFIG, RDQS_MODE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRAM_CONFIG, GROUP_BITS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRAM_CONFIG, BANK_BITS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRAM_CONFIG, COL_BITS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRAM_CONFIG, ROW_BITS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRAM_CONFIG, DRAM_TYPE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRAM_TIMING1, TRAS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRAM_TIMING1, TRRD); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRAM_TIMING1, TRP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRAM_TIMING1, TRCD); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRAM_TIMING2, TRTP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRAM_TIMING2, TWR); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRAM_TIMING2, TCWL); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRAM_TIMING2, TCAS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRAM_TIMING3, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRAM_TIMING3, TCAL); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRAM_TIMING3, TRTW); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRAM_TIMING3, TWTR); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRAM_TIMING3, TRFC); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DRAM_TIMING4, TEMP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CALIBRATE, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CALIBRATE, CALIB_FAST); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CALIBRATE, HALF_STEPS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CALIBRATE, UPDATE_FAST); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CALIBRATE, UPDATE_REGS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CALIBRATE, CALIB_FTM2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CALIBRATE, CALIB_PHYBIST); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CALIBRATE, CALIB_ONCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CALIB_STATUS1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CALIB_STATUS1, CALIB_TOTAL_STEPS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CALIB_STATUS1, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CALIB_STATUS1, CALIB_BUS_ERROR); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CALIB_STATUS1, CALIB_REGS_DONE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CALIB_STATUS1, CALIB_LOCK_6B); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CALIB_STATUS1, CALIB_LOCK_4B); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CALIB_STATUS1, CALIB_LOCK_2B); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CALIB_STATUS1, CALIB_IDLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CALIB_STATUS2, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CALIB_STATUS2, CALIB_4B_STEPS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CALIB_STATUS2, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CALIB_STATUS2, CALIB_2B_STEPS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_CONTROL, INTERVAL); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_CONTROL, UPDATE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_CONTROL, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_CONTROL, DATA_RATE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_CONTROL, ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_REF, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_REF, MONITOR_4B_STEPS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_REF, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_REF, MONITOR_2B_STEPS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_STATUS, MONITOR_CHANGE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_STATUS, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_STATUS, MONITOR_TOTAL); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_STATUS, ERROR); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_STATUS, PAUSED); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_STATUS, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_OVERRIDE, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_OVERRIDE, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_OVERRIDE, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_OVERRIDE, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_OVERRIDE, ADJ); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_OUT_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_OUT_CONTROL, LOWER_LIMIT); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_OUT_CONTROL, UPPER_LIMIT); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_OUT_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_OUT_CONTROL, ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUS, MONITOR_CHANGE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUS, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUS, MONITOR_TOTAL); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUS, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUS, VALID); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUS, CLEAR_RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUS, CLEAR_CLEAR); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUS_CLEAR, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUS_CLEAR, CLEAR); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD00, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD00, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD00, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD00, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD00, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD00, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD00, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD01, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD01, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD01, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD01, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD01, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD01, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD01, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD02, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD02, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD02, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD02, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD02, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD02, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD02, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD03, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD03, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD03, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD03, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD03, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD03, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD03, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD04, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD04, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD04, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD04, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD04, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD04, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD04, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD05, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD05, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD05, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD05, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD05, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD05, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD05, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD06, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD06, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD06, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD06, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD06, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD06, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD06, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD07, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD07, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD07, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD07, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD07, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD07, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD07, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD08, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD08, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD08, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD08, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD08, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD08, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD08, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD09, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD09, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD09, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD09, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD09, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD09, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD09, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD10, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD10, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD10, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD10, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD10, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD10, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD10, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD11, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD11, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD11, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD11, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD11, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD11, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD11, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD12, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD12, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD12, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD12, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD12, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD12, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD12, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD13, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD13, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD13, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD13, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD13, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD13, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD13, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD14, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD14, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD14, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD14, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD14, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD14, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD14, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD15, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD15, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD15, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD15, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD15, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD15, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AD15, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_BA0, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_BA0, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_BA0, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_BA0, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_BA0, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_BA0, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_BA0, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_BA1, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_BA1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_BA1, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_BA1, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_BA1, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_BA1, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_BA1, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_BA2, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_BA2, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_BA2, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_BA2, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_BA2, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_BA2, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_BA2, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AUX0, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AUX0, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AUX0, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AUX0, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AUX0, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AUX0, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AUX0, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AUX1, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AUX1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AUX1, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AUX1, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AUX1, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AUX1, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AUX1, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AUX2, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AUX2, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AUX2, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AUX2, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AUX2, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AUX2, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_AUX2, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_CS0, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_CS0, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_CS0, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_CS0, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_CS0, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_CS0, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_CS0, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_CS1, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_CS1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_CS1, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_CS1, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_CS1, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_CS1, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_CS1, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_PAR, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_PAR, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_PAR, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_PAR, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_PAR, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_PAR, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_PAR, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_RAS_N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_RAS_N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_RAS_N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_RAS_N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_RAS_N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_RAS_N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_RAS_N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_CAS_N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_CAS_N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_CAS_N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_CAS_N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_CAS_N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_CAS_N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_CAS_N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_CKE, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_CKE, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_CKE, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_CKE, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_CKE, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_CKE, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_CKE, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_RST_N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_RST_N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_RST_N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_RST_N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_RST_N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_RST_N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_RST_N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_ODT, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_ODT, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_ODT, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_ODT, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_ODT, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_ODT, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_ODT, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_WE_N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_WE_N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_WE_N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_WE_N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_WE_N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_WE_N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_WE_N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CLK_CONTROL, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CLK_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CLK_CONTROL, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CLK_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CLK_CONTROL, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CLK_CONTROL, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_CLK_CONTROL, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_LDE_CONTROL, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_LDE_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_LDE_CONTROL, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_LDE_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_LDE_CONTROL, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_LDE_CONTROL, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VDL_LDE_CONTROL, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_CMD_CYC_DLY, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_CMD_CYC_DLY, CKE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_CMD_CYC_DLY, WE_N); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_CMD_CYC_DLY, CAS_N); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_CMD_CYC_DLY, RAS_N); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_CMD_CYC_DLY, CS_N); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_UPDATE_VDL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_UPDATE_VDL, MODE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_UPDATE_VDL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_UPDATE_VDL, DISABLE_INPUT); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_UPDATE_VDL, ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_UPDATE_VDL, SNOOP1_RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_UPDATE_VDL, SNOOP1_MASK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_UPDATE_VDL, SNOOP1_RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_UPDATE_VDL, SNOOP1_CMD); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_UPDATE_VDL, SNOOP1_RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_UPDATE_VDL, SNOOP1_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_UPDATE_VDL, SNOOP2_RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_UPDATE_VDL, SNOOP2_MASK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_UPDATE_VDL, SNOOP2_RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_UPDATE_VDL, SNOOP2_CMD); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_UPDATE_VDL, SNOOP2_RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_UPDATE_VDL, SNOOP2_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_UPDATE_VDL_SNOOP1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_UPDATE_VDL_SNOOP1, MASK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_UPDATE_VDL_SNOOP1, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_UPDATE_VDL_SNOOP1, CMD); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_UPDATE_VDL_SNOOP1, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_UPDATE_VDL_SNOOP1, ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_UPDATE_VDL_SNOOP2, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_UPDATE_VDL_SNOOP2, MASK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_UPDATE_VDL_SNOOP2, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_UPDATE_VDL_SNOOP2, CMD); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_UPDATE_VDL_SNOOP2, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_UPDATE_VDL_SNOOP2, ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_REG1, MCP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_REG1, CS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_REG1, AUX); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_REG1, ACT); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_REG1, WE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_REG1, CAS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_REG1, RAS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_REG1, BA); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_REG1, AD); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_AUX_REG1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_AUX_REG1, AD16); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_REG2, MCP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_REG2, CS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_REG2, AUX); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_REG2, ACT); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_REG2, WE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_REG2, CAS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_REG2, RAS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_REG2, BA); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_REG2, AD); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_AUX_REG2, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_AUX_REG2, AD16); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_REG3, MCP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_REG3, CS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_REG3, AUX); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_REG3, ACT); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_REG3, WE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_REG3, CAS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_REG3, RAS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_REG3, BA); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_REG3, AD); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_AUX_REG3, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_AUX_REG3, AD16); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_REG4, MCP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_REG4, CS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_REG4, AUX); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_REG4, ACT); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_REG4, WE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_REG4, CAS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_REG4, RAS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_REG4, BA); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_REG4, AD); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_AUX_REG4, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_COMMAND_AUX_REG4, AD16); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_MODE_REG0, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_MODE_REG0, VALID); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_MODE_REG0, AD); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_MODE_REG1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_MODE_REG1, VALID); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_MODE_REG1, AD); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_MODE_REG2, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_MODE_REG2, VALID); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_MODE_REG2, AD); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_MODE_REG3, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_MODE_REG3, VALID); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_MODE_REG3, AD); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_MODE_REG4, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_MODE_REG4, VALID); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_MODE_REG4, AD); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_MODE_REG5, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_MODE_REG5, VALID); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_MODE_REG5, AD); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_MODE_REG6, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_MODE_REG6, VALID); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_MODE_REG6, AD); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_MODE_REG7, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_MODE_REG7, VALID); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_MODE_REG7, AD); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_MODE_REG8, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_MODE_REG8, VALID); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_MODE_REG8, AD); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_MODE_REG15, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_MODE_REG15, VALID); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_MODE_REG15, AD); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_MODE_REG63, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_MODE_REG63, VALID); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_MODE_REG63, AD); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ALERT_CLEAR, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ALERT_CLEAR, CLEAR); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ALERT_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ALERT_STATUS, BL3_STATE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ALERT_STATUS, BL2_STATE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ALERT_STATUS, BL1_STATE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ALERT_STATUS, BL0_STATE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ALERT_STATUS, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ALERT_STATUS, BL3_ALERT); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ALERT_STATUS, BL2_ALERT); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ALERT_STATUS, BL1_ALERT); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ALERT_STATUS, BL0_ALERT); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ALERT_DFI, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ALERT_DFI, MASK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_CA_PLAYBACK_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_CA_PLAYBACK_CONTROL, COUNT); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_CA_PLAYBACK_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_CA_PLAYBACK_CONTROL, SAMPLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_CA_PLAYBACK_STATUS0, VALID); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_CA_PLAYBACK_STATUS0, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_CA_PLAYBACK_STATUS0, DATA1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_CA_PLAYBACK_STATUS0, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_CA_PLAYBACK_STATUS0, DATA0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_CA_PLAYBACK_STATUS1, VALID); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_CA_PLAYBACK_STATUS1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_CA_PLAYBACK_STATUS1, DATA1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_CA_PLAYBACK_STATUS1, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_CA_PLAYBACK_STATUS1, DATA0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_WRITE_LEVELING_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_WRITE_LEVELING_CONTROL, COUNT); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_WRITE_LEVELING_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_WRITE_LEVELING_CONTROL, SAMPLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_WRITE_LEVELING_CONTROL, CONTINUOUS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_WRITE_LEVELING_CONTROL, ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_WRITE_LEVELING_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_WRITE_LEVELING_STATUS, STATUS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_WRITE_LEVELING_STATUS, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_WRITE_LEVELING_STATUS, VALID); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_READ_ENABLE_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_READ_ENABLE_CONTROL, TEST_CYCLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_READ_ENABLE_CONTROL, SELECT); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_READ_ENABLE_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_READ_ENABLE_CONTROL, CS_N); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_READ_ENABLE_CONTROL, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_READ_ENABLE_CONTROL, DQS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_READ_ENABLE_CONTROL, ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_READ_ENABLE_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_READ_ENABLE_STATUS, DATA); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_READ_ENABLE_STATUS, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_READ_ENABLE_STATUS, BL3_STATUS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_READ_ENABLE_STATUS, BL2_STATUS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_READ_ENABLE_STATUS, BL1_STATUS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_READ_ENABLE_STATUS, BL0_STATUS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_READ_ENABLE_STATUS, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_READ_ENABLE_STATUS, VALID); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROL, RESERVED_FOR_ECO1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROL, MAX_NOISE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROL, LOW_NOISE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROL, LOW_VTT); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROL, HIGH_VTT); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROL, ERROR_RESET); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROL, ENABLE_CTL_IDLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROL, ENABLE_CS_IDLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROL, ENABLE_CKE_IDLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VIRTUAL_VTT_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VIRTUAL_VTT_STATUS, ERROR); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VIRTUAL_VTT_STATUS, ERROR_LOW); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VIRTUAL_VTT_STATUS, ERROR_HIGH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VIRTUAL_VTT_STATUS, READY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VIRTUAL_VTT_CONNECTIONS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VIRTUAL_VTT_CONNECTIONS, MASK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VIRTUAL_VTT_OVERRIDE, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VIRTUAL_VTT_OVERRIDE, MASK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VREF_DAC_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VREF_DAC_CONTROL, AUX_GT_INT); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VREF_DAC_CONTROL, TESTOUT_MUX_CTL); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VREF_DAC_CONTROL, TEST); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VREF_DAC_CONTROL, PDN3); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VREF_DAC_CONTROL, PDN2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VREF_DAC_CONTROL, PDN1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VREF_DAC_CONTROL, PDN0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VREF_DAC_CONTROL, DAC1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_VREF_DAC_CONTROL, DAC0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_CNTRL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_CNTRL, PATTERN_SEL); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_CNTRL, FORCE_DQ_ERROR_SEL); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_CNTRL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_CNTRL, FORCE_BL_ERROR_SEL); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_CNTRL, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_CNTRL, FORCE_CTL_ERROR_SEL); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_CNTRL, RESERVED3); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_CNTRL, FORCE_DAT_ERROR); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_CNTRL, FORCE_CTL_ERROR); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_CNTRL, SSO); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_CNTRL, LENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_CNTRL, MODE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_CNTRL, ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_SEED, SEED); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_CA_MASK, MASK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_DQ_MASK1, MASK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_DQ_MASK2, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_DQ_MASK2, ALERT_MASK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_DQ_MASK2, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_DQ_MASK2, DM_MASK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_DQ_MASK2, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_STATUS, DAT_PASS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_STATUS, CTL_PASS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_STATUS, DAT_DONE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_STATUS, CTL_DONE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_CTL_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_CTL_STATUS, CTL_ERRORS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_BL0_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_BL0_STATUS, ALERT); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_BL0_STATUS, DM); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_BL0_STATUS, DQ); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_BL1_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_BL1_STATUS, ALERT); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_BL1_STATUS, DM); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_BL1_STATUS, DQ); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_BL2_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_BL2_STATUS, ALERT); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_BL2_STATUS, DM); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_BL2_STATUS, DQ); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_BL3_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_BL3_STATUS, ALERT); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_BL3_STATUS, DM); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_BL3_STATUS, DQ); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_BL4_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_BL4_STATUS, ALERT); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_BL4_STATUS, DM); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_PHYBIST_BL4_STATUS, DQ); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_STANDBY_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_STANDBY_CONTROL, PHY_STANDBY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_STANDBY_CONTROL, FORCE_CKE_RST_N); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_STANDBY_CONTROL, RST_N); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_STANDBY_CONTROL, CKE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_STANDBY_CONTROL, BIT_PIPE_RESET); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DEBUG_FREEZE_ENABLE, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DEBUG_FREEZE_ENABLE, WL1_BL1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DEBUG_FREEZE_ENABLE, WL1_BL0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DEBUG_FREEZE_ENABLE, WL0_BL1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DEBUG_FREEZE_ENABLE, WL0_BL0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DEBUG_MUX_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DEBUG_MUX_CONTROL, BYTE_SEL); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DEBUG_MUX_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DEBUG_MUX_CONTROL, PHASE_SEL); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DEBUG_MUX_CONTROL, SOURCE_SEL); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DFI_CNTRL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DFI_CNTRL, SELF_REFRESH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DFI_CNTRL, DFI_CS1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DFI_CNTRL, DFI_CS0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DFI_CNTRL, DFI_RST_N); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DFI_CNTRL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DFI_CNTRL, DFI_CKE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DFI_CNTRL, ACK_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DFI_CNTRL, ACK_STATUS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_DFI_CNTRL, ASSERT_REQ); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_WRITE_ODT_CNTRL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_WRITE_ODT_CNTRL, ODT1_FORCE_VALUE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_WRITE_ODT_CNTRL, ODT1_FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_WRITE_ODT_CNTRL, ODT1_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_WRITE_ODT_CNTRL, ODT1_LENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_WRITE_ODT_CNTRL, ODT1_DELAY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_WRITE_ODT_CNTRL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_WRITE_ODT_CNTRL, ODT0_FORCE_VALUE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_WRITE_ODT_CNTRL, ODT0_FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_WRITE_ODT_CNTRL, ODT0_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_WRITE_ODT_CNTRL, ODT0_LENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_WRITE_ODT_CNTRL, ODT0_DELAY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ABI_PAR_CNTRL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ABI_PAR_CNTRL, PAR_ERROR); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ABI_PAR_CNTRL, PAR_INCLUDE_AUX); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ABI_PAR_CNTRL, PAR_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ABI_PAR_CNTRL, ABI_INCLUDE_AUX); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ABI_PAR_CNTRL, ABI_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ZQ_CAL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ZQ_CAL, ZQ_PCOMP_STATUS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ZQ_CAL, ZQ_NCOMP_STATUS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ZQ_CAL, ZQ_IDDQ); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ZQ_CAL, ZQ_DRIVE_P); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ZQ_CAL, ZQ_DRIVE_N); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ZQ_CAL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ZQ_CAL, ZQ_PCOMP_ENB); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ZQ_CAL, ZQ_NCOMP_ENB); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ZQ_CAL2, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ZQ_CAL2, ZQ_CAL_RESET); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ZQ_CAL2, ZQ_CAL_ERROR_N); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ZQ_CAL2, ZQ_CAL_ERROR_P); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ZQ_CAL2, ZQ_NSETTING); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ZQ_CAL2, ZQ_PSETTING); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ZQ_CAL2, ZQ_CAL_OP_COMPLETE_N); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ZQ_CAL2, ZQ_CAL_OP_COMPLETE_P); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ZQ_CAL2, ZQ_CAL_START_N); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_ZQ_CAL2, ZQ_CAL_START_P); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_RO_PROC_MON_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_RO_PROC_MON_CTL, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_RO_PROC_MON_CTL, TRIGGER); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_RO_PROC_MON_CTL, RESET); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_RO_PROC_MON_CTL, SEL); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_RO_PROC_MON_CTL, EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_RO_PROC_MON_STATUS, CLK_COUNT); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_RO_PROC_MON_STATUS, RO_COUNT); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_CK_DRIVE_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_CK_DRIVE_PAD_CTL, ADDR_CTL_PD_IDLE_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_CK_DRIVE_PAD_CTL, ADDR_CTL_ND_IDLE_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_CK_DRIVE_PAD_CTL, ADDR_CTL_PD_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_CK_DRIVE_PAD_CTL, ADDR_CTL_ND_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_CK_DRIVE_PAD_CTL, ADDR_CTL_PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_CK_DRIVE_PAD_CTL, ADDR_CTL_ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_BYTE_LANE_ALIGN_MASKS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_BYTE_LANE_ALIGN_MASKS, MASK_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_BYTE_LANE_ALIGN_MASKS, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_BYTE_LANE_ALIGN_MASKS, BL3_MASK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_BYTE_LANE_ALIGN_MASKS, BL2_MASK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_BYTE_LANE_ALIGN_MASKS, BL1_MASK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_BYTE_LANE_ALIGN_MASKS, BL0_MASK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_CLOCK_IDLE, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_CLOCK_IDLE, BIT_CLOCK_CS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_CLOCK_IDLE, BIT_CLOCK_LS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_CLOCK_IDLE, TO_MC_CLOCK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_CLOCK_IDLE, FROM_MC_CLOCK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_CLOCK_IDLE, BIT_CLOCK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_CONTROL_REGS_AC_SPARE_REG, RESERVED_FOR_ECO0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ0, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ0, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ0, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ0, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ0, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ0, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ0, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ1, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ1, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ1, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ1, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ1, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ1, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ2, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ2, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ2, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ2, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ2, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ2, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ2, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ3, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ3, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ3, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ3, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ3, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ3, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ3, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ4, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ4, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ4, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ4, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ4, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ4, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ4, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ5, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ5, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ5, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ5, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ5, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ5, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ5, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ6, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ6, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ6, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ6, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ6, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ6, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ6, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ7, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ7, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ7, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ7, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ7, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ7, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ7, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DM, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DM, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DM, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DM, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DM, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DM, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_DM, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_ALERT, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_ALERT, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_ALERT, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_ALERT, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_ALERT, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_ALERT, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_WR_ALERT, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSP, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSP, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSP, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSP, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSP, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSP, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSP, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSN, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSN, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSN, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSN, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSN, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSN, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSN, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMP, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMP, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMP, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMP, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMP, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMP, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMP, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMN, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMN, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMN, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMN, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMN, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMN, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMN, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS0, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS0, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS0, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS0, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS0, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS0, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS0, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS1, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS1, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS1, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS1, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS1, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS1, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CLK_CONTROL, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CLK_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CLK_CONTROL, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CLK_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CLK_CONTROL, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CLK_CONTROL, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_CLK_CONTROL, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_LDE_CONTROL, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_LDE_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_LDE_CONTROL, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_LDE_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_LDE_CONTROL, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_LDE_CONTROL, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_VDL_LDE_CONTROL, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_RD_EN_DLY_CYC, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_RD_EN_DLY_CYC, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_RD_EN_DLY_CYC, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_RD_EN_DLY_CYC, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_RD_EN_DLY_CYC, ENABLE_CS1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_RD_EN_DLY_CYC, CS1_CYCLES); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_RD_EN_DLY_CYC, CS0_CYCLES); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_WR_CHAN_DLY_CYC, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_WR_CHAN_DLY_CYC, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_WR_CHAN_DLY_CYC, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_WR_CHAN_DLY_CYC, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_WR_CHAN_DLY_CYC, DQ_CYCLES); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_WR_CHAN_DLY_CYC, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_WR_CHAN_DLY_CYC, CYCLES); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_READ_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_READ_CONTROL, MODE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_READ_CONTROL, RD_EN_MODE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_READ_CONTROL, RD_DATA_DLY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_READ_FIFO_ADDR, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_READ_FIFO_ADDR, ADDR); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_READ_FIFO_DATA, DATA); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_READ_FIFO_DM_DBI, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_READ_FIFO_DM_DBI, DM_DBI); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_READ_FIFO_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_READ_FIFO_STATUS, OVERFLOW); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_READ_FIFO_STATUS, UNDERFLOW); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_READ_FIFO_CLEAR, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_READ_FIFO_CLEAR, CLEAR); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_IDLE_PAD_CONTROL, IDLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_IDLE_PAD_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_IDLE_PAD_CONTROL, AUTO_DQ_RXENB_MODE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_IDLE_PAD_CONTROL, AUTO_DQ_IDDQ_MODE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_IDLE_PAD_CONTROL, IO_IDLE_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_IDLE_PAD_CONTROL, RXENB); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_IDLE_PAD_CONTROL, IDDQ); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_IDLE_PAD_CONTROL, DOUT_N); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_IDLE_PAD_CONTROL, DOUT_P); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_DRIVE_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_DRIVE_PAD_CTL, BL_PD_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_DRIVE_PAD_CTL, BL_ND_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_DRIVE_PAD_CTL, BL_PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_DRIVE_PAD_CTL, BL_ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_DQSP_DRIVE_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_DQSP_DRIVE_PAD_CTL, BL_PD_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_DQSP_DRIVE_PAD_CTL, BL_ND_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_DQSP_DRIVE_PAD_CTL, BL_PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_DQSP_DRIVE_PAD_CTL, BL_ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_DQSN_DRIVE_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_DQSN_DRIVE_PAD_CTL, BL_PD_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_DQSN_DRIVE_PAD_CTL, BL_ND_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_DQSN_DRIVE_PAD_CTL, BL_PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_DQSN_DRIVE_PAD_CTL, BL_ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_ALERT_DRIVE_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_ALERT_DRIVE_PAD_CTL, BL_PD_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_ALERT_DRIVE_PAD_CTL, BL_ND_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_ALERT_DRIVE_PAD_CTL, BL_PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_ALERT_DRIVE_PAD_CTL, BL_ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_RD_EN_DRIVE_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_RD_EN_DRIVE_PAD_CTL, BL_RD_EN_PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_RD_EN_DRIVE_PAD_CTL, BL_RD_EN_ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_STATIC_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_STATIC_PAD_CTL, DM_INVERT); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_STATIC_PAD_CTL, WDBI_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_STATIC_PAD_CTL, RDBI_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_STATIC_PAD_CTL, DM_MODE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_STATIC_PAD_CTL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_STATIC_PAD_CTL, DQS_TX_DIS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_STATIC_PAD_CTL, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_STATIC_PAD_CTL, RX_MODE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_WR_PREAMBLE_MODE, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_WR_PREAMBLE_MODE, POSTAM_TERM_BITS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_WR_PREAMBLE_MODE, DM_POSTAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_WR_PREAMBLE_MODE, DM_PREAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_WR_PREAMBLE_MODE, DQ_POSTAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_WR_PREAMBLE_MODE, DQ_PREAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_WR_PREAMBLE_MODE, DQS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_WR_PREAMBLE_MODE, DQS_POSTAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_WR_PREAMBLE_MODE, DQS_PREAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_ODT_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_ODT_CONTROL, ODT_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_ODT_CONTROL, ODT_DELAY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_ODT_CONTROL, ODT_POST_LENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_ODT_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_ODT_CONTROL, ODT_PRE_LENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_CLOCK_ENABLE, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_CLOCK_ENABLE, TO_MC_CLOCK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_CLOCK_ENABLE, FROM_MC_CLOCK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_CLOCK_ENABLE, BIT_CLOCK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_CLOCK_IDLE, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_CLOCK_IDLE, TO_MC_CLOCK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_CLOCK_IDLE, FROM_MC_CLOCK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_CLOCK_IDLE, BIT_CLOCK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_0_BL_SPARE_REG, RESERVED_FOR_ECO0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ0, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ0, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ0, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ0, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ0, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ0, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ0, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ1, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ1, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ1, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ1, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ1, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ1, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ2, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ2, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ2, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ2, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ2, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ2, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ2, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ3, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ3, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ3, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ3, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ3, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ3, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ3, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ4, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ4, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ4, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ4, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ4, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ4, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ4, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ5, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ5, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ5, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ5, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ5, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ5, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ5, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ6, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ6, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ6, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ6, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ6, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ6, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ6, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ7, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ7, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ7, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ7, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ7, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ7, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ7, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DM, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DM, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DM, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DM, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DM, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DM, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_DM, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_ALERT, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_ALERT, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_ALERT, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_ALERT, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_ALERT, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_ALERT, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_WR_ALERT, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSP, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSP, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSP, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSP, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSP, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSP, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSP, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSN, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSN, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSN, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSN, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSN, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSN, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSN, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMP, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMP, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMP, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMP, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMP, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMP, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMP, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMN, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMN, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMN, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMN, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMN, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMN, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMN, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS0, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS0, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS0, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS0, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS0, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS0, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS0, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS1, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS1, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS1, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS1, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS1, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS1, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CLK_CONTROL, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CLK_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CLK_CONTROL, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CLK_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CLK_CONTROL, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CLK_CONTROL, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_CLK_CONTROL, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_LDE_CONTROL, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_LDE_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_LDE_CONTROL, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_LDE_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_LDE_CONTROL, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_LDE_CONTROL, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_VDL_LDE_CONTROL, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_RD_EN_DLY_CYC, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_RD_EN_DLY_CYC, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_RD_EN_DLY_CYC, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_RD_EN_DLY_CYC, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_RD_EN_DLY_CYC, ENABLE_CS1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_RD_EN_DLY_CYC, CS1_CYCLES); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_RD_EN_DLY_CYC, CS0_CYCLES); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_WR_CHAN_DLY_CYC, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_WR_CHAN_DLY_CYC, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_WR_CHAN_DLY_CYC, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_WR_CHAN_DLY_CYC, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_WR_CHAN_DLY_CYC, DQ_CYCLES); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_WR_CHAN_DLY_CYC, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_WR_CHAN_DLY_CYC, CYCLES); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_READ_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_READ_CONTROL, MODE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_READ_CONTROL, RD_EN_MODE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_READ_CONTROL, RD_DATA_DLY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_READ_FIFO_ADDR, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_READ_FIFO_ADDR, ADDR); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_READ_FIFO_DATA, DATA); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_READ_FIFO_DM_DBI, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_READ_FIFO_DM_DBI, DM_DBI); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_READ_FIFO_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_READ_FIFO_STATUS, OVERFLOW); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_READ_FIFO_STATUS, UNDERFLOW); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_READ_FIFO_CLEAR, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_READ_FIFO_CLEAR, CLEAR); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_IDLE_PAD_CONTROL, IDLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_IDLE_PAD_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_IDLE_PAD_CONTROL, AUTO_DQ_RXENB_MODE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_IDLE_PAD_CONTROL, AUTO_DQ_IDDQ_MODE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_IDLE_PAD_CONTROL, IO_IDLE_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_IDLE_PAD_CONTROL, RXENB); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_IDLE_PAD_CONTROL, IDDQ); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_IDLE_PAD_CONTROL, DOUT_N); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_IDLE_PAD_CONTROL, DOUT_P); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_DRIVE_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_DRIVE_PAD_CTL, BL_PD_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_DRIVE_PAD_CTL, BL_ND_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_DRIVE_PAD_CTL, BL_PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_DRIVE_PAD_CTL, BL_ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_DQSP_DRIVE_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_DQSP_DRIVE_PAD_CTL, BL_PD_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_DQSP_DRIVE_PAD_CTL, BL_ND_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_DQSP_DRIVE_PAD_CTL, BL_PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_DQSP_DRIVE_PAD_CTL, BL_ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_DQSN_DRIVE_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_DQSN_DRIVE_PAD_CTL, BL_PD_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_DQSN_DRIVE_PAD_CTL, BL_ND_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_DQSN_DRIVE_PAD_CTL, BL_PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_DQSN_DRIVE_PAD_CTL, BL_ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_ALERT_DRIVE_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_ALERT_DRIVE_PAD_CTL, BL_PD_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_ALERT_DRIVE_PAD_CTL, BL_ND_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_ALERT_DRIVE_PAD_CTL, BL_PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_ALERT_DRIVE_PAD_CTL, BL_ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_RD_EN_DRIVE_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_RD_EN_DRIVE_PAD_CTL, BL_RD_EN_PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_RD_EN_DRIVE_PAD_CTL, BL_RD_EN_ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_STATIC_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_STATIC_PAD_CTL, DM_INVERT); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_STATIC_PAD_CTL, WDBI_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_STATIC_PAD_CTL, RDBI_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_STATIC_PAD_CTL, DM_MODE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_STATIC_PAD_CTL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_STATIC_PAD_CTL, DQS_TX_DIS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_STATIC_PAD_CTL, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_STATIC_PAD_CTL, RX_MODE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_WR_PREAMBLE_MODE, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_WR_PREAMBLE_MODE, POSTAM_TERM_BITS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_WR_PREAMBLE_MODE, DM_POSTAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_WR_PREAMBLE_MODE, DM_PREAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_WR_PREAMBLE_MODE, DQ_POSTAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_WR_PREAMBLE_MODE, DQ_PREAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_WR_PREAMBLE_MODE, DQS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_WR_PREAMBLE_MODE, DQS_POSTAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_WR_PREAMBLE_MODE, DQS_PREAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_ODT_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_ODT_CONTROL, ODT_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_ODT_CONTROL, ODT_DELAY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_ODT_CONTROL, ODT_POST_LENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_ODT_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_ODT_CONTROL, ODT_PRE_LENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_CLOCK_ENABLE, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_CLOCK_ENABLE, TO_MC_CLOCK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_CLOCK_ENABLE, FROM_MC_CLOCK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_CLOCK_ENABLE, BIT_CLOCK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_CLOCK_IDLE, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_CLOCK_IDLE, TO_MC_CLOCK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_CLOCK_IDLE, FROM_MC_CLOCK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_CLOCK_IDLE, BIT_CLOCK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_1_BL_SPARE_REG, RESERVED_FOR_ECO0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ0, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ0, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ0, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ0, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ0, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ0, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ0, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ1, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ1, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ1, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ1, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ1, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ1, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ2, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ2, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ2, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ2, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ2, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ2, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ2, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ3, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ3, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ3, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ3, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ3, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ3, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ3, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ4, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ4, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ4, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ4, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ4, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ4, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ4, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ5, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ5, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ5, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ5, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ5, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ5, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ5, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ6, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ6, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ6, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ6, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ6, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ6, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ6, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ7, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ7, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ7, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ7, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ7, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ7, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ7, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DM, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DM, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DM, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DM, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DM, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DM, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_DM, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_ALERT, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_ALERT, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_ALERT, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_ALERT, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_ALERT, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_ALERT, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_WR_ALERT, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSP, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSP, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSP, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSP, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSP, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSP, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSP, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSN, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSN, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSN, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSN, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSN, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSN, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSN, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMP, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMP, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMP, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMP, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMP, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMP, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMP, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMN, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMN, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMN, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMN, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMN, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMN, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMN, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS0, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS0, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS0, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS0, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS0, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS0, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS0, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS1, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS1, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS1, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS1, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS1, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS1, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CLK_CONTROL, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CLK_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CLK_CONTROL, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CLK_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CLK_CONTROL, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CLK_CONTROL, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_CLK_CONTROL, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_LDE_CONTROL, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_LDE_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_LDE_CONTROL, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_LDE_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_LDE_CONTROL, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_LDE_CONTROL, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_VDL_LDE_CONTROL, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_RD_EN_DLY_CYC, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_RD_EN_DLY_CYC, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_RD_EN_DLY_CYC, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_RD_EN_DLY_CYC, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_RD_EN_DLY_CYC, ENABLE_CS1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_RD_EN_DLY_CYC, CS1_CYCLES); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_RD_EN_DLY_CYC, CS0_CYCLES); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_WR_CHAN_DLY_CYC, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_WR_CHAN_DLY_CYC, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_WR_CHAN_DLY_CYC, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_WR_CHAN_DLY_CYC, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_WR_CHAN_DLY_CYC, DQ_CYCLES); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_WR_CHAN_DLY_CYC, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_WR_CHAN_DLY_CYC, CYCLES); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_READ_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_READ_CONTROL, MODE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_READ_CONTROL, RD_EN_MODE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_READ_CONTROL, RD_DATA_DLY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_READ_FIFO_ADDR, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_READ_FIFO_ADDR, ADDR); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_READ_FIFO_DATA, DATA); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_READ_FIFO_DM_DBI, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_READ_FIFO_DM_DBI, DM_DBI); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_READ_FIFO_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_READ_FIFO_STATUS, OVERFLOW); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_READ_FIFO_STATUS, UNDERFLOW); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_READ_FIFO_CLEAR, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_READ_FIFO_CLEAR, CLEAR); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_IDLE_PAD_CONTROL, IDLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_IDLE_PAD_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_IDLE_PAD_CONTROL, AUTO_DQ_RXENB_MODE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_IDLE_PAD_CONTROL, AUTO_DQ_IDDQ_MODE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_IDLE_PAD_CONTROL, IO_IDLE_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_IDLE_PAD_CONTROL, RXENB); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_IDLE_PAD_CONTROL, IDDQ); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_IDLE_PAD_CONTROL, DOUT_N); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_IDLE_PAD_CONTROL, DOUT_P); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_DRIVE_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_DRIVE_PAD_CTL, BL_PD_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_DRIVE_PAD_CTL, BL_ND_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_DRIVE_PAD_CTL, BL_PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_DRIVE_PAD_CTL, BL_ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_DQSP_DRIVE_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_DQSP_DRIVE_PAD_CTL, BL_PD_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_DQSP_DRIVE_PAD_CTL, BL_ND_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_DQSP_DRIVE_PAD_CTL, BL_PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_DQSP_DRIVE_PAD_CTL, BL_ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_DQSN_DRIVE_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_DQSN_DRIVE_PAD_CTL, BL_PD_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_DQSN_DRIVE_PAD_CTL, BL_ND_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_DQSN_DRIVE_PAD_CTL, BL_PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_DQSN_DRIVE_PAD_CTL, BL_ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_ALERT_DRIVE_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_ALERT_DRIVE_PAD_CTL, BL_PD_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_ALERT_DRIVE_PAD_CTL, BL_ND_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_ALERT_DRIVE_PAD_CTL, BL_PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_ALERT_DRIVE_PAD_CTL, BL_ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_RD_EN_DRIVE_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_RD_EN_DRIVE_PAD_CTL, BL_RD_EN_PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_RD_EN_DRIVE_PAD_CTL, BL_RD_EN_ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_STATIC_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_STATIC_PAD_CTL, DM_INVERT); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_STATIC_PAD_CTL, WDBI_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_STATIC_PAD_CTL, RDBI_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_STATIC_PAD_CTL, DM_MODE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_STATIC_PAD_CTL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_STATIC_PAD_CTL, DQS_TX_DIS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_STATIC_PAD_CTL, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_STATIC_PAD_CTL, RX_MODE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_WR_PREAMBLE_MODE, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_WR_PREAMBLE_MODE, POSTAM_TERM_BITS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_WR_PREAMBLE_MODE, DM_POSTAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_WR_PREAMBLE_MODE, DM_PREAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_WR_PREAMBLE_MODE, DQ_POSTAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_WR_PREAMBLE_MODE, DQ_PREAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_WR_PREAMBLE_MODE, DQS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_WR_PREAMBLE_MODE, DQS_POSTAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_WR_PREAMBLE_MODE, DQS_PREAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_ODT_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_ODT_CONTROL, ODT_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_ODT_CONTROL, ODT_DELAY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_ODT_CONTROL, ODT_POST_LENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_ODT_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_ODT_CONTROL, ODT_PRE_LENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_CLOCK_ENABLE, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_CLOCK_ENABLE, TO_MC_CLOCK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_CLOCK_ENABLE, FROM_MC_CLOCK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_CLOCK_ENABLE, BIT_CLOCK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_CLOCK_IDLE, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_CLOCK_IDLE, TO_MC_CLOCK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_CLOCK_IDLE, FROM_MC_CLOCK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_CLOCK_IDLE, BIT_CLOCK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_2_BL_SPARE_REG, RESERVED_FOR_ECO0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ0, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ0, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ0, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ0, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ0, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ0, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ0, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ1, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ1, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ1, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ1, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ1, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ1, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ2, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ2, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ2, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ2, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ2, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ2, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ2, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ3, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ3, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ3, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ3, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ3, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ3, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ3, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ4, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ4, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ4, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ4, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ4, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ4, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ4, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ5, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ5, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ5, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ5, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ5, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ5, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ5, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ6, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ6, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ6, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ6, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ6, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ6, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ6, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ7, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ7, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ7, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ7, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ7, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ7, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ7, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DM, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DM, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DM, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DM, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DM, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DM, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_DM, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_ALERT, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_ALERT, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_ALERT, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_ALERT, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_ALERT, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_ALERT, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_WR_ALERT, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSP, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSP, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSP, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSP, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSP, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSP, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSP, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSN, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSN, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSN, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSN, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSN, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSN, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSN, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7P, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7P, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7P, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7P, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7P, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7P, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7P, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7N, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7N, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7N, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7N, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7N, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7N, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7N, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMP, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMP, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMP, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMP, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMP, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMP, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMP, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMN, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMN, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMN, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMN, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMN, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMN, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMN, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS0, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS0, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS0, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS0, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS0, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS0, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS0, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS1, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS1, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS1, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS1, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS1, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS1, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS1, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CLK_CONTROL, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CLK_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CLK_CONTROL, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CLK_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CLK_CONTROL, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CLK_CONTROL, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_CLK_CONTROL, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_LDE_CONTROL, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_LDE_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_LDE_CONTROL, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_LDE_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_LDE_CONTROL, ADJ_EN); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_LDE_CONTROL, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_VDL_LDE_CONTROL, VDL_STEP); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_RD_EN_DLY_CYC, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_RD_EN_DLY_CYC, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_RD_EN_DLY_CYC, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_RD_EN_DLY_CYC, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_RD_EN_DLY_CYC, ENABLE_CS1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_RD_EN_DLY_CYC, CS1_CYCLES); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_RD_EN_DLY_CYC, CS0_CYCLES); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_WR_CHAN_DLY_CYC, BUSY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_WR_CHAN_DLY_CYC, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_WR_CHAN_DLY_CYC, FORCE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_WR_CHAN_DLY_CYC, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_WR_CHAN_DLY_CYC, DQ_CYCLES); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_WR_CHAN_DLY_CYC, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_WR_CHAN_DLY_CYC, CYCLES); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_READ_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_READ_CONTROL, MODE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_READ_CONTROL, RD_EN_MODE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_READ_CONTROL, RD_DATA_DLY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_READ_FIFO_ADDR, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_READ_FIFO_ADDR, ADDR); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_READ_FIFO_DATA, DATA); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_READ_FIFO_DM_DBI, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_READ_FIFO_DM_DBI, DM_DBI); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_READ_FIFO_STATUS, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_READ_FIFO_STATUS, OVERFLOW); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_READ_FIFO_STATUS, UNDERFLOW); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_READ_FIFO_CLEAR, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_READ_FIFO_CLEAR, CLEAR); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_IDLE_PAD_CONTROL, IDLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_IDLE_PAD_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_IDLE_PAD_CONTROL, AUTO_DQ_RXENB_MODE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_IDLE_PAD_CONTROL, AUTO_DQ_IDDQ_MODE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_IDLE_PAD_CONTROL, IO_IDLE_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_IDLE_PAD_CONTROL, RXENB); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_IDLE_PAD_CONTROL, IDDQ); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_IDLE_PAD_CONTROL, DOUT_N); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_IDLE_PAD_CONTROL, DOUT_P); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_DRIVE_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_DRIVE_PAD_CTL, BL_PD_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_DRIVE_PAD_CTL, BL_ND_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_DRIVE_PAD_CTL, BL_PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_DRIVE_PAD_CTL, BL_ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_DQSP_DRIVE_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_DQSP_DRIVE_PAD_CTL, BL_PD_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_DQSP_DRIVE_PAD_CTL, BL_ND_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_DQSP_DRIVE_PAD_CTL, BL_PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_DQSP_DRIVE_PAD_CTL, BL_ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_DQSN_DRIVE_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_DQSN_DRIVE_PAD_CTL, BL_PD_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_DQSN_DRIVE_PAD_CTL, BL_ND_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_DQSN_DRIVE_PAD_CTL, BL_PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_DQSN_DRIVE_PAD_CTL, BL_ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_ALERT_DRIVE_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_ALERT_DRIVE_PAD_CTL, BL_PD_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_ALERT_DRIVE_PAD_CTL, BL_ND_TERM_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_ALERT_DRIVE_PAD_CTL, BL_PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_ALERT_DRIVE_PAD_CTL, BL_ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_RD_EN_DRIVE_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_RD_EN_DRIVE_PAD_CTL, BL_RD_EN_PD_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_RD_EN_DRIVE_PAD_CTL, BL_RD_EN_ND_STRENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_STATIC_PAD_CTL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_STATIC_PAD_CTL, DM_INVERT); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_STATIC_PAD_CTL, WDBI_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_STATIC_PAD_CTL, RDBI_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_STATIC_PAD_CTL, DM_MODE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_STATIC_PAD_CTL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_STATIC_PAD_CTL, DQS_TX_DIS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_STATIC_PAD_CTL, RESERVED2); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_STATIC_PAD_CTL, RX_MODE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_WR_PREAMBLE_MODE, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_WR_PREAMBLE_MODE, POSTAM_TERM_BITS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_WR_PREAMBLE_MODE, DM_POSTAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_WR_PREAMBLE_MODE, DM_PREAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_WR_PREAMBLE_MODE, DQ_POSTAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_WR_PREAMBLE_MODE, DQ_PREAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_WR_PREAMBLE_MODE, DQS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_WR_PREAMBLE_MODE, DQS_POSTAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_WR_PREAMBLE_MODE, DQS_PREAM_BITS); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_ODT_CONTROL, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_ODT_CONTROL, ODT_ENABLE); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_ODT_CONTROL, ODT_DELAY); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_ODT_CONTROL, ODT_POST_LENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_ODT_CONTROL, RESERVED1); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_ODT_CONTROL, ODT_PRE_LENGTH); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_CLOCK_ENABLE, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_CLOCK_ENABLE, TO_MC_CLOCK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_CLOCK_ENABLE, FROM_MC_CLOCK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_CLOCK_ENABLE, BIT_CLOCK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_CLOCK_IDLE, RESERVED0); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_CLOCK_IDLE, TO_MC_CLOCK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_CLOCK_IDLE, FROM_MC_CLOCK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_CLOCK_IDLE, BIT_CLOCK); \
    INIT_DDR34_SIGNE_FIELD(F0, PHY_BYTE_LANE_3_BL_SPARE_REG, RESERVED_FOR_ECO0); \
}

/*******************************************************************************
 *
 ******************************************************************************/
/***************************************************************************
 *DDR34_E0_PHY_CONTROL_REGS - DDR34 CORE DDR34 Address/Comand control registers
 ***************************************************************************/
#define READ_DDR34_PHY_CONTROL_REGS_REVISIONr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_REVISIONr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_REVISIONr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_REVISIONr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_REVISIONr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_REVISIONr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_PLL_STATUSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_STATUSr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_PLL_STATUSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_STATUSr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_PLL_STATUSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_STATUSr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_PLL_CONFIGr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_CONFIGr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_PLL_CONFIGr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_CONFIGr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_PLL_CONFIGr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_CONFIGr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_PLL_CONTROL1r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_CONTROL1r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_PLL_CONTROL1r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_CONTROL1r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_PLL_CONTROL1r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_CONTROL1r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_PLL_CONTROL2r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_CONTROL2r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_PLL_CONTROL2r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_CONTROL2r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_PLL_CONTROL2r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_CONTROL2r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_PLL_CONTROL3r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_CONTROL3r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_PLL_CONTROL3r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_CONTROL3r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_PLL_CONTROL3r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_CONTROL3r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_PLL_DIVIDERSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_DIVIDERSr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_PLL_DIVIDERSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_DIVIDERSr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_PLL_DIVIDERSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_DIVIDERSr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_PLL_FRAC_DIVIDERr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_FRAC_DIVIDERr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_PLL_FRAC_DIVIDERr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_FRAC_DIVIDERr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_PLL_FRAC_DIVIDERr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_FRAC_DIVIDERr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_PLL_SS_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_SS_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_PLL_SS_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_SS_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_PLL_SS_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_SS_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_PLL_SS_LIMITr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_SS_LIMITr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_PLL_SS_LIMITr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_SS_LIMITr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_PLL_SS_LIMITr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PLL_SS_LIMITr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_AUX_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_AUX_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_AUX_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_AUX_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_AUX_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_AUX_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_IDLE_PAD_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_IDLE_PAD_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_IDLE_PAD_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_IDLE_PAD_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_IDLE_PAD_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_IDLE_PAD_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_IDLE_PAD_ENABLE0r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_IDLE_PAD_ENABLE0r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_IDLE_PAD_ENABLE0r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_IDLE_PAD_ENABLE0r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_IDLE_PAD_ENABLE0r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_IDLE_PAD_ENABLE0r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_IDLE_PAD_ENABLE1r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_IDLE_PAD_ENABLE1r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_IDLE_PAD_ENABLE1r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_IDLE_PAD_ENABLE1r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_IDLE_PAD_ENABLE1r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_IDLE_PAD_ENABLE1r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTLr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTLr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTLr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_STATIC_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_STATIC_PAD_CTLr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_STATIC_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_STATIC_PAD_CTLr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_STATIC_PAD_CTLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_STATIC_PAD_CTLr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_DRAM_CONFIGr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DRAM_CONFIGr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_DRAM_CONFIGr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DRAM_CONFIGr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_DRAM_CONFIGr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DRAM_CONFIGr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_DRAM_TIMING1r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DRAM_TIMING1r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_DRAM_TIMING1r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DRAM_TIMING1r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_DRAM_TIMING1r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DRAM_TIMING1r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_DRAM_TIMING2r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DRAM_TIMING2r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_DRAM_TIMING2r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DRAM_TIMING2r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_DRAM_TIMING2r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DRAM_TIMING2r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_DRAM_TIMING3r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DRAM_TIMING3r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_DRAM_TIMING3r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DRAM_TIMING3r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_DRAM_TIMING3r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DRAM_TIMING3r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_DRAM_TIMING4r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DRAM_TIMING4r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_DRAM_TIMING4r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DRAM_TIMING4r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_DRAM_TIMING4r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DRAM_TIMING4r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CALIBRATEr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CALIBRATEr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CALIBRATEr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CALIBRATEr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CALIBRATEr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CALIBRATEr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CALIB_STATUS1r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CALIB_STATUS1r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CALIB_STATUS1r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CALIB_STATUS1r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CALIB_STATUS1r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CALIB_STATUS1r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CALIB_STATUS2r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CALIB_STATUS2r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CALIB_STATUS2r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CALIB_STATUS2r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CALIB_STATUS2r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CALIB_STATUS2r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_MONITOR_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_MONITOR_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_MONITOR_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_MONITOR_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_MONITOR_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_MONITOR_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_MONITOR_REFr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_MONITOR_REFr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_MONITOR_REFr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_MONITOR_REFr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_MONITOR_REFr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_MONITOR_REFr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_MONITOR_STATUSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_MONITOR_STATUSr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_MONITOR_STATUSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_MONITOR_STATUSr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_MONITOR_STATUSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_MONITOR_STATUSr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OVERRIDEr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OVERRIDEr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OVERRIDEr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OVERRIDEr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OVERRIDEr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OVERRIDEr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUSr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUSr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUSr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUS_CLEARr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUS_CLEARr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUS_CLEARr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUS_CLEARr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUS_CLEARr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_MONITOR_OUT_STATUS_CLEARr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD00r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD00r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD00r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD00r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD00r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD00r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD01r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD01r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD01r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD01r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD01r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD01r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD02r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD02r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD02r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD02r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD02r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD02r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD03r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD03r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD03r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD03r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD03r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD03r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD04r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD04r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD04r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD04r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD04r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD04r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD05r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD05r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD05r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD05r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD05r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD05r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD06r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD06r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD06r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD06r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD06r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD06r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD07r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD07r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD07r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD07r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD07r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD07r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD08r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD08r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD08r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD08r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD08r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD08r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD09r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD09r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD09r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD09r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD09r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD09r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD10r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD10r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD10r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD10r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD10r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD10r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD11r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD11r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD11r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD11r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD11r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD11r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD12r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD12r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD12r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD12r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD12r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD12r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD13r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD13r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD13r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD13r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD13r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD13r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD14r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD14r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD14r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD14r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD14r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD14r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD15r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD15r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD15r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD15r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD15r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AD15r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA0r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA0r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA0r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA0r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA0r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA0r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA1r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA1r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA1r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA1r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA1r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA1r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA2r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA2r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA2r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA2r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA2r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_BA2r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX0r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX0r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX0r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX0r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX0r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX0r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX1r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX1r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX1r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX1r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX1r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX1r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX2r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX2r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX2r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX2r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX2r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_AUX2r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CS0r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CS0r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CS0r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CS0r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CS0r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CS0r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CS1r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CS1r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CS1r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CS1r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CS1r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CS1r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_PARr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_PARr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_PARr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_PARr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_PARr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_PARr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_RAS_Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_RAS_Nr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_RAS_Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_RAS_Nr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_RAS_Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_RAS_Nr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CAS_Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CAS_Nr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CAS_Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CAS_Nr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CAS_Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CAS_Nr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CKEr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CKEr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CKEr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CKEr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CKEr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_CKEr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_RST_Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_RST_Nr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_RST_Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_RST_Nr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_RST_Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_RST_Nr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_ODTr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_ODTr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_ODTr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_ODTr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_ODTr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_ODTr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_WE_Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_WE_Nr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_WE_Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_WE_Nr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_WE_Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_WE_Nr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_Pr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_Pr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_Pr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_Nr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_Nr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CK_Nr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CLK_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CLK_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CLK_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CLK_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CLK_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CLK_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_LDE_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_LDE_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_LDE_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_LDE_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_LDE_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_LDE_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_REFRESHr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_REFRESHr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_REFRESHr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_REFRESHr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_REFRESHr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_REFRESHr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_UPDATE_VDLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_UPDATE_VDLr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_UPDATE_VDLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_UPDATE_VDLr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_UPDATE_VDLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_UPDATE_VDLr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP1r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP1r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP1r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP1r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP1r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP1r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP2r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP2r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP2r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP2r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP2r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_UPDATE_VDL_SNOOP2r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_COMMAND_REG1r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_REG1r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_COMMAND_REG1r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_REG1r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_COMMAND_REG1r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_REG1r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG1r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG1r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG1r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG1r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG1r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG1r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_COMMAND_REG2r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_REG2r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_COMMAND_REG2r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_REG2r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_COMMAND_REG2r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_REG2r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG2r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG2r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG2r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG2r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG2r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG2r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_COMMAND_REG3r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_REG3r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_COMMAND_REG3r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_REG3r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_COMMAND_REG3r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_REG3r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG3r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG3r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG3r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG3r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG3r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG3r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_COMMAND_REG4r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_REG4r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_COMMAND_REG4r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_REG4r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_COMMAND_REG4r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_REG4r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG4r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG4r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG4r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG4r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG4r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_AUX_REG4r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_COMMAND_REG_TIMERr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_REG_TIMERr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_COMMAND_REG_TIMERr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_REG_TIMERr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_COMMAND_REG_TIMERr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_COMMAND_REG_TIMERr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_MODE_REG0r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG0r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_MODE_REG0r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG0r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_MODE_REG0r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG0r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_MODE_REG1r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG1r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_MODE_REG1r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG1r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_MODE_REG1r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG1r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_MODE_REG2r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG2r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_MODE_REG2r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG2r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_MODE_REG2r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG2r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_MODE_REG3r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG3r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_MODE_REG3r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG3r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_MODE_REG3r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG3r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_MODE_REG4r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG4r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_MODE_REG4r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG4r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_MODE_REG4r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG4r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_MODE_REG5r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG5r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_MODE_REG5r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG5r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_MODE_REG5r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG5r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_MODE_REG6r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG6r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_MODE_REG6r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG6r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_MODE_REG6r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG6r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_MODE_REG7r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG7r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_MODE_REG7r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG7r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_MODE_REG7r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG7r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_MODE_REG8r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG8r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_MODE_REG8r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG8r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_MODE_REG8r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG8r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_MODE_REG15r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG15r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_MODE_REG15r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG15r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_MODE_REG15r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG15r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_MODE_REG63r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG63r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_MODE_REG63r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG63r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_MODE_REG63r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_MODE_REG63r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_ALERT_CLEARr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_ALERT_CLEARr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_ALERT_CLEARr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_ALERT_CLEARr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_ALERT_CLEARr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_ALERT_CLEARr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_ALERT_STATUSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_ALERT_STATUSr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_ALERT_STATUSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_ALERT_STATUSr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_ALERT_STATUSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_ALERT_STATUSr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_CA_PARITYr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_CA_PARITYr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_CA_PARITYr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_CA_PARITYr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_CA_PARITYr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_CA_PARITYr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_STATUS0r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_STATUS0r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_STATUS0r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_STATUS0r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_STATUS0r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_STATUS0r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_STATUS1r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_STATUS1r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_STATUS1r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_STATUS1r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_STATUS1r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_CA_PLAYBACK_STATUS1r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_WRITE_LEVELING_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_WRITE_LEVELING_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_WRITE_LEVELING_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_WRITE_LEVELING_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_WRITE_LEVELING_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_WRITE_LEVELING_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_WRITE_LEVELING_STATUSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_WRITE_LEVELING_STATUSr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_WRITE_LEVELING_STATUSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_WRITE_LEVELING_STATUSr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_WRITE_LEVELING_STATUSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_WRITE_LEVELING_STATUSr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_READ_ENABLE_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_READ_ENABLE_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_READ_ENABLE_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_READ_ENABLE_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_READ_ENABLE_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_READ_ENABLE_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_READ_ENABLE_STATUSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_READ_ENABLE_STATUSr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_READ_ENABLE_STATUSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_READ_ENABLE_STATUSr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_READ_ENABLE_STATUSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_READ_ENABLE_STATUSr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_LFSR_SEEDr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_LFSR_SEEDr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_LFSR_SEEDr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_LFSR_SEEDr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_LFSR_SEEDr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_LFSR_SEEDr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ADDRESS1r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ADDRESS1r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ADDRESS1r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ADDRESS1r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ADDRESS1r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ADDRESS1r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ADDRESS2r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ADDRESS2r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ADDRESS2r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ADDRESS2r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ADDRESS2r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ADDRESS2r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_DATA_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_DATA_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_DATA_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_DATA_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_DATA_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_DATA_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_DQ_MASKr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_DQ_MASKr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_DQ_MASKr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_DQ_MASKr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_DQ_MASKr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_DQ_MASKr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ECC_DQ_MASKr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ECC_DQ_MASKr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ECC_DQ_MASKr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ECC_DQ_MASKr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ECC_DQ_MASKr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ECC_DQ_MASKr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_STATUSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_STATUSr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_STATUSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_STATUSr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_STATUSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_STATUSr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_DQ_STATUSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_DQ_STATUSr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_DQ_STATUSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_DQ_STATUSr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_DQ_STATUSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_DQ_STATUSr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ECC_STATUSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ECC_STATUSr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ECC_STATUSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ECC_STATUSr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ECC_STATUSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ECC_STATUSr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ERR_CNT_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ERR_CNT_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ERR_CNT_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ERR_CNT_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ERR_CNT_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ERR_CNT_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ERR_CNT_STATUSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ERR_CNT_STATUSr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ERR_CNT_STATUSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ERR_CNT_STATUSr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ERR_CNT_STATUSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_TRAFFIC_GEN_ERR_CNT_STATUSr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_STATUSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_STATUSr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_STATUSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_STATUSr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_STATUSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_STATUSr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_CONNECTIONSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_CONNECTIONSr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_CONNECTIONSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_CONNECTIONSr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_CONNECTIONSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_CONNECTIONSr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_OVERRIDEr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_OVERRIDEr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_OVERRIDEr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_OVERRIDEr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_OVERRIDEr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VIRTUAL_VTT_OVERRIDEr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VREF_DAC_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VREF_DAC_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VREF_DAC_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VREF_DAC_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VREF_DAC_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VREF_DAC_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_PHYBIST_CNTRLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_CNTRLr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_PHYBIST_CNTRLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_CNTRLr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_PHYBIST_CNTRLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_CNTRLr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_PHYBIST_SEEDr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_SEEDr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_PHYBIST_SEEDr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_SEEDr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_PHYBIST_SEEDr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_SEEDr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_PHYBIST_STATUSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_STATUSr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_PHYBIST_STATUSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_STATUSr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_PHYBIST_STATUSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_STATUSr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_PHYBIST_CTL_STATUSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_CTL_STATUSr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_PHYBIST_CTL_STATUSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_CTL_STATUSr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_PHYBIST_CTL_STATUSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_CTL_STATUSr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_PHYBIST_BL0_STATUSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_BL0_STATUSr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_PHYBIST_BL0_STATUSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_BL0_STATUSr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_PHYBIST_BL0_STATUSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_BL0_STATUSr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_PHYBIST_BL1_STATUSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_BL1_STATUSr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_PHYBIST_BL1_STATUSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_BL1_STATUSr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_PHYBIST_BL1_STATUSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_BL1_STATUSr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_PHYBIST_BL2_STATUSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_BL2_STATUSr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_PHYBIST_BL2_STATUSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_BL2_STATUSr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_PHYBIST_BL2_STATUSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_BL2_STATUSr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_PHYBIST_BL3_STATUSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_BL3_STATUSr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_PHYBIST_BL3_STATUSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_BL3_STATUSr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_PHYBIST_BL3_STATUSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_BL3_STATUSr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_PHYBIST_BL4_STATUSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_BL4_STATUSr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_PHYBIST_BL4_STATUSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_BL4_STATUSr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_PHYBIST_BL4_STATUSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_BL4_STATUSr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_STANDBY_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_STANDBY_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_STANDBY_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_STANDBY_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_STANDBY_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_STANDBY_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_DEBUG_FREEZE_ENABLEr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DEBUG_FREEZE_ENABLEr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_DEBUG_FREEZE_ENABLEr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DEBUG_FREEZE_ENABLEr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_DEBUG_FREEZE_ENABLEr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DEBUG_FREEZE_ENABLEr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_DEBUG_MUX_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DEBUG_MUX_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_DEBUG_MUX_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DEBUG_MUX_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_DEBUG_MUX_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DEBUG_MUX_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_DFI_CNTRLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DFI_CNTRLr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_DFI_CNTRLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DFI_CNTRLr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_DFI_CNTRLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DFI_CNTRLr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_WRITE_ODT_CNTRLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_WRITE_ODT_CNTRLr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_WRITE_ODT_CNTRLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_WRITE_ODT_CNTRLr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_WRITE_ODT_CNTRLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_WRITE_ODT_CNTRLr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_ABI_PAR_CNTRLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_ABI_PAR_CNTRLr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_ABI_PAR_CNTRLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_ABI_PAR_CNTRLr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_ABI_PAR_CNTRLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_ABI_PAR_CNTRLr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_ZQ_CALr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_ZQ_CALr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_ZQ_CALr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_ZQ_CALr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_ZQ_CALr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_ZQ_CALr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_AC_SPARE_REGr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_AC_SPARE_REGr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_AC_SPARE_REGr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_AC_SPARE_REGr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_AC_SPARE_REGr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_AC_SPARE_REGr), (_val), (_mask))


/***************************************************************************
 *DDR34_E0_PHY_BYTE_LANE_0 - DDR34 CORE DDR34 Byte Lane #0 control registers
 ***************************************************************************/
#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_Pr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_Pr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_Pr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_Nr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_Nr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQS_Nr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ0r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ0r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ0r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ0r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ0r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ0r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ1r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ1r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ1r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ1r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ1r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ1r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ2r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ2r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ2r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ2r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ2r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ2r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ3r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ3r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ3r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ3r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ3r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ3r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ4r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ4r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ4r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ4r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ4r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ4r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ5r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ5r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ5r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ5r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ5r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ5r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ6r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ6r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ6r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ6r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ6r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ6r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ7r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ7r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ7r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ7r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ7r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQ7r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DMr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DMr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DMr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DMr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DMr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DMr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_EDCr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_EDCr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_EDCr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_EDCr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_EDCr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_EDCr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSPr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSPr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSPr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSPr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSPr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSPr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSNr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSNr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSNr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSNr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSNr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQSNr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0Pr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0Pr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0Pr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0Nr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0Nr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ0Nr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1Pr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1Pr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1Pr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1Nr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1Nr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ1Nr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2Pr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2Pr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2Pr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2Nr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2Nr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ2Nr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3Pr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3Pr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3Pr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3Nr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3Nr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ3Nr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4Pr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4Pr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4Pr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4Nr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4Nr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ4Nr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5Pr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5Pr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5Pr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5Nr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5Nr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ5Nr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6Pr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6Pr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6Pr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6Nr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6Nr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ6Nr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7Pr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7Pr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7Pr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7Nr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7Nr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DQ7Nr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMPr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMPr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMPr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMPr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMPr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMPr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMNr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMNr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMNr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMNr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMNr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_DMNr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCPr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCPr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCPr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCPr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCPr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCPr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCNr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCNr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCNr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCNr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCNr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EDCNr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS0r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS0r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS0r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS0r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS0r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS0r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS1r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS1r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS1r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS1r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS1r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_RD_EN_CS1r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CLK_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CLK_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CLK_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CLK_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CLK_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CLK_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_LDE_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_LDE_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_LDE_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_LDE_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_LDE_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_LDE_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_RD_EN_DLY_CYCr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_RD_EN_DLY_CYCr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_RD_EN_DLY_CYCr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_RD_EN_DLY_CYCr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_RD_EN_DLY_CYCr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_RD_EN_DLY_CYCr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_WR_CHAN_DLY_CYCr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_WR_CHAN_DLY_CYCr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_WR_CHAN_DLY_CYCr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_WR_CHAN_DLY_CYCr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_WR_CHAN_DLY_CYCr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_WR_CHAN_DLY_CYCr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_READ_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_READ_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_READ_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_READ_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_READ_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_READ_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_READ_FIFO_ADDRr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_READ_FIFO_ADDRr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_READ_FIFO_ADDRr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_READ_FIFO_ADDRr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_READ_FIFO_ADDRr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_READ_FIFO_ADDRr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_READ_FIFO_DATAr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_READ_FIFO_DATAr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_READ_FIFO_DATAr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_READ_FIFO_DATAr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_READ_FIFO_DATAr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_READ_FIFO_DATAr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_READ_FIFO_DM_DBIr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_READ_FIFO_DM_DBIr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_READ_FIFO_DM_DBIr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_READ_FIFO_DM_DBIr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_READ_FIFO_DM_DBIr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_READ_FIFO_DM_DBIr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_READ_FIFO_STATUSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_READ_FIFO_STATUSr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_READ_FIFO_STATUSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_READ_FIFO_STATUSr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_READ_FIFO_STATUSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_READ_FIFO_STATUSr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_READ_FIFO_CLEARr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_READ_FIFO_CLEARr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_READ_FIFO_CLEARr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_READ_FIFO_CLEARr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_READ_FIFO_CLEARr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_READ_FIFO_CLEARr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_IDLE_PAD_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_IDLE_PAD_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_IDLE_PAD_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_IDLE_PAD_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_IDLE_PAD_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_IDLE_PAD_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_DRIVE_PAD_CTLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_DRIVE_PAD_CTLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_DRIVE_PAD_CTLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_DRIVE_PAD_CTLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_RD_EN_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_RD_EN_DRIVE_PAD_CTLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_RD_EN_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_RD_EN_DRIVE_PAD_CTLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_RD_EN_DRIVE_PAD_CTLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_RD_EN_DRIVE_PAD_CTLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_STATIC_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_STATIC_PAD_CTLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_STATIC_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_STATIC_PAD_CTLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_STATIC_PAD_CTLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_STATIC_PAD_CTLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_WR_PREAMBLE_MODEr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_WR_PREAMBLE_MODEr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_WR_PREAMBLE_MODEr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_WR_PREAMBLE_MODEr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_WR_PREAMBLE_MODEr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_WR_PREAMBLE_MODEr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_ODT_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_ODT_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_ODT_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_ODT_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_ODT_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_ODT_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_EDC_DPD_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_EDC_DPD_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_EDC_DPD_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_EDC_DPD_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_EDC_DPD_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_EDC_DPD_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_EDC_DPD_STATUSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_EDC_DPD_STATUSr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_EDC_DPD_STATUSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_EDC_DPD_STATUSr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_EDC_DPD_STATUSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_EDC_DPD_STATUSr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_STATUSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_STATUSr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_STATUSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_STATUSr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_STATUSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_STATUSr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_STATUS_CLEARr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_STATUS_CLEARr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_STATUS_CLEARr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_STATUS_CLEARr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_STATUS_CLEARr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_EDC_DPD_OUT_STATUS_CLEARr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_EDC_CRC_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_EDC_CRC_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_EDC_CRC_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_EDC_CRC_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_EDC_CRC_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_EDC_CRC_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_EDC_CRC_STATUSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_EDC_CRC_STATUSr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_EDC_CRC_STATUSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_EDC_CRC_STATUSr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_EDC_CRC_STATUSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_EDC_CRC_STATUSr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_EDC_CRC_COUNTr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_EDC_CRC_COUNTr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_EDC_CRC_COUNTr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_EDC_CRC_COUNTr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_EDC_CRC_COUNTr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_EDC_CRC_COUNTr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_EDC_CRC_STATUS_CLEARr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_EDC_CRC_STATUS_CLEARr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_EDC_CRC_STATUS_CLEARr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_EDC_CRC_STATUS_CLEARr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_EDC_CRC_STATUS_CLEARr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_EDC_CRC_STATUS_CLEARr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_BL_SPARE_REGr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_BL_SPARE_REGr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_BL_SPARE_REGr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_BL_SPARE_REGr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_BL_SPARE_REGr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_BL_SPARE_REGr), (_val), (_mask))


/***************************************************************************
 *DDR34_E0_PHY_BYTE_LANE_1 - DDR34 CORE DDR34 Byte Lane #1 control registers
 ***************************************************************************/
#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_Pr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_Pr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_Pr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_Nr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_Nr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQS_Nr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ0r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ0r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ0r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ0r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ0r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ0r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ1r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ1r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ1r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ1r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ1r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ1r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ2r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ2r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ2r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ2r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ2r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ2r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ3r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ3r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ3r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ3r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ3r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ3r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ4r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ4r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ4r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ4r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ4r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ4r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ5r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ5r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ5r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ5r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ5r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ5r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ6r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ6r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ6r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ6r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ6r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ6r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ7r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ7r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ7r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ7r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ7r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQ7r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DMr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DMr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DMr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DMr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DMr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DMr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_EDCr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_EDCr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_EDCr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_EDCr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_EDCr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_EDCr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSPr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSPr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSPr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSPr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSPr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSPr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSNr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSNr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSNr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSNr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSNr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQSNr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0Pr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0Pr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0Pr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0Nr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0Nr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ0Nr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1Pr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1Pr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1Pr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1Nr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1Nr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ1Nr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2Pr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2Pr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2Pr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2Nr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2Nr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ2Nr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3Pr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3Pr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3Pr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3Nr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3Nr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ3Nr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4Pr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4Pr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4Pr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4Nr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4Nr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ4Nr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5Pr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5Pr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5Pr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5Nr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5Nr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ5Nr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6Pr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6Pr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6Pr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6Nr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6Nr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ6Nr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7Pr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7Pr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7Pr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7Nr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7Nr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DQ7Nr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMPr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMPr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMPr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMPr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMPr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMPr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMNr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMNr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMNr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMNr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMNr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_DMNr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCPr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCPr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCPr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCPr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCPr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCPr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCNr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCNr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCNr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCNr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCNr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EDCNr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS0r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS0r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS0r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS0r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS0r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS0r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS1r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS1r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS1r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS1r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS1r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_RD_EN_CS1r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CLK_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CLK_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CLK_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CLK_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CLK_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CLK_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_LDE_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_LDE_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_LDE_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_LDE_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_LDE_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_LDE_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_RD_EN_DLY_CYCr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_RD_EN_DLY_CYCr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_RD_EN_DLY_CYCr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_RD_EN_DLY_CYCr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_RD_EN_DLY_CYCr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_RD_EN_DLY_CYCr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_WR_CHAN_DLY_CYCr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_WR_CHAN_DLY_CYCr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_WR_CHAN_DLY_CYCr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_WR_CHAN_DLY_CYCr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_WR_CHAN_DLY_CYCr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_WR_CHAN_DLY_CYCr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_READ_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_READ_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_READ_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_READ_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_READ_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_READ_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_READ_FIFO_ADDRr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_READ_FIFO_ADDRr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_READ_FIFO_ADDRr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_READ_FIFO_ADDRr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_READ_FIFO_ADDRr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_READ_FIFO_ADDRr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_READ_FIFO_DATAr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_READ_FIFO_DATAr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_READ_FIFO_DATAr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_READ_FIFO_DATAr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_READ_FIFO_DATAr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_READ_FIFO_DATAr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_READ_FIFO_DM_DBIr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_READ_FIFO_DM_DBIr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_READ_FIFO_DM_DBIr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_READ_FIFO_DM_DBIr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_READ_FIFO_DM_DBIr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_READ_FIFO_DM_DBIr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_READ_FIFO_STATUSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_READ_FIFO_STATUSr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_READ_FIFO_STATUSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_READ_FIFO_STATUSr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_READ_FIFO_STATUSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_READ_FIFO_STATUSr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_READ_FIFO_CLEARr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_READ_FIFO_CLEARr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_READ_FIFO_CLEARr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_READ_FIFO_CLEARr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_READ_FIFO_CLEARr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_READ_FIFO_CLEARr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_IDLE_PAD_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_IDLE_PAD_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_IDLE_PAD_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_IDLE_PAD_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_IDLE_PAD_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_IDLE_PAD_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_DRIVE_PAD_CTLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_DRIVE_PAD_CTLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_DRIVE_PAD_CTLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_DRIVE_PAD_CTLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_RD_EN_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_RD_EN_DRIVE_PAD_CTLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_RD_EN_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_RD_EN_DRIVE_PAD_CTLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_RD_EN_DRIVE_PAD_CTLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_RD_EN_DRIVE_PAD_CTLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_STATIC_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_STATIC_PAD_CTLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_STATIC_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_STATIC_PAD_CTLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_STATIC_PAD_CTLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_STATIC_PAD_CTLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_WR_PREAMBLE_MODEr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_WR_PREAMBLE_MODEr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_WR_PREAMBLE_MODEr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_WR_PREAMBLE_MODEr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_WR_PREAMBLE_MODEr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_WR_PREAMBLE_MODEr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_ODT_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_ODT_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_ODT_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_ODT_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_ODT_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_ODT_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_EDC_DPD_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_EDC_DPD_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_EDC_DPD_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_EDC_DPD_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_EDC_DPD_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_EDC_DPD_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_EDC_DPD_STATUSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_EDC_DPD_STATUSr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_EDC_DPD_STATUSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_EDC_DPD_STATUSr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_EDC_DPD_STATUSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_EDC_DPD_STATUSr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_STATUSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_STATUSr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_STATUSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_STATUSr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_STATUSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_STATUSr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_STATUS_CLEARr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_STATUS_CLEARr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_STATUS_CLEARr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_STATUS_CLEARr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_STATUS_CLEARr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_EDC_DPD_OUT_STATUS_CLEARr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_EDC_CRC_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_EDC_CRC_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_EDC_CRC_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_EDC_CRC_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_EDC_CRC_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_EDC_CRC_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_EDC_CRC_STATUSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_EDC_CRC_STATUSr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_EDC_CRC_STATUSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_EDC_CRC_STATUSr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_EDC_CRC_STATUSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_EDC_CRC_STATUSr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_EDC_CRC_COUNTr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_EDC_CRC_COUNTr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_EDC_CRC_COUNTr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_EDC_CRC_COUNTr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_EDC_CRC_COUNTr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_EDC_CRC_COUNTr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_EDC_CRC_STATUS_CLEARr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_EDC_CRC_STATUS_CLEARr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_EDC_CRC_STATUS_CLEARr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_EDC_CRC_STATUS_CLEARr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_EDC_CRC_STATUS_CLEARr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_EDC_CRC_STATUS_CLEARr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_BL_SPARE_REGr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_BL_SPARE_REGr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_BL_SPARE_REGr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_BL_SPARE_REGr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_BL_SPARE_REGr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_BL_SPARE_REGr), (_val), (_mask))


/***************************************************************************
 *DDR34_E0_PHY_BYTE_LANE_2 - DDR34 CORE DDR34 Byte Lane #2 control registers
 ***************************************************************************/
#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_Pr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_Pr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_Pr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_Nr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_Nr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQS_Nr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ0r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ0r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ0r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ0r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ0r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ0r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ1r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ1r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ1r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ1r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ1r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ1r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ2r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ2r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ2r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ2r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ2r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ2r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ3r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ3r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ3r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ3r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ3r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ3r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ4r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ4r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ4r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ4r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ4r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ4r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ5r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ5r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ5r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ5r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ5r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ5r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ6r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ6r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ6r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ6r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ6r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ6r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ7r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ7r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ7r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ7r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ7r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQ7r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DMr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DMr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DMr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DMr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DMr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DMr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_EDCr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_EDCr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_EDCr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_EDCr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_EDCr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_EDCr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSPr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSPr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSPr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSPr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSPr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSPr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSNr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSNr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSNr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSNr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSNr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQSNr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0Pr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0Pr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0Pr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0Nr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0Nr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ0Nr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1Pr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1Pr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1Pr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1Nr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1Nr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ1Nr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2Pr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2Pr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2Pr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2Nr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2Nr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ2Nr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3Pr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3Pr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3Pr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3Nr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3Nr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ3Nr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4Pr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4Pr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4Pr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4Nr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4Nr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ4Nr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5Pr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5Pr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5Pr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5Nr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5Nr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ5Nr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6Pr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6Pr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6Pr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6Nr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6Nr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ6Nr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7Pr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7Pr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7Pr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7Nr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7Nr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DQ7Nr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMPr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMPr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMPr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMPr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMPr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMPr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMNr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMNr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMNr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMNr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMNr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_DMNr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCPr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCPr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCPr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCPr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCPr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCPr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCNr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCNr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCNr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCNr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCNr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EDCNr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS0r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS0r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS0r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS0r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS0r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS0r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS1r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS1r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS1r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS1r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS1r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_RD_EN_CS1r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CLK_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CLK_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CLK_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CLK_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CLK_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CLK_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_LDE_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_LDE_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_LDE_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_LDE_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_LDE_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_LDE_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_RD_EN_DLY_CYCr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_RD_EN_DLY_CYCr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_RD_EN_DLY_CYCr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_RD_EN_DLY_CYCr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_RD_EN_DLY_CYCr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_RD_EN_DLY_CYCr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_WR_CHAN_DLY_CYCr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_WR_CHAN_DLY_CYCr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_WR_CHAN_DLY_CYCr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_WR_CHAN_DLY_CYCr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_WR_CHAN_DLY_CYCr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_WR_CHAN_DLY_CYCr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_READ_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_READ_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_READ_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_READ_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_READ_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_READ_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_READ_FIFO_ADDRr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_READ_FIFO_ADDRr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_READ_FIFO_ADDRr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_READ_FIFO_ADDRr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_READ_FIFO_ADDRr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_READ_FIFO_ADDRr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_READ_FIFO_DATAr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_READ_FIFO_DATAr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_READ_FIFO_DATAr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_READ_FIFO_DATAr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_READ_FIFO_DATAr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_READ_FIFO_DATAr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_READ_FIFO_DM_DBIr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_READ_FIFO_DM_DBIr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_READ_FIFO_DM_DBIr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_READ_FIFO_DM_DBIr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_READ_FIFO_DM_DBIr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_READ_FIFO_DM_DBIr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_READ_FIFO_STATUSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_READ_FIFO_STATUSr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_READ_FIFO_STATUSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_READ_FIFO_STATUSr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_READ_FIFO_STATUSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_READ_FIFO_STATUSr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_READ_FIFO_CLEARr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_READ_FIFO_CLEARr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_READ_FIFO_CLEARr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_READ_FIFO_CLEARr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_READ_FIFO_CLEARr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_READ_FIFO_CLEARr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_IDLE_PAD_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_IDLE_PAD_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_IDLE_PAD_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_IDLE_PAD_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_IDLE_PAD_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_IDLE_PAD_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_DRIVE_PAD_CTLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_DRIVE_PAD_CTLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_DRIVE_PAD_CTLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_DRIVE_PAD_CTLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_RD_EN_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_RD_EN_DRIVE_PAD_CTLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_RD_EN_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_RD_EN_DRIVE_PAD_CTLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_RD_EN_DRIVE_PAD_CTLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_RD_EN_DRIVE_PAD_CTLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_STATIC_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_STATIC_PAD_CTLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_STATIC_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_STATIC_PAD_CTLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_STATIC_PAD_CTLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_STATIC_PAD_CTLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_WR_PREAMBLE_MODEr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_WR_PREAMBLE_MODEr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_WR_PREAMBLE_MODEr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_WR_PREAMBLE_MODEr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_WR_PREAMBLE_MODEr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_WR_PREAMBLE_MODEr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_ODT_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_ODT_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_ODT_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_ODT_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_ODT_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_ODT_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_EDC_DPD_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_EDC_DPD_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_EDC_DPD_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_EDC_DPD_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_EDC_DPD_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_EDC_DPD_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_EDC_DPD_STATUSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_EDC_DPD_STATUSr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_EDC_DPD_STATUSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_EDC_DPD_STATUSr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_EDC_DPD_STATUSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_EDC_DPD_STATUSr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_STATUSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_STATUSr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_STATUSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_STATUSr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_STATUSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_STATUSr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_STATUS_CLEARr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_STATUS_CLEARr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_STATUS_CLEARr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_STATUS_CLEARr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_STATUS_CLEARr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_EDC_DPD_OUT_STATUS_CLEARr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_EDC_CRC_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_EDC_CRC_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_EDC_CRC_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_EDC_CRC_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_EDC_CRC_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_EDC_CRC_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_EDC_CRC_STATUSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_EDC_CRC_STATUSr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_EDC_CRC_STATUSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_EDC_CRC_STATUSr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_EDC_CRC_STATUSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_EDC_CRC_STATUSr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_EDC_CRC_COUNTr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_EDC_CRC_COUNTr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_EDC_CRC_COUNTr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_EDC_CRC_COUNTr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_EDC_CRC_COUNTr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_EDC_CRC_COUNTr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_EDC_CRC_STATUS_CLEARr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_EDC_CRC_STATUS_CLEARr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_EDC_CRC_STATUS_CLEARr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_EDC_CRC_STATUS_CLEARr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_EDC_CRC_STATUS_CLEARr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_EDC_CRC_STATUS_CLEARr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_BL_SPARE_REGr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_BL_SPARE_REGr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_BL_SPARE_REGr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_BL_SPARE_REGr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_BL_SPARE_REGr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_BL_SPARE_REGr), (_val), (_mask))


/***************************************************************************
 *DDR34_E0_PHY_BYTE_LANE_3 - DDR34 CORE DDR34 Byte Lane #3 control registers
 ***************************************************************************/
#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_Pr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_Pr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_Pr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_Nr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_Nr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQS_Nr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ0r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ0r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ0r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ0r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ0r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ0r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ1r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ1r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ1r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ1r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ1r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ1r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ2r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ2r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ2r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ2r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ2r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ2r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ3r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ3r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ3r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ3r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ3r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ3r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ4r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ4r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ4r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ4r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ4r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ4r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ5r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ5r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ5r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ5r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ5r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ5r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ6r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ6r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ6r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ6r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ6r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ6r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ7r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ7r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ7r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ7r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ7r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQ7r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DMr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DMr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DMr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DMr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DMr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DMr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_EDCr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_EDCr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_EDCr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_EDCr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_EDCr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_EDCr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSPr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSPr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSPr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSPr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSPr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSPr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSNr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSNr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSNr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSNr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSNr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQSNr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0Pr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0Pr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0Pr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0Nr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0Nr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ0Nr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1Pr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1Pr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1Pr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1Nr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1Nr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ1Nr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2Pr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2Pr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2Pr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2Nr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2Nr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ2Nr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3Pr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3Pr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3Pr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3Nr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3Nr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ3Nr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4Pr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4Pr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4Pr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4Nr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4Nr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ4Nr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5Pr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5Pr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5Pr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5Nr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5Nr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ5Nr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6Pr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6Pr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6Pr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6Nr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6Nr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ6Nr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7Pr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7Pr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7Pr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7Pr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7Pr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7Pr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7Nr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7Nr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7Nr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7Nr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7Nr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DQ7Nr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMPr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMPr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMPr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMPr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMPr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMPr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMNr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMNr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMNr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMNr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMNr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_DMNr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCPr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCPr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCPr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCPr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCPr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCPr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCNr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCNr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCNr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCNr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCNr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EDCNr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS0r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS0r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS0r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS0r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS0r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS0r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS1r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS1r), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS1r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS1r), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS1r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_RD_EN_CS1r), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CLK_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CLK_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CLK_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CLK_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CLK_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CLK_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_LDE_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_LDE_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_LDE_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_LDE_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_LDE_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_LDE_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_RD_EN_DLY_CYCr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_RD_EN_DLY_CYCr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_RD_EN_DLY_CYCr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_RD_EN_DLY_CYCr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_RD_EN_DLY_CYCr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_RD_EN_DLY_CYCr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_WR_CHAN_DLY_CYCr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_WR_CHAN_DLY_CYCr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_WR_CHAN_DLY_CYCr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_WR_CHAN_DLY_CYCr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_WR_CHAN_DLY_CYCr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_WR_CHAN_DLY_CYCr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_READ_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_READ_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_READ_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_READ_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_READ_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_READ_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_READ_FIFO_ADDRr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_READ_FIFO_ADDRr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_READ_FIFO_ADDRr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_READ_FIFO_ADDRr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_READ_FIFO_ADDRr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_READ_FIFO_ADDRr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_READ_FIFO_DATAr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_READ_FIFO_DATAr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_READ_FIFO_DATAr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_READ_FIFO_DATAr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_READ_FIFO_DATAr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_READ_FIFO_DATAr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_READ_FIFO_DM_DBIr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_READ_FIFO_DM_DBIr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_READ_FIFO_DM_DBIr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_READ_FIFO_DM_DBIr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_READ_FIFO_DM_DBIr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_READ_FIFO_DM_DBIr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_READ_FIFO_STATUSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_READ_FIFO_STATUSr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_READ_FIFO_STATUSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_READ_FIFO_STATUSr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_READ_FIFO_STATUSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_READ_FIFO_STATUSr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_READ_FIFO_CLEARr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_READ_FIFO_CLEARr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_READ_FIFO_CLEARr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_READ_FIFO_CLEARr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_READ_FIFO_CLEARr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_READ_FIFO_CLEARr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_IDLE_PAD_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_IDLE_PAD_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_IDLE_PAD_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_IDLE_PAD_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_IDLE_PAD_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_IDLE_PAD_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_DRIVE_PAD_CTLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_DRIVE_PAD_CTLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_DRIVE_PAD_CTLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_DRIVE_PAD_CTLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_RD_EN_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_RD_EN_DRIVE_PAD_CTLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_RD_EN_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_RD_EN_DRIVE_PAD_CTLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_RD_EN_DRIVE_PAD_CTLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_RD_EN_DRIVE_PAD_CTLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_STATIC_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_STATIC_PAD_CTLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_STATIC_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_STATIC_PAD_CTLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_STATIC_PAD_CTLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_STATIC_PAD_CTLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_WR_PREAMBLE_MODEr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_WR_PREAMBLE_MODEr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_WR_PREAMBLE_MODEr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_WR_PREAMBLE_MODEr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_WR_PREAMBLE_MODEr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_WR_PREAMBLE_MODEr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_ODT_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_ODT_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_ODT_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_ODT_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_ODT_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_ODT_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_EDC_DPD_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_EDC_DPD_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_EDC_DPD_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_EDC_DPD_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_EDC_DPD_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_EDC_DPD_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_EDC_DPD_STATUSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_EDC_DPD_STATUSr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_EDC_DPD_STATUSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_EDC_DPD_STATUSr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_EDC_DPD_STATUSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_EDC_DPD_STATUSr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_STATUSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_STATUSr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_STATUSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_STATUSr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_STATUSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_STATUSr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_STATUS_CLEARr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_STATUS_CLEARr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_STATUS_CLEARr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_STATUS_CLEARr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_STATUS_CLEARr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_EDC_DPD_OUT_STATUS_CLEARr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_EDC_CRC_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_EDC_CRC_CONTROLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_EDC_CRC_CONTROLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_EDC_CRC_CONTROLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_EDC_CRC_CONTROLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_EDC_CRC_CONTROLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_EDC_CRC_STATUSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_EDC_CRC_STATUSr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_EDC_CRC_STATUSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_EDC_CRC_STATUSr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_EDC_CRC_STATUSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_EDC_CRC_STATUSr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_EDC_CRC_COUNTr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_EDC_CRC_COUNTr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_EDC_CRC_COUNTr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_EDC_CRC_COUNTr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_EDC_CRC_COUNTr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_EDC_CRC_COUNTr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_EDC_CRC_STATUS_CLEARr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_EDC_CRC_STATUS_CLEARr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_EDC_CRC_STATUS_CLEARr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_EDC_CRC_STATUS_CLEARr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_EDC_CRC_STATUS_CLEARr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_EDC_CRC_STATUS_CLEARr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_BL_SPARE_REGr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_BL_SPARE_REGr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_BL_SPARE_REGr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_BL_SPARE_REGr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_BL_SPARE_REGr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_BL_SPARE_REGr), (_val), (_mask))


/****************************************************************************
*****************************************************************************
****************************************************************************/
/****************************************************************************
 * DDR34_F0_PHY_CONTROL_REGS
 ***************************************************************************/
#define READ_DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTL_2Tr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTL_2Tr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTL_2Tr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTL_2Tr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTL_2Tr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_DRIVE_PAD_CTL_2Tr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CKr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CKr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CKr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CKr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CKr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_VDL_CONTROL_DDR_CKr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_CMD_CYC_DLYr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_CMD_CYC_DLYr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_CMD_CYC_DLYr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_CMD_CYC_DLYr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_CMD_CYC_DLYr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_CMD_CYC_DLYr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_ALERT_DFIr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_ALERT_DFIr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_ALERT_DFIr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_ALERT_DFIr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_ALERT_DFIr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_ALERT_DFIr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_PHYBIST_CA_MASKr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_CA_MASKr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_PHYBIST_CA_MASKr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_CA_MASKr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_PHYBIST_CA_MASKr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_CA_MASKr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_PHYBIST_DQ_MASK1r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_DQ_MASK1r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_PHYBIST_DQ_MASK1r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_DQ_MASK1r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_PHYBIST_DQ_MASK1r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_DQ_MASK1r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_PHYBIST_DQ_MASK2r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_DQ_MASK2r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_PHYBIST_DQ_MASK2r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_DQ_MASK2r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_PHYBIST_DQ_MASK2r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_PHYBIST_DQ_MASK2r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_ZQ_CAL2r(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_ZQ_CAL2r), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_ZQ_CAL2r(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_ZQ_CAL2r), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_ZQ_CAL2r(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_ZQ_CAL2r), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_RO_PROC_MON_CTLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_RO_PROC_MON_CTLr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_RO_PROC_MON_CTLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_RO_PROC_MON_CTLr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_RO_PROC_MON_CTLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_RO_PROC_MON_CTLr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_RO_PROC_MON_STATUSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_RO_PROC_MON_STATUSr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_RO_PROC_MON_STATUSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_RO_PROC_MON_STATUSr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_RO_PROC_MON_STATUSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_RO_PROC_MON_STATUSr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_CK_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_CK_DRIVE_PAD_CTLr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_CK_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_CK_DRIVE_PAD_CTLr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_CK_DRIVE_PAD_CTLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_CK_DRIVE_PAD_CTLr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_BYTE_LANE_ALIGN_MASKSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_BYTE_LANE_ALIGN_MASKSr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_BYTE_LANE_ALIGN_MASKSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_BYTE_LANE_ALIGN_MASKSr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_BYTE_LANE_ALIGN_MASKSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_BYTE_LANE_ALIGN_MASKSr), (_val), (_mask))

#define READ_DDR34_PHY_CONTROL_REGS_CLOCK_IDLEr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_CLOCK_IDLEr), (_val)) 
#define WRITE_DDR34_PHY_CONTROL_REGS_CLOCK_IDLEr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_CLOCK_IDLEr), (_val))
#define MODIFY_DDR34_PHY_CONTROL_REGS_CLOCK_IDLEr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_CONTROL_REGS_CLOCK_IDLEr), (_val), (_mask))


/****************************************************************************
 * DDR34_F0_PHY_BYTE_LANE_0
 ***************************************************************************/
#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQSr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQSr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_DQSr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_ALERTr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_ALERTr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_ALERTr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_ALERTr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_ALERTr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_VDL_CONTROL_WR_ALERTr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_DQSP_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_DQSP_DRIVE_PAD_CTLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_DQSP_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_DQSP_DRIVE_PAD_CTLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_DQSP_DRIVE_PAD_CTLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_DQSP_DRIVE_PAD_CTLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_DQSN_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_DQSN_DRIVE_PAD_CTLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_DQSN_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_DQSN_DRIVE_PAD_CTLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_DQSN_DRIVE_PAD_CTLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_DQSN_DRIVE_PAD_CTLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_ALERT_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_ALERT_DRIVE_PAD_CTLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_ALERT_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_ALERT_DRIVE_PAD_CTLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_ALERT_DRIVE_PAD_CTLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_ALERT_DRIVE_PAD_CTLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_CLOCK_ENABLEr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_CLOCK_ENABLEr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_CLOCK_ENABLEr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_CLOCK_ENABLEr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_CLOCK_ENABLEr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_CLOCK_ENABLEr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_0_CLOCK_IDLEr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_CLOCK_IDLEr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_0_CLOCK_IDLEr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_CLOCK_IDLEr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_0_CLOCK_IDLEr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_0_CLOCK_IDLEr), (_val), (_mask))


/****************************************************************************
 * DDR34_F0_PHY_BYTE_LANE_1
 ***************************************************************************/
#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQSr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQSr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_DQSr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_ALERTr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_ALERTr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_ALERTr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_ALERTr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_ALERTr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_VDL_CONTROL_WR_ALERTr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_DQSP_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_DQSP_DRIVE_PAD_CTLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_DQSP_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_DQSP_DRIVE_PAD_CTLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_DQSP_DRIVE_PAD_CTLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_DQSP_DRIVE_PAD_CTLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_DQSN_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_DQSN_DRIVE_PAD_CTLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_DQSN_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_DQSN_DRIVE_PAD_CTLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_DQSN_DRIVE_PAD_CTLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_DQSN_DRIVE_PAD_CTLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_ALERT_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_ALERT_DRIVE_PAD_CTLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_ALERT_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_ALERT_DRIVE_PAD_CTLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_ALERT_DRIVE_PAD_CTLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_ALERT_DRIVE_PAD_CTLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_CLOCK_ENABLEr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_CLOCK_ENABLEr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_CLOCK_ENABLEr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_CLOCK_ENABLEr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_CLOCK_ENABLEr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_CLOCK_ENABLEr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_1_CLOCK_IDLEr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_CLOCK_IDLEr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_1_CLOCK_IDLEr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_CLOCK_IDLEr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_1_CLOCK_IDLEr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_1_CLOCK_IDLEr), (_val), (_mask))


/****************************************************************************
 * DDR34_F0_PHY_BYTE_LANE_2
 ***************************************************************************/
#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQSr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQSr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_DQSr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_ALERTr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_ALERTr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_ALERTr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_ALERTr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_ALERTr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_VDL_CONTROL_WR_ALERTr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_DQSP_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_DQSP_DRIVE_PAD_CTLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_DQSP_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_DQSP_DRIVE_PAD_CTLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_DQSP_DRIVE_PAD_CTLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_DQSP_DRIVE_PAD_CTLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_DQSN_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_DQSN_DRIVE_PAD_CTLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_DQSN_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_DQSN_DRIVE_PAD_CTLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_DQSN_DRIVE_PAD_CTLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_DQSN_DRIVE_PAD_CTLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_ALERT_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_ALERT_DRIVE_PAD_CTLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_ALERT_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_ALERT_DRIVE_PAD_CTLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_ALERT_DRIVE_PAD_CTLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_ALERT_DRIVE_PAD_CTLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_CLOCK_ENABLEr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_CLOCK_ENABLEr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_CLOCK_ENABLEr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_CLOCK_ENABLEr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_CLOCK_ENABLEr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_CLOCK_ENABLEr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_2_CLOCK_IDLEr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_CLOCK_IDLEr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_2_CLOCK_IDLEr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_CLOCK_IDLEr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_2_CLOCK_IDLEr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_2_CLOCK_IDLEr), (_val), (_mask))


/****************************************************************************
 * DDR34_F0_PHY_BYTE_LANE_3
 ***************************************************************************/
#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQSr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQSr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQSr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQSr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQSr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_DQSr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_ALERTr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_ALERTr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_ALERTr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_ALERTr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_ALERTr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_VDL_CONTROL_WR_ALERTr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_DQSP_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_DQSP_DRIVE_PAD_CTLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_DQSP_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_DQSP_DRIVE_PAD_CTLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_DQSP_DRIVE_PAD_CTLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_DQSP_DRIVE_PAD_CTLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_DQSN_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_DQSN_DRIVE_PAD_CTLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_DQSN_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_DQSN_DRIVE_PAD_CTLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_DQSN_DRIVE_PAD_CTLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_DQSN_DRIVE_PAD_CTLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_ALERT_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_ALERT_DRIVE_PAD_CTLr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_ALERT_DRIVE_PAD_CTLr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_ALERT_DRIVE_PAD_CTLr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_ALERT_DRIVE_PAD_CTLr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_ALERT_DRIVE_PAD_CTLr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_CLOCK_ENABLEr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_CLOCK_ENABLEr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_CLOCK_ENABLEr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_CLOCK_ENABLEr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_CLOCK_ENABLEr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_CLOCK_ENABLEr), (_val), (_mask))

#define READ_DDR34_PHY_BYTE_LANE_3_CLOCK_IDLEr(_unit, _pc, _val) \
        DDR34_REG_READ((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_CLOCK_IDLEr), (_val)) 
#define WRITE_DDR34_PHY_BYTE_LANE_3_CLOCK_IDLEr(_unit, _pc, _val) \
        DDR34_REG_WRITE((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_CLOCK_IDLEr), (_val))
#define MODIFY_DDR34_PHY_BYTE_LANE_3_CLOCK_IDLEr(_unit, _pc, _val, _mask) \
        DDR34_REG_MODIFY((_unit), (_pc), 0x00, DDR34_PHY_REGS_TABLE(DDR34_PHY_BYTE_LANE_3_CLOCK_IDLEr), (_val), (_mask))


#endif /* DDR34_H__ */

